// -----------------------------------------------------------------------------
//  AppDelegate.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe delegata di applicazione

#import 	"AppDelegate.h"

#import		"PrefWinCtrl.h"
#import		"AboutWinCtrl.h"
#import		"InfoWinCtrl.h"
#import		"CoverPalWinCtl.h"
#import		"ElemParamWinCtl.h"
#import		"UnitsSizeGridWinCtl.h"

@implementation	AppDelegate

static AppDelegate		* applicationGlobals  = nil ;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
init
{
	if (  applicationGlobals )
		return ( applicationGlobals );
	// se arrivo qui devo costruire l'istanza condivisa
	self = [ super init ] ;
	// l' istanza condivisa e' proprio questa
	applicationGlobals = self ;
	// sottoscrivo alcune notifiche
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(mainWindowChanged:)
		name:NSWindowDidBecomeMainNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(mainWindowResigned:)
		name:NSWindowDidResignMainNotification object:nil];
	// osservatore sul cambiamento della selezione corrente
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( outViewSelChange: )
		name: NSOutlineViewSelectionDidChangeNotification object: nil ] ;
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(cvrViewSelChange:)
		name:@"CoverViewChangedSelection" object:nil];
	return (self );	// restituisco l'istanza
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
mainWindowChanged: (NSNotification *)notification
{
	NSWindowController *controller = [[notification object] windowController];
	if (controller && [controller isKindOfClass:[CoverWinCtl class]])
	{
		[ self updateListWinInfo: nil ] ;
		[ self updateCoverViewInfo: [(CoverWinCtl *)controller cvrView] ] ;
	}
	else if (controller && [controller isKindOfClass:[ListWinCtl class]])
	{
		[ self updateListWinInfo: [((ListWinCtl *)controller) fullList ] ] ;
		[ self updateCoverViewInfo: nil ] ;
	}
	else
	{
		[ self updateListWinInfo: nil ] ;
		[ self updateCoverViewInfo: nil ] ;
	};
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
mainWindowResigned: (NSNotification *)notification
{
	[ self updateListWinInfo: nil ] ;
	[ self updateCoverViewInfo: nil ];
}

// ---------------------------------------------------------------------
// metodo eseguito quando cambia la selezione all'interno di una outlineView
- (void )
outViewSelChange: ( NSNotification *) notification
{
	NSOutlineView   * outV = (NSOutlineView*) [ notification object ];
	if ( [ outV tag] == TAG_CDCATDOC_FULLLIST )
		[ self updateListWinInfo: [ notification object ] ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo eseguito quando cambia la selezione all'interno di una coverView
- (void)
cvrViewSelChange: (NSNotification *)notification
{
	id notifObj = [notification object] ;
	if ( [notifObj isKindOfClass:[CoverView class]])
			[ self updateCoverViewInfo: (CoverView*) notifObj ];
	else	[ self updateCoverViewInfo: nil ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// conseguenze al cambio di selezione in una coverview
- (void)
updateCoverViewInfo: (CoverView *) coverView
{
	ElemParamWinCtl  *tempWC1 = (ElemParamWinCtl*)
			[ ElemParamWinCtl sharedElemParamWinCtl ];
	UnitsSizeGridWinCtl	*tempWC3 = (UnitsSizeGridWinCtl*)
		[ UnitsSizeGridWinCtl sharedGridDialogWinCtl ];

		// aggiorno la coverview in primo piano
	theDrawView = coverView ;
	if ( theDrawView == nil )
	{
		// se non c'e' la coverview, non ci sono elementi selezionati
		theSelectedElem = nil ;
	}
	else
	{
		NSMutableArray  * selArr ;
		int				numElem ;
		// guardo quanti elementi sono correntemente selezionati
		selArr = [ [ theDrawView theElements] getSelectedGraphics ];
		numElem = [ selArr count ];
		// se non ce ne sono...
		if ( numElem == 0 )
		{
			// dico che se ne fa nulla
			theSelectedElem = nil ;
		}
		// ... o ce ne sono troppi
		else if ( numElem > 1 )
		{
			// dico che se ne fa nulla
			theSelectedElem = (CCE_BasicForm*)(-1) ;
		}
		else
		{
			// c'e' un solo elemento, imposto il puntatore
			theSelectedElem = [ selArr objectAtIndex: 0 ] ;
		}
	}
	// impongo l'aggiornamento alle varie finestre accessorie
	[ tempWC1 setSelectedElem: theSelectedElem ] ;
	[ tempWC3 updateWindow: theDrawView ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// conseguenze al cambio di selezione in una listview
- (void)
updateListWinInfo: (NSOutlineView *) outView
{
	InfoWinCtrl  *tempWC1 = (InfoWinCtrl*) [ InfoWinCtrl sharedInfoWinCtrl ];
	theOutlineView = outView ;
	// impongo l'aggiornamento alle varie finestre accessorie
	[ tempWC1 updateInfo: theOutlineView ] ;
}

// ---------------------------------------------------------------------
- (NSOutlineView *)theOutlineView {
	return theOutlineView ;
}

- (void)setTheOutlineView:(NSOutlineView *)newTheOutlineView {
	theOutlineView = newTheOutlineView ;
}

- (CoverView *)theDrawView {
	return theDrawView;
}

- (void)setTheDrawView:(CoverView *)newTheDrawView {
	theDrawView = newTheDrawView ;
}

- (CCE_BasicForm *)theSelectedElem {
	return theSelectedElem ;
}

- (void)setTheSelectedElem:(CCE_BasicForm *)newTheSelectedElem {
	theSelectedElem = newTheSelectedElem ;
}

// ---------------------------------------------------------------------
// metodi di classe per accedere alle variabili globali
+ (NSOutlineView *)
getTheOutlineView
{
	return ( [ applicationGlobals theOutlineView ] );
}

+ (CoverView *)
getTheDrawView
{
	return ( [ applicationGlobals theDrawView ] );
}
+ (CCE_BasicForm *)
getTheSelectedElem
{
	return ( [ applicationGlobals theSelectedElem ] );
}

// ---------------------------------------------------------------------
// metodi per la visualizzazione delle finestre accessorie
- (IBAction)
showPrefsWin: (id)sender
{
	// mostro la finestra invocando l'istanza condivisa
	[ [PrefWinCtrl sharedPrefsCtrl] showWindow: sender] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
showAboutBox:(id)sender
{
	// questa volta, invece di fare le solite cose, mi chiedo se l'utente ha
	// tenuto premuto il taso ALT (altrimenti noto come Option), per mostrare
	// la finestra normale oppure la nuova finestra come "Easter Egg"

	// recupero l'evento corrente, controllo lo stato della tastiera
	// e verifico se il tasto Alt  premuto
	if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
	{
		// la risposta e' si', procedo alla visualizzazione della nuova finestra
		// uso la solita tecnica
		[ [AboutWinCtrl sharedAboutWinCtrl] showWindow: sender] ;
	}
	else
	{
		// nessun tasto premuto, faccio vedere le solite cose
		[ NSApp	orderFrontStandardAboutPanel: self ];
	}
}

@end
