// -----------------------------------------------------------------------------
//  CCE_BasicForm.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// interfaccia della classe base degli elementi grafici

#import		<Foundation/Foundation.h>

@class  	CoverView ;
#import		"CatDataSrc.h"
#import		"djZeroUtils.h"

void		ccex_drawHandleAt( NSPoint where, BOOL how );

#define 	kUndoLineWidth		1
#define 	kUndoFillColor		2
#define 	kUndoLineColor		3
#define 	kUndoIsFilled		4
#define 	kUndoIsStroked		5
#define 	kUndoIsSelected		6
#define 	kUndoIsLocked		7

#define		kStrElem_ClassName		@"_ClassName"
#define		kStrElem_objID			@"objID"
#define		kStrElem_drawPoint		@"drawPoint"
#define		kStrElem_rotAngle		@"rotAngle"
#define		kStrElem_numOfHdl		@"numOfHdl"
#define		kStrElem_localSize		@"localSize"
#define		kStrElem_cceLineWidth	@"cceLineWidth"
#define		kStrElem_cceFillColor	@"cceFillColor"
#define		kStrElem_cceLineColor	@"cceLineColor"
#define		kStrElem_cceIsFilled	@"cceIsFilled"
#define		kStrElem_cceIsStroked	@"cceIsStroked"
#define		kStrElem_cceIsLocked	@"cceIsLocked"
#define		kStrElem_elemArray		@"elemArray"
#define		kStrElem_textStorage	@"textStorage"
#define		kStrElem_automaticText	@"automaticText"
#define		kStrElem_numOfColumns	@"numOfColumns"
#define		kStrElem_compFraction	@"compFraction"
#define		kStrElem_image			@"image"

@interface CCE_BasicForm : NSObject {
	// identificatore dell'elemento
	long			objID ;
	// punto nodale dell'elemento
	NSPoint			drawPoint ;
	// angolo di rotazione corrente
	float			rotAngle ;
	// lista degli handle, sono al massimo otto
	int				numOfHdl ;
	NSPoint			hdlList[8] ;
	// la view in cui si trova
	CoverView		* ownerView ;

	// metto da parte la tf affine
	NSAffineTransform   * localTF ;
	NSAffineTransform   * reverTF ;
	// dimensioni del rettangolo in coordinate locali
	NSSize			localSize ;
	// tengo il percorso per meglio disegnare
	NSBezierPath	* theDrawPath ;

	// spessore della linea di contorno
	float			cceLineWidth ;
	// colore dell'eventuale interno e del contorno
	NSColor			* cceFillColor ;
	NSColor			* cceLineColor ;
	// se devo colorare l'interno o meno
	BOOL			cceIsFilled ;
	BOOL			cceIsStroked ;
	// se l'elemento e' selezionato o meno
	BOOL			cceIsSelected ;
	// se l'lemento puo' essere manipolato o meno
	BOOL			cceIsLocked ;
}
// metodi che ogni sottoclasse deve realizzare
// inizializzazione, in qualche forma
- (id)		initWithId: (int) ident inView: (CoverView*) cv ;
- (void)	dealloc ;
// disegno dell'elemento
- (void)	drawElement: (NSRect) inRect ;
- (void)	specificDrawing: (NSRect) inRect ;
- (void)	drawHandles ;
// costruzione dell'elemento
- (BOOL)	handleCreation: (NSEvent *)theEvent ;
- (void)	buildMeFromPt: (NSPoint) stPt withSize: (NSSize) dim ;
- (void)	buildHdlList ;
- (void)	startEditing ;
- (void)	endEditing ;
// gestione dei clic sull'elemento
- (BOOL)	checkClickOnMe:(NSPoint)point ;
- (short)	clicIntoHandle: (NSPoint) point ;
// selezione e ridimensionamento
- (void)	selectMeIfInRect: (NSRect) selRect  addingToSelez: (BOOL) add2sel ;
- (void)	moveMeBy: (NSPoint) distance onlyIfSelected: (BOOL) selOnly ;
- (BOOL)	handleResize: (NSEvent *)theEvent movingHandle: (short) hdlIdx ;
- (void)	resizeMeWith: (NSSize) newSize ;
- (void)	rotateMeAtAngle: (float) angle ;
// salvataggio e ripristino degli oggetti
- (void)	encodeWithCoder:(NSCoder *)encoder ;
- (id)		initWithCoder:(NSCoder *)decoder ;
- (NSMutableDictionary *)	getXMLDescription ;
+ (id)		buildElemFromXMLDesc:(NSDictionary *)dict ;
- (void)	loadElemXMLDesc:(NSDictionary *)dict ;

// lettura della geometria
- (NSNumber*) getElemTop ;
- (NSNumber*) getElemLeft ;
- (NSNumber*) getElemRight;
- (NSNumber*) getElemBottom  ;
- (NSNumber*) getElemMiddle ;
- (NSNumber*) getElemCenter ;
// per la gestione dello undo
- (NSUndoManager *) undoManager ;
// stringa di descrizione
- (NSString*)   describeMe ;
// aggiornamento dell'eventuale testo automatico
- (void)	updAutoText: (CatDataSrc *) catalog ;
- (NSMenu*)	contextMenu ;
// debugging
- (void)	printMeWithLevel: (int) lev ;
- (void)	calcLocalTranform ;

// metodi accessor
- (NSBezierPath *)theDrawPath;
- (void)setTheDrawPath:(NSBezierPath *)newThePath;
- (float)cceLineWidth;
- (void)setCceLineWidth:(float)newLineWidth;
- (NSColor *)cceFillColor;
- (void)setCceFillColor:(NSColor *)newFillColor;
- (NSColor *)cceLineColor;
- (void)setCceLineColor:(NSColor *)newLineColor;
- (BOOL)cceIsSelected;
- (void)setCceIsSelected:(BOOL)newIsSelected;
- (BOOL)cceIsFilled;
- (void)setCceIsFilled:(BOOL)newCceIsFilled;
- (BOOL)cceIsStroked;
- (void)setCceIsStroked:(BOOL)newCceIsStroked;
- (BOOL)cceIsLocked;
- (void)setCceIsLocked:(BOOL)newIsSelected;
- (CoverView *)ownerView;
- (void)setOwnerView:(CoverView *)newOwnerView;
- (long)objID;
- (void)setObjID:(long)newObjID;
- (float)rotAngle;
- (void)setRotAngle:(float)newRotAngle;
- (NSPoint)drawPoint;
- (void)setDrawPoint:(NSPoint)newdrawPoint;
- (NSAffineTransform *)localTF;
- (void)setLocalTF:(NSAffineTransform *)newLocalTF;
- (NSAffineTransform *)reverTF;
- (void)setReverTF:(NSAffineTransform *)newLocalTF;
- (NSSize)localSize;
- (void)setLocalSize:(NSSize)newLocalSize;

@end
