// -----------------------------------------------------------------------------
//  CCE_BasicForm.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// Realizzazione della classe base degli elementi grafici

#import		"CCE_BasicForm.h"

#import		"CoverView.h"
#import		"CoverWinCtl.h"

@implementation CCE_BasicForm

// ---------------------------------------------------------------------
// inizializzazione base di un elemento
- (id)
initWithId: (int) ident inView: (CoverView*) cv
{
	// classica inizializzazione della superclasse
	self = [super init];
	if (self)
	{
		// inizializzazione delle variabili d'istanza comune
		// objId, e' compito del chiamante
		objID = ident ;
		// disegnato nell'origine, per il momento
		drawPoint = NSMakePoint( 0, 0 );
		// con angolo di rotazione nullo
		[ self setRotAngle: 0.0F ];
		// aggiusto le trasformazioni affini
		[ self calcLocalTranform ];
		[ self setTheDrawPath: [NSBezierPath bezierPathWithRect: NSMakeRect(0.0, 0.0, 1.0, 1.0) ] ];
		// ci sono otto handle standard
		numOfHdl = 8 ;
		[ self buildHdlList ];
		// coverView, e' compito del chiamante
		ownerView = cv ;
		// spessore della linea, poca roba
		cceLineWidth = 1.0 ;
		// colore di riempimento, bianco
		[self setCceFillColor:[NSColor whiteColor]];
		// colore del contorno, nero
		[self setCceLineColor:[NSColor blackColor]];
		// non e' normalmente pieno
		cceIsFilled = FALSE ;
		// non e' selezionato ne' bloccato
		cceIsSelected = FALSE ;
		cceIsLocked = FALSE ;
	}
	return self;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// inizializzatore per i casi disperati
- (id)
init
{
	return ( [ self initWithId: 0 inView: nil ]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// costruzione di un elemento con punto base e rettangolo circoscritto
- (void)
buildMeFromPt: (NSPoint) stPt withSize: (NSSize) dim
{
	// da sovrascrivere
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
resizeMeWith: (NSSize) newSize
{
	// gestione dello undo del movimento
	[[self undoManager] setActionName: @"BasicForm -> resizeMe"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		resizeElem: self withSize: localSize ];
	// metto a posto il rettangolo che lo contiene
	[self buildMeFromPt: drawPoint withSize: newSize ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
dealloc
{
	// distruggo tutto cio' che ho creato
	// se distruggo questi due NSColor, ho dei messaggi strani
//	[ cceLineColor dealloc ];
//	[ cceFillColor dealloc ];
	[ theDrawPath dealloc ] ;
	[ localTF dealloc ] ;
	[ reverTF dealloc ] ;
	[ super dealloc ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rappresentazione in XML
- (NSMutableDictionary *)
getXMLDescription 
{
	// un dizionario per contenere le varie proprieta' XML
	NSMutableDictionary *dict = [NSMutableDictionary dictionary];
	// nome della classe
	[dict setObject: NSStringFromClass([self class])
			forKey: kStrElem_ClassName ];
	// tutte le altre variabili d'istanza fondamentali
	[dict setObject:[NSString stringWithFormat:@"%d", objID] 
			forKey: kStrElem_objID ];
	[dict setObject:NSStringFromPoint(drawPoint)
			forKey: kStrElem_drawPoint ];
	[dict setObject:[NSString stringWithFormat:@"%f", rotAngle]
			forKey: kStrElem_rotAngle ];
	[dict setObject:[NSString stringWithFormat:@"%d", numOfHdl]
			forKey: kStrElem_numOfHdl ];
	[dict setObject:NSStringFromSize(localSize)
			forKey: kStrElem_localSize ];
	[dict setObject:[NSString stringWithFormat:@"%f", cceLineWidth]
			forKey: kStrElem_cceLineWidth ];
	[dict setObject:[NSArchiver archivedDataWithRootObject: cceFillColor ]
			forKey: kStrElem_cceFillColor ];
	[dict setObject:[NSArchiver archivedDataWithRootObject: cceLineColor ]
			forKey: kStrElem_cceLineColor ];
	[dict setObject:(cceIsFilled ? @"YES" : @"NO")
			forKey: kStrElem_cceIsFilled ];
	[dict setObject:(cceIsStroked ? @"YES" : @"NO")
			forKey: kStrElem_cceIsStroked ];
	[dict setObject:(cceIsLocked ? @"YES" : @"NO")
			forKey: kStrElem_cceIsLocked ];
	return dict;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// costruzione di un elemento a partire da XML
+ (id)
buildElemFromXMLDesc:(NSDictionary *)dict 
{
	id elem = nil;
	// determino il tipo della classe
	Class theClass = NSClassFromString(
			[dict objectForKey: kStrElem_ClassName ]);
	// se ce l'ho, vado a costruire l'elemento
	if (theClass) 
	{
		// costruisco l'elemento
		elem = [ [ [ theClass alloc ] init] autorelease];
		// e poi finisco di caricarlo con i suoi dati
		if (elem) 
		{
			[elem loadElemXMLDesc:dict];
		}
	}
	return elem;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// finisco di inizializzare un elemento con le proprieta' XML 
- (void)
loadElemXMLDesc:(NSDictionary *)dict 
{
	// estraggo ogni singola variabile e l'assegno
	objID = [ [ dict objectForKey: kStrElem_objID ] intValue ] ;
	drawPoint = NSPointFromString( [dict objectForKey: kStrElem_drawPoint ] ) ;
	rotAngle = [ [ dict objectForKey: kStrElem_rotAngle ] floatValue ] ;
	numOfHdl = [ [ dict objectForKey: kStrElem_numOfHdl ] intValue ] ;
	localSize = NSSizeFromString( [ dict objectForKey: kStrElem_localSize ] ) ;
	[ self calcLocalTranform ];
	[ self setTheDrawPath: [ NSBezierPath bezierPathWithRect: 
			NSMakeRect(0, 0, localSize.width, localSize.height) ] ];
	[ self buildHdlList ];
	cceLineWidth = [ [dict objectForKey: kStrElem_cceLineWidth ] floatValue ] ;
	[ self setCceFillColor: [ NSUnarchiver unarchiveObjectWithData: 
						[dict objectForKey: kStrElem_cceFillColor ] ] ];
	[ self setCceLineColor: [ NSUnarchiver unarchiveObjectWithData: 
						[dict objectForKey: kStrElem_cceLineColor ] ] ];
	cceIsFilled = [ [dict objectForKey: kStrElem_cceIsFilled ]
							isEqualToString:@"YES"];
	cceIsLocked = [ [dict objectForKey: kStrElem_cceIsLocked ]
							isEqualToString:@"YES"];
	cceIsStroked = [ [dict objectForKey: kStrElem_cceIsStroked ]
							isEqualToString:@"YES"];
	cceIsSelected = FALSE ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// salvataggio dell'elemento con un encoder
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	if ( [encoder allowsKeyedCoding] ) 
	{		
		// ogni variabile e' codificata con un opportuno identificatore,
		// formato dallo id seguito dal nome
		// alcune variabili si saltano perche' ricostruibili dalle altre
		[ encoder encodeInt32: objID forKey: keyStr( 0, kStrElem_objID) ] ;
		[ encoder encodePoint: drawPoint forKey: keyStr(objID, kStrElem_drawPoint) ] ;
		[ encoder encodeFloat: rotAngle forKey: keyStr(objID, kStrElem_rotAngle) ] ;
		[ encoder encodeInt: numOfHdl forKey: keyStr(objID, kStrElem_numOfHdl) ] ;
		[ encoder encodeSize: localSize forKey: keyStr(objID, kStrElem_localSize) ] ;
		[ encoder encodeFloat: cceLineWidth forKey: keyStr(objID, kStrElem_cceLineWidth) ] ;
		[ encoder encodeObject: cceFillColor forKey: keyStr(objID, kStrElem_cceFillColor) ] ;
		[ encoder encodeObject: cceLineColor forKey: keyStr(objID, kStrElem_cceLineColor) ] ;
		[ encoder encodeBool: cceIsFilled forKey: keyStr(objID, kStrElem_cceIsFilled) ] ;
		[ encoder encodeBool: cceIsStroked forKey: keyStr(objID, kStrElem_cceIsStroked) ] ;
		[ encoder encodeBool: cceIsLocked forKey: keyStr(objID, kStrElem_cceIsLocked) ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// ripristino di un elemento da un decoder
- (id)
initWithCoder:(NSCoder *)decoder 
{
	if ( [decoder allowsKeyedCoding] ) 
	{
		objID = [decoder decodeInt32ForKey: keyStr( 0, kStrElem_objID) ];	
		drawPoint = [decoder decodePointForKey: keyStr( objID, kStrElem_drawPoint) ];
		rotAngle = [decoder decodeFloatForKey: keyStr( objID, kStrElem_rotAngle) ];
		numOfHdl = [decoder decodeIntForKey: keyStr( objID, kStrElem_numOfHdl) ];	
		localSize = [decoder decodeSizeForKey: keyStr( objID, kStrElem_localSize) ];
		[ self calcLocalTranform ];
		[ self setTheDrawPath: [NSBezierPath bezierPathWithRect: 
			NSMakeRect(0, 0, localSize.width, localSize.height) ] ];
		[ self buildHdlList ];
		cceLineWidth = [decoder decodeFloatForKey: keyStr( objID, kStrElem_cceLineWidth) ];
		[ self setCceFillColor: [ decoder decodeObjectForKey: keyStr( objID, kStrElem_cceFillColor) ] ];
		[ self setCceLineColor: [ decoder decodeObjectForKey: keyStr( objID, kStrElem_cceLineColor) ] ];
		cceIsFilled = [decoder decodeBoolForKey: keyStr( objID, kStrElem_cceIsFilled) ];	
		cceIsLocked = [decoder decodeBoolForKey: keyStr( objID, kStrElem_cceIsLocked) ];	
		cceIsStroked = [decoder decodeBoolForKey: keyStr( objID, kStrElem_cceIsStroked) ];	
		cceIsSelected = FALSE ;
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// calcolo della trasformazione affine e delal sua inversa in base
// al punto base ed alla rotazione
- (void)
calcLocalTranform
{
	// trasformazione affine per la rototraslazione
	NSAffineTransform *transform = [NSAffineTransform transform];
	// rototraslazione
	[ transform translateXBy: drawPoint.x yBy: drawPoint.y ];
	[ transform rotateByDegrees: rotAngle ];
	[ self setLocalTF: transform ];
	// gia' che ci sono, calcolo anche l'inversa
	[ transform invert ];
	[ self setReverTF: transform ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// costruzione degli handle
- (void)
buildHdlList
{
		// gli handle sono genericamente ai quattro estremi del
		// rettangolo limite e nei quattro punti mediani
		hdlList[0] = NSMakePoint( 0, 0) ;
		hdlList[2] = NSMakePoint( localSize.width, 0) ;
		hdlList[4] = NSMakePoint( localSize.width, localSize.height) ;
		hdlList[6] = NSMakePoint( 0, localSize.height ) ;
		hdlList[1].x = (hdlList[0].x + hdlList[2].x ) * 0.5 ;
		hdlList[1].y = (hdlList[0].y + hdlList[2].y ) * 0.5 ;
		hdlList[3].x = (hdlList[2].x + hdlList[4].x ) * 0.5 ;
		hdlList[3].y = (hdlList[2].y + hdlList[4].y ) * 0.5 ;
		hdlList[5].x = (hdlList[4].x + hdlList[6].x ) * 0.5 ;
		hdlList[5].y = (hdlList[4].y + hdlList[6].y ) * 0.5 ;
		hdlList[7].x = (hdlList[6].x + hdlList[0].x ) * 0.5 ;
		hdlList[7].y = (hdlList[6].y + hdlList[0].y ) * 0.5 ;
}

// ---------------------------------------------------------------------
// metodo per il disegno complessivo
- (void)
drawElement: (NSRect) inRect
{
	// per salvare il contesto
	NSGraphicsContext *context = [NSGraphicsContext currentContext];
	[context saveGraphicsState];
	[ localTF concat ];
	[ self specificDrawing: inRect ];
	if ( [context isDrawingToScreen ] )
		[ self drawHandles ] ;
	// ripristino del contesto
	[context restoreGraphicsState];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
specificDrawing: (NSRect) inRect
{
	// da sovrascrivere
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo per il disegno degli handle
- (void)
drawHandles
{
	if ( cceIsSelected )
	{
		int  i;
		// disegno i manici per la manipolazione
		for ( i = 0 ; i < numOfHdl ; i ++ )
			ccex_drawHandleAt( hdlList[i], cceIsLocked );
	}
}

// ---------------------------------------------------------------------
// creazione di un elemento tramite mouse
- (BOOL)
handleCreation: (NSEvent *)theEvent
{
	CoverView   * view = [self ownerView] ;
	// recupero la coordinata locale
	NSPoint 	point1 = [ view convertPoint:
		[theEvent locationInWindow] fromView:nil];
	NSPoint 	point2  ;
	BOOL 		shiftKeyPress ;
	NSSize		tmpSize ;
	// piglio il punto piu' vicino sulla griglia (se necessario)
	point1 = getNearestSnapPoint( point1, view);
	// inizializzo l'aspetto dell'oggetto (praticamente vuoto)
	[ self buildMeFromPt: point1 withSize: NSMakeSize(1,1) ] ;
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse
		// oppure un mouseup (che finisce il tutto)
		theEvent = [[view window] nextEventMatchingMask:
			(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
		shiftKeyPress = (([theEvent modifierFlags] & NSShiftKeyMask) ? YES : NO);
		// recupero la coordinata della finestra
		point2 = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		point2 = getNearestSnapPoint( point2, view);
		tmpSize = NSMakeSize(point2.x-point1.x, point2.y-point1.y) ;
		if ( shiftKeyPress )
		{
			// costringo il rettangolo ad essere quadrato
			float   delta1 = tmpSize.width ;
			float   delta2 = tmpSize.height ;
			float	dirx = 1, diry = 1 ;
			if ( delta1 < 0 )
			{
				delta1 = - delta1 ; dirx = -1 ;
			}
			if ( delta2 < 0 )
			{
				delta2 = - delta2 ; diry = -1 ;
			}
			// piglio la dimensione piu' corta
			if ( delta1 < delta2 )
				tmpSize.height = diry * delta1 ;
			else
				tmpSize.width = dirx * delta2 ;
		}
		// aggiusto l'aspetto dell'oggetto
		[ self buildMeFromPt: point1 withSize: tmpSize ] ;
		// forzo un ridisegno della finestra
		[ view setNeedsDisplay: YES ];
	}
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// ridimensionamento di un elemento tramite mouse
- (BOOL)
handleResize: (NSEvent *)theEvent
	movingHandle: (short) hdlIdx
{
	NSPoint point ;
	NSSize  newSize ;
	CoverView   * view = [self ownerView] ;
	// indice dello handle che tengo fisso
	static short fixedHandle[8] =  { 4, 4, 6, 0, 0, 0, 2, 4 };
	// indice dello handle che devo muovere (opposto)
	static short movingHandle[8] = { 0, 0, 2, 4, 4, 4, 6, 0 };
	// identificatore delle dimensioni che posso modificare:
	// con 2, solo width, con 1 solo height, con 3, w e h
	static short movingPoints[8] = { 3, 1, 3, 2, 3, 1, 3, 2 };
	short   movingCoord = movingPoints[ hdlIdx ];
	NSPoint fixedPoint = hdlList[ fixedHandle[ hdlIdx ] ];
	NSPoint movingPoint = hdlList[ movingHandle[hdlIdx] ];
	// se e' locked, non se ne fa nulla
	if ( cceIsLocked ) return YES ;
	// trasformo i punti fissi e mobili nelle coordinate della finestra
	fixedPoint = [ localTF transformPoint: fixedPoint ];
	movingPoint = [ localTF transformPoint: movingPoint ];
	// ridisegno l'oggetto con un nuovo sistema di riferimento
	newSize = [ reverTF transformSize:
		NSMakeSize(movingPoint.x-fixedPoint.x, movingPoint.y-fixedPoint.y) ] ;
	[ self buildMeFromPt: fixedPoint withSize: newSize ] ;
	// ho ricostruito l'oggetto uguale a se stesso; tuttavia, potrebbe essere
	// cambiato il punto base e la rotazione
	// mi preparo lo undo
	[[self undoManager] setActionName: @"BasicForm -> handleResize"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		resizeElem: self withSize: localSize ];
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse
		// oppure un mouseup (che finisce il tutto)
		theEvent = [[view window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
		// recupero la coordinata della finestra
		point = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		point = getNearestSnapPoint( point, view);
		// ho una nuova dimensione dell'oggetto
		newSize = [ reverTF transformSize:
			NSMakeSize(point.x-fixedPoint.x, point.y-fixedPoint.y) ] ;
		// se il caso, lascio invariata una delle due dimensioni
		if ( movingCoord == 1)
			newSize.width = localSize.width ;
		else if ( movingCoord == 2)
			newSize.height = localSize.height ;
		// ricostruisco l'oggetto con una nuova dimensione
		[ self buildMeFromPt: fixedPoint withSize: newSize] ;
	}
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
startEditing
{
	// da sovrascrivere nei casi opportuni
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
endEditing
{
	// da sovrascrivere nei casi opportuni
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per sapere se il clic e' sopra l'elemento
- (BOOL)
checkClickOnMe:(NSPoint)point
{
	float   ev = cceLineWidth + 2;
	// trasformo il punto in coordinate locali
	NSPoint locPt = [ reverTF transformPoint: point ];
	BOOL	insideX = NO, insideY = NO ;
	// qui faccio un po' di confusione, in quanto PointInRect
	// va in confusione con dimensioni negative o nulle
	if ( localSize.width >= 0 )
	{
		// se la dimensione e' positiva ...
		// controllo se la coordinata del punto e' compresa
		// tra un Zero e la lunghezza (con tolleranza...)
		if ( (-ev < locPt.x) && (locPt.x < (localSize.width + ev)) )
			insideX = YES ;
	}
	else
	{
		// se la dimensione e' negativa...
		// controllo se la coordinata del punto e' compresa tra la
		// lunghezza (negativa) e zero (con tolleranza)
		if ( ((- ev + localSize.width) < locPt.x) && (locPt.x < ev) )
			insideX = YES ;
	}
	// faccio lo stesso per l'altra dimensione
	if ( localSize.height >= 0 )
	{
		if ( (-ev < locPt.y) && (locPt.y< (localSize.height + ev)) )
			insideY = YES ;
	}
	else
	{
		if ( ((- ev + localSize.height) < locPt.y) && (locPt.y < ev) )
			insideY = YES ;
	}
	// il punto sta dentro se e' compreso in entrambe le coordinate
	return ( insideX && insideY );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per sapere se il clic e' sopra uno degli handle
- (short)
clicIntoHandle: (NSPoint) point
{
	int  i;
	// ricavo le coordinate locali del punto
	point = [ reverTF transformPoint: point ];
	for ( i = 0 ; i < numOfHdl ; i ++ )
	{
		// se il punto designato e lo handle sono abbastanza vicini
		float   a1 = hdlList[i].x - point.x ;
		float   a2 = hdlList[i].y - point.y ;
		if ( a1 * a1 + a2 * a2 <= 4 )
		{
			return ( i );
		}
	}
	return ( -1 ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// selezione di un elemento attraverso un rettangolo
- (void)
selectMeIfInRect: (NSRect) selRect addingToSelez: (BOOL) add2sel
{
	BOOL	inrect ;
	// calcolo il rettangolo che circoscrive il percorso dell'elemento
	NSRect  limRect = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	// lo allargo di un pelo per tenere conto di rettangoli nulli
	// (che non fanno funzionare le cose)
	limRect = NSInsetRect(limRect,  -1, -1 );
	// vedo se il rettangolo di selezione comprende l'elemento
	inrect = NSContainsRect( selRect, limRect ) ;
	// se non devo aggiungere alla selezione
	if ( ! add2sel )
	{
		// vince il criterio del rettangolo
		[ self setCceIsSelected: inrect ];
	}
	else
	{
		// qui e' da aggiungere alla selezione
		// se sono dentro il rettangolo
		if ( inrect )
			// per certo lo seleziono
			[ self setCceIsSelected: YES ];
		// altrimenti lascio la selezione precedente
	}
	[ [ self ownerView ] setNeedsDisplay: YES ];
}

// ---------------------------------------------------------------------
- (void)
moveMeBy: (NSPoint) distance onlyIfSelected: (BOOL) selOnly
{
	// se selOnly e' Falso, muovo sicuro
	// se invece e' vero, solo se non locked e selected
	if ( (selOnly == NO) || ((! cceIsLocked) && cceIsSelected) )
	{
		// gestione dello undo del movimento
		[[self undoManager] setActionName: @"BasicForm -> moveMeBy"];
		[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
			moveElem: self by: NSMakePoint(-distance.x, -distance.y)];
		// effettuo il movimento vero e proprio: sposto il punto base
		drawPoint.x += distance.x ;
		drawPoint.y += distance.y ;
		// aggiorno la trasformazione affine
		[ self calcLocalTranform ];
		// ricosctruisco gli handle
		[ self buildHdlList ] ;
		// devo sicuramente ridisegnare qualcosa
		[ [ self ownerView ] setNeedsDisplay: YES ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rotazione incondizionata dell'elemento
- (void)
rotateMeAtAngle: (float) angle
{
	NSAffineTransform * transform;
	// preparo lo undo
	[[self undoManager] setActionName: @"BasicForm -> rotateMeAtAngle"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		rotateElem: self byAngle: rotAngle ];
	// parto dalla trasformazione corrente
	transform = [ [ NSAffineTransform alloc] initWithTransform: localTF ] ;
	// mi spostoal centro dell'elemento
	[ transform translateXBy: localSize.width*0.5 yBy: localSize.height*0.5 ];
	// eseguo la rotazione residua sull'oggetto
	[ transform rotateByDegrees: (angle - rotAngle) ];
	// ritorno nel punto base
	[ transform translateXBy: -localSize.width*0.5 yBy: -localSize.height*0.5 ];
	// bisogna aggiornare la variabili interne
	[ self setRotAngle: angle ] ;
	// ma guarda dov'e' finita l'origine...
	drawPoint = [ transform transformPoint: NSMakePoint(0, 0) ];
	// calcolo ed assegno le varie trasformazioni
	[ self setLocalTF: transform ];
	[ transform invert ];
	[ self setReverTF: transform ];
	// aggiorno gli handle
	[ self buildHdlList ] ;
	[ [ self ownerView ] setNeedsDisplay: YES ];
}

// ---------------------------------------------------------------------
// per accere comodamente ai limiti dell'elemento
- (NSNumber*)
getElemTop
{
	// piglio il percorso e lo trasformo nelle coordinate della finestra
	// da qui, ricavo il rettangolo circoscritto, e poi l'estremo di interesse
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: limR.origin.y] );
}

- (NSNumber*)
getElemLeft
{
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: limR.origin.x] );
}

- (NSNumber*)
getElemRight
{
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: (limR.origin.x + limR.size.width)] );
}

- (NSNumber*)
getElemBottom
{
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: (limR.origin.y + limR.size.height)] );
}

- (NSNumber*)
getElemMiddle
{
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: (limR.origin.y + 0.5 * limR.size.height)] );
}

- (NSNumber*)
getElemCenter
{
	NSRect  limR = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	return ( [ NSNumber numberWithFloat: (limR.origin.x + 0.5 * limR.size.width)] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (NSUndoManager *)
undoManager
{
	return [[[ownerView winCtl] document] undoManager];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// stringa che descrive l'elemento
- (NSString*)
describeMe
{
	return ([ NSString stringWithFormat: @"Generic Elem ID %d", [ self objID] ]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
updAutoText: (CatDataSrc *) catalog 
{
	// normalmente faccio nulla
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// menu contestuale proprio dell'elemento
- (NSMenu*)
contextMenu 
{
	return ( [ ownerView stdElemMenu ] );
}

// ---------------------------------------------------------------------
// metodi accessor

- (float)cceLineWidth {
	return cceLineWidth;
}

- (void)setCceLineWidth:(float)newLineWidth
{
	// gestione dello undo
	[[self undoManager] setActionName: @"BasicForm -> cceLineWidth"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		changeElem: self attrib: kUndoLineWidth value: [NSNumber numberWithFloat: cceLineWidth] ];
	// assegnazione effettiva
	cceLineWidth = newLineWidth;
}

- (NSColor *)cceFillColor {
	return cceFillColor ;
}

- (void)setCceFillColor:(NSColor *)newFillColor
{
	if (cceFillColor != newFillColor)
	{
		// gestione dello undo
		[[self undoManager] setActionName: @"BasicForm -> cceFillColor"];
		[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
			changeElem: self attrib: kUndoFillColor value: cceFillColor ];
		// assegnazione effettiva
		[cceFillColor release];
		cceFillColor = [newFillColor copy];
	}
}

- (NSColor *)cceLineColor {
	return cceLineColor ;
}

- (void)setCceLineColor:(NSColor *)newLineColor
{
	if (cceLineColor != newLineColor)
	{
		// gestione dello undo
		[[self undoManager] setActionName: @"BasicForm -> cceLineColor"];
		[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
			changeElem: self attrib: kUndoLineColor value: cceLineColor ];
		// gestione dello undo
		[cceLineColor release];
		cceLineColor = [newLineColor copy];
	}
}

- (BOOL)cceIsSelected {
	return cceIsSelected;
}

- (void)setCceIsSelected:(BOOL)newCceIsSelected
{
	if (cceIsSelected != newCceIsSelected)
	{
		// gestione dello undo
		[[self undoManager] setActionName: @"BasicForm -> cceIsSelected"];
		[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
			changeElem: self attrib: kUndoIsSelected value: [NSNumber numberWithBool: cceIsSelected] ];
		// gestione dello undo
		cceIsSelected = newCceIsSelected;
		[ [ self ownerView ] setNeedsDisplay: YES ];
	}
}

- (BOOL)cceIsLocked {
	return cceIsLocked;
}

- (void)setCceIsLocked:(BOOL)newIsLocked
{
	// gestione dello undo
	[[self undoManager] setActionName: @"BasicForm -> cceIsLocked"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		changeElem: self attrib: kUndoIsLocked value: [NSNumber numberWithBool: cceIsLocked] ];
	// assegnazione effettiva
	cceIsLocked = newIsLocked;
}

- (BOOL)cceIsFilled {
	return cceIsFilled;
}

- (void)setCceIsFilled:(BOOL)newCceIsFilled
{
	// gestione dello undo
	[[self undoManager] setActionName: @"BasicForm -> cceIsFilled"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		changeElem: self attrib: kUndoIsFilled value: [NSNumber numberWithBool: cceIsFilled] ];
	// assegnazione effettiva
	cceIsFilled = newCceIsFilled;
}

- (BOOL)cceIsStroked {
	return cceIsStroked;
}

- (void)setCceIsStroked:(BOOL)newCceIsStroked
{
	// gestione dello undo
	[[self undoManager] setActionName: @"BasicForm -> cceIsStroked"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		changeElem: self attrib: kUndoIsStroked value: [NSNumber numberWithBool: cceIsStroked] ];
	// assegnazione effettiva
	cceIsStroked = newCceIsStroked;
}

- (CoverView *)ownerView {
	return ownerView ;
}

- (void)setOwnerView:(CoverView *)newOwnerView {
		ownerView = newOwnerView;
}

- (long)objID {
	return objID;
}

- (void)setObjID:(long)newObjID {
		objID = newObjID;
}

- (float)rotAngle {
	return rotAngle;
}

- (void)
setRotAngle:(float)newRotAngle
{
	rotAngle = newRotAngle;
	// per evitare che l'angolo esploda,
	// lo riporto sempre tra 0 e 360
	while ( rotAngle >= 360 )
		rotAngle -= 360 ;
	while ( rotAngle < 0 )
	rotAngle += 360 ;
}

- (NSPoint)drawPoint {
	return drawPoint;
}
- (void)setDrawPoint:(NSPoint)newdrawPoint {
		drawPoint = newdrawPoint;
}

- (NSSize)localSize {
	return localSize;
}

- (void)setLocalSize:(NSSize)newLocalSize {
	localSize = newLocalSize;
}

- (NSAffineTransform *)localTF {
	return [[localTF retain] autorelease];
}
- (NSAffineTransform *)reverTF {
	return [[reverTF retain] autorelease];
}

- (void)setLocalTF:(NSAffineTransform *)newLocalTF {
	if (localTF != newLocalTF) {
		[localTF release];
		localTF = [newLocalTF copy];
	}
}

- (void)setReverTF:(NSAffineTransform *)newLocalTF {
	if (reverTF != newLocalTF) {
		[reverTF release];
		reverTF = [newLocalTF copy];
	}
}

- (NSBezierPath *)theDrawPath {
	return [[theDrawPath retain] autorelease];
}

- (void)setTheDrawPath:(NSBezierPath *)newTheRect {
	if (theDrawPath != newTheRect) {
		[theDrawPath release];
		theDrawPath = [newTheRect copy];
	}
}

// ---------------------------------------------------------------------
// per avere una stamap delle caratteristiche dell'elemento
- (void)
printMeWithLevel: (int) lev
{
	int i;
	char	blankChar[40];
	NSString	* msg ;
	if ( lev > 10 ) lev = 10 ;
	for ( i = 0; i < lev*4 ; i ++ )
		blankChar[i]= ' ';
	blankChar[lev*4] = 0 ;
	msg = [ NSString stringWithCString: blankChar];
	NSLog( @"%@%@, Dp:( %7.2f, %7.2f) w: %7.2f h: %7.2f a: %7.2f)",
		msg, [ self describeMe], drawPoint.x, drawPoint.y,
		localSize.width, localSize.height, rotAngle );
	NSLog( @"%@   Lw: %6.2f, F: %2d, S: %2d, Sel: %2d, Lk: %2d ",
		msg, cceLineWidth, cceIsFilled,cceIsStroked, cceIsSelected, cceIsLocked );
}
@end

// ---------------------------------------------------------------------
void
ccex_drawHandleAt( NSPoint where, BOOL how )
{
	// per ora la dimensione del manico e' fissa
	int		handleDim = 3 ;
	NSRect	hRect, rRect ;
	// disegno un rettangolo pieno attorno al punto
	hRect = NSMakeRect( where.x, where.y,2*handleDim, 2*handleDim );
	rRect = NSOffsetRect( hRect , -handleDim, -handleDim);
	hRect = NSOffsetRect(rRect, 1.0, 1.0);
	if ( how )
			[[NSColor darkGrayColor] set];
	else	[[NSColor lightGrayColor] set];
	NSRectFill(hRect);
	if ( how )
			[[NSColor lightGrayColor] set];
	else	[[NSColor darkGrayColor] set];
	NSRectFill(rRect);
}
