// -----------------------------------------------------------------------------
//  CCE_ElemGroup.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe che modella un gruppo di elementi

#import		"CCE_ElemGroup.h"

#import		"CCE_BasicForm.h"
#import		"CoverView.h"

@implementation CCE_ElemGroup

// ---------------------------------------------------------------------
// per costruire l'elemento, inizializzo il vettore
- (id)
initWithId: (int) ident inView: (CoverView*) cv
{
	self = [ super initWithId: ident inView: cv ] ;
	if ( self )
	{
		[ self setElemArray: [ NSMutableArray array ]];
	}
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-(id)
init
{
	return ( [ self initWithId: 0 inView: nil ])  ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// distruggo il vettore
-(void)
dealloc
{
	[ elemArray release ] ;
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rappresentazione in XML
- (NSMutableDictionary *)
getXMLDescription
{
	// la descrizione dell'elemento stesso
    NSMutableDictionary * dict = [super getXMLDescription];
	// ed un array con la descrizione degli elementi del gruppo
	NSMutableArray		* newArr = [ NSMutableArray arrayWithCapacity: 1 ] ;
	int					i , numElem ;
	// ciclo sugli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		CCE_BasicForm		* elem ;
		NSMutableDictionary * dictx ;
		// piglio l'elemento
		elem = [ elemArray objectAtIndex: i ];
		// e costruisco un dizionario con le sue proprieta'
		dictx = [ elem getXMLDescription ];
		// che aggiungo allo array
		[ newArr addObject: dictx ];
	}
	// al dizionario aggiungo appunto lo array
	[dict setObject: newArr forKey: kStrElem_elemArray ];
	return ( dict );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// finisco di inizializzare questo elemento con le proprieta' XML 
- (void)
loadElemXMLDesc:(NSDictionary *)dict 
{
	NSArray		* newArr ;
	int			i ;
	// predispongo le variabili comuni
    [super loadElemXMLDesc:dict];
	// recupero lo array con i vari elementi del gruppo
	newArr = [ NSArray arrayWithArray:(NSArray *) 
				[dict objectForKey: kStrElem_elemArray ] ] ;
	// guardo i singoli elementi
	for ( i = 0 ; i < [ newArr count ] ; i ++ )
	{
		CCE_BasicForm		* elem ;
		NSMutableDictionary * dictx ;
		// recupero la descrizione dell'elemento in XML
		dictx = [ newArr objectAtIndex: i ];
		// costruisco l'elemento stesso
		elem = [ CCE_BasicForm buildElemFromXMLDesc: dictx ] ;
		// e lo aggungo al gruppo di elementi
		[ elemArray addObject: elem ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// codifica elemento
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	[ super encodeWithCoder: encoder ] ;
	// devo aggiungere la codifica di tutti gli elementi
	if ( [encoder allowsKeyedCoding] ) 
	{		
		// basta codificare solo lo array
		[ encoder encodeObject: elemArray forKey: keyStr(objID, kStrElem_elemArray) ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
initWithCoder:(NSCoder *)decoder 
{
	[ super initWithCoder: decoder ] ;
    if ( [decoder allowsKeyedCoding] ) 
	{
		[ self setElemArray: [ decoder decodeObjectForKey: keyStr(objID, kStrElem_elemArray) ] ];
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// disegno ogni singolo elemento del vettore
- (void)
specificDrawing: (NSRect) aRect
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	numElem = [ elemArray count ];
	for ( i = numElem-1 ; i >= 0 ; i -- )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem drawElement: aRect ];
	}
}

// ---------------------------------------------------------------------
// aggiungo un elemento al vettore
- (void)
addElem: (CCE_BasicForm *) elem atIndex: (int) i
{
	// gestione dello undo per aggiunta/rimozione elementi
	[[self undoManager] setActionName: @"ElemGroup -> addElem"];
	[[[self undoManager] prepareWithInvocationTarget:self] remElemAtIndex: i ];
	// inserisco davanti a tutto
	[ elemArray insertObject: elem atIndex:i];
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// tolgo un elemento al vettore
- (void)
remElemAtIndex: (int) i
{
	// gestione dello undo per aggiunta/rimozione elementi
	[[self undoManager] setActionName: @"ElemGroup -> remElem"];
	[[[self undoManager] prepareWithInvocationTarget: self] addElem: [ elemArray objectAtIndex: i ] atIndex: i ];
	// eseguo la cancellazione
	[ elemArray removeObjectAtIndex: i ];
	[ ownerView setNeedsDisplay: YES ];
}

// ---------------------------------------------------------------------
// creazione di un gruppo a partire dagli elementi selezionati
- (void)
makeGroupFromSelected
{
	int		i , numElem, firstIdx = -1, lastIdx ;
	CCE_BasicForm   * elem ;
	CCE_ElemGroup   * newGrp ;
	// ciclo su tutti gli elementi del gruppo
	lastIdx = numElem = [ elemArray count ];
	newGrp = [[[ CCE_ElemGroup alloc] initWithId:
		[ownerView getNextObjId] inView: ownerView ] autorelease ];
	for ( i = numElem-1 ; i >= 0; i -- )
	{
		elem = [ elemArray objectAtIndex: i ];
		if (  [elem cceIsLocked] || ! [ elem cceIsSelected ] )
			continue ;
		// se arrivo qui, l'elemento e' libero e selezionato
		[ elem retain ];
		if ( firstIdx == -1 )
			firstIdx = i ;
		if ( lastIdx > i )
			lastIdx = i ;
		// comunque vada; lo tolgo dallo array
		[ elemArray removeObjectAtIndex: i ];
		// e lo aggiungo al nuovo gruppo
		[ [newGrp elemArray] addObject: elem ];
		[ elem release ];
	}
	// quando arrivo qui, ho costruito un nuovo gruppo
	// se non c'e' nulla, faccio nulla
	if ( firstIdx == -1)
		return ;
	// se c'e' un solo elemento, lo rimetto dov'era
	if ( [ [newGrp elemArray] count ] == 1 )
	{
		[ elemArray insertObject:
			[[newGrp elemArray] objectAtIndex:0] atIndex: firstIdx];
		return ;
	}
	// finalmente, se arrivo qui, ho un gruppo vero e proprio
	// aggiusto tutti i valori del gruppo
	[ newGrp adaptModification ] ;
	// tutti gli elementi interni non sono selezionati
	[ newGrp cascadeSelection: NO ];
	// il gruppo invece rimane selezionato
	[ newGrp setCceIsSelected: YES ];
	// inserisco il gruppo nel posto opportuno
	[ elemArray insertObject: newGrp atIndex: lastIdx];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
adaptModification
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	float		xmin, xmax, ymin, ymax ;

	numElem = [ elemArray count ];
	elem = [ elemArray objectAtIndex: 0 ];
	ymin = [[ elem getElemTop] floatValue];
	xmin = [[ elem getElemLeft] floatValue];
	xmax = [[ elem getElemRight] floatValue];
	ymax = [[ elem getElemBottom] floatValue];
	// ciclo su tutti gli elementi del gruppo
	for ( i = 1 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		// cerco i limiti del rettangolo circoscritto
		if ( ymin > [[ elem getElemTop] floatValue])
			ymin = [[ elem getElemTop] floatValue];
		if ( xmin > [[ elem getElemLeft] floatValue] )
			xmin = [[ elem getElemLeft] floatValue];
		if ( xmax < [[ elem getElemRight] floatValue] )
			xmax = [[ elem getElemRight] floatValue];
		if ( ymax < [[ elem getElemBottom] floatValue] )
			ymax = [[ elem getElemBottom] floatValue];
	}
	[ self setTheDrawPath: [NSBezierPath bezierPathWithRect: 
			NSMakeRect( 0, 0, xmax - xmin, ymax - ymin) ] ];
	[ self setDrawPoint: NSMakePoint( xmin, ymin ) ];
	[ self setRotAngle: 0 ];
	[ self setLocalSize: NSMakeSize( xmax - xmin, ymax - ymin) ] ;
	[ self calcLocalTranform ];
	[ self buildHdlList ] ;
	// sposto le origini di ogni elemento riferendole al gruppo
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem setDrawPoint: NSMakePoint( [elem drawPoint].x -xmin, [elem drawPoint].y - ymin ) ];
		[ elem calcLocalTranform ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// distruggo un gruppo e riporto gli elementi al livello superiore
- (void )
ungroupAndInsertIn: (CCE_ElemGroup *) masterArray  startingFrom: (int) idx
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	// per ogni elemento presente, lo aggiungo
	for ( i = 0 ; i < numElem ; i ++ )
	{
		NSAffineTransform	* tf1 = [ NSAffineTransform transform ];
		elem = [ elemArray objectAtIndex: i ];
		[ elem setCceIsSelected: YES ] ;
		[ [masterArray elemArray] insertObject: elem atIndex: idx ];
		[ tf1 initWithTransform: localTF ];
		[ tf1 prependTransform: [ elem localTF ] ];
		[ elem setLocalTF: tf1 ] ;
		[ elem setDrawPoint: [ tf1 transformPoint: NSMakePoint(0, 0) ] ];
		[ elem setRotAngle: ( [elem rotAngle] + rotAngle) ];
		[ tf1 invert ];
		[ elem setReverTF: tf1 ];
	}
}

// ---------------------------------------------------------------------
// spostamento degli elementi avanti ed indietro
- (void )
sendElemFrontAndBack: (int) howMuch
{
	int		i , numElem, newIdx ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	// devo distinguere tra l'avanti e l'indietro
	if ( howMuch > 0 )
	{
		for ( i = 1 ; i < numElem ; i ++ )
		{
			elem = [ elemArray objectAtIndex: i ];
			if ( [elem cceIsLocked] || ! [ elem cceIsSelected ] )
				continue ;
			// se arrivo qui, devo spostare l'elemento
			[ elem retain ];
			// comunque vada; lo tolgo dallo array
			[ elemArray removeObjectAtIndex: i ];
			// vedo cosa ne devo fare
			if ( howMuch == +1 )
					newIdx = i-1 ;
			else	newIdx = 0 ;
			[ elemArray insertObject: elem atIndex: newIdx ];
			[ elem release ];
		}
	}
	else
	{
		for ( i = numElem-2 ; i >= 0 ; i -- )
		{
			elem = [ elemArray objectAtIndex: i ];
			if ( [elem cceIsLocked] || ! [ elem cceIsSelected ] )
				continue ;
			// se arrivo qui, devo spostare l'elemento
			[ elem retain ];
			// comunque vada; lo tolgo dallo array
			[ elemArray removeObjectAtIndex: i ];
			// vedo cosa ne devo fare
			if ( howMuch == -1 )
					newIdx = i+1 ;
			else	newIdx = numElem-1 ;
			[ elemArray insertObject: elem atIndex: newIdx ];
			[ elem release ];
		}
	}
}

// ---------------------------------------------------------------------
// attribuzione a cascata dell'attributo di selezione
- (void)
cascadeSelection:(BOOL)newIsSelected
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if ( [elem isKindOfClass:[CCE_ElemGroup class]] )
			[ ((CCE_ElemGroup*) elem) cascadeSelection: newIsSelected ];
		[ elem setCceIsSelected: newIsSelected ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// movimento degli elementi selezionati
- (void)
moveSelectedElements: (NSPoint) distance
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem moveMeBy: distance onlyIfSelected: YES ];
	}
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rotazione degli elementi selezionati
- (void)
rotateElemAngle: (float) angle absolute: (BOOL) absInc
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if (  [ elem cceIsLocked] || ( ! [ elem cceIsSelected] ))
			continue ;
		if ( absInc )
				[ elem rotateMeAtAngle: angle ] ;
		else	[ elem rotateMeAtAngle: ([ elem rotAngle] + angle) ]  ;
	}
}

// ---------------------------------------------------------------------
// costruzione di un array coi soli elementi selezionati
- (NSMutableArray *)
getSelectedGraphics
{
	NSMutableArray  * selArr = [[ NSMutableArray alloc] init ] ;
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if ([ elem cceIsSelected] )
			[ selArr addObject: elem ];
	}
	return ( selArr );
}

// ---------------------------------------------------------------------
// cambio del font di tutti gli elementi testo selezionati
- (void)
changeFont:(id)sender
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		if ( [ elem cceIsSelected ] && [elem isKindOfClass:[CCE_Text class]])
			[ elem changeFont: sender ];
	}
}

// ---------------------------------------------------------------------
// metodi per la gestione unificata degli undo
// movimento
- (void)
moveElem: (CCE_BasicForm*) elem by: (NSPoint) distance
{
	[ elem moveMeBy: distance onlyIfSelected: NO ];
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// ridimensionamento
- (void)
resizeElem: (CCE_BasicForm*) elem withSize: (NSSize) newSize
{
	[ elem resizeMeWith: newSize ];
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rotazione
- (void)
rotateElem: (CCE_BasicForm*) elem byAngle: (float) angle
{
	[ elem rotateMeAtAngle: angle ];
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cambio attributi
- (void)
changeElem: (CCE_BasicForm*) elem attrib: (int) quale value: (id) valore
{
	switch ( quale ) {
	case kUndoLineWidth:
		[ elem setCceLineWidth: [(NSNumber*)valore floatValue ] ];
		break ;
	case kUndoFillColor:
		[ elem setCceFillColor: (NSColor*)valore ];
		break ;
	case kUndoLineColor:
		[ elem setCceLineColor: (NSColor*)valore ];
		break ;
	case kUndoIsFilled:
		[ elem setCceIsFilled:	[(NSNumber*)valore boolValue ] ];
		break ;
	case kUndoIsStroked:
		[ elem setCceIsStroked:	[(NSNumber*)valore boolValue ] ];
		break ;
	case kUndoIsSelected:
		[ elem setCceIsSelected:[(NSNumber*)valore boolValue ] ];
		break ;
	case kUndoIsLocked:
		[ elem setCceIsLocked:	[(NSNumber*)valore boolValue ] ];
		break ;
	}
	[ [ self ownerView ] setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// stringa che descrive l'elemento
- (NSString*)
describeMe
{
	return ( [ NSString stringWithFormat: @"Group Elem ID %d", [ self objID] ] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
updAutoText: (CatDataSrc *) catalog 
{
	// passo la palla agli elementi del gruppo
	[ elemArray makeObjectsPerformSelector: @selector(updAutoText:) withObject: catalog ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// menu contestuale proprio dell'elemento
- (NSMenu*)
contextMenu 
{
	return ( [ ownerView grpElemMenu ] );
}

// ---------------------------------------------------------------------
// accessor riscritti per propagare il valore della variabile
- (void)
setCceIsLocked:(BOOL)newVal
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceIsLocked: newVal ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceIsLocked: newVal ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCceLineWidth:(float)newLineWidth
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceLineWidth: newLineWidth ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceLineWidth: newLineWidth ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCceFillColor:(NSColor *)newFillColor
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceFillColor: newFillColor ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceFillColor: newFillColor ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCceLineColor:(NSColor *)newLineColor
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceLineColor: newLineColor ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceLineColor: newLineColor ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCceIsFilled:(BOOL)newCceIsFilled
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceIsFilled: newCceIsFilled ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceIsFilled: newCceIsFilled ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCceIsStroked:(BOOL)newCceIsStroked
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setCceIsStroked: newCceIsStroked ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setCceIsStroked: newCceIsStroked ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setOwnerView:(CoverView *)newOwnerView 
{
	NSEnumerator *enumerator = [elemArray objectEnumerator];
	CCE_BasicForm   * elem ;

	[ super setOwnerView: newOwnerView ];
	while (elem = [enumerator nextObject]) 
	{
		[ elem setOwnerView: newOwnerView ];
	}
}


// ---------------------------------------------------------------------
// accessor
- (NSMutableArray *)elemArray {
	return elemArray ;
}

- (void)setElemArray:(NSMutableArray *)newElemArray {
	if (elemArray != newElemArray) {
		[elemArray release];
		elemArray = [newElemArray retain];
	}
}

// ---------------------------------------------------------------------
- (void)
printMeWithLevel: (int) lev
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;

	[ super printMeWithLevel: lev ];
	// ciclo su tutti gli elementi del gruppo
	numElem = [ elemArray count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ elemArray objectAtIndex: i ];
		[ elem printMeWithLevel: (lev+1) ];
	}
}

@end
