// -----------------------------------------------------------------------------
//  CCE_Image.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe per l'elemento immagine

#import		"CCE_Image.h"

#import		"CoverView.h"
#import		"djZeroUtils.h"

@implementation CCE_Image

// ---------------------------------------------------------------------
// almeno le variabili d'istanza
- (id)
initWithId: (int) ident inView: (CoverView*) cv withRect: (NSRect) aRect andAngle: (float) angle
{
	self = [ super initWithId: ident inView: cv withRect: aRect andAngle: angle ] ;
	if ( self )
	{
		// imposto una immagine di default
		[ self setImage: [ NSImage imageNamed: @"NSApplicationIcon" ] ];
		// metto a posto la dimensione originale ed la trasparenza
		imgRepDim = [ image size ];
		compFraction = 1.0F ;
		// l'immagine ha le coordinate rovesciate come la view
		[ image setFlipped: YES ];
	}
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
init
{
	return( [ self initWithId: 0 inView: nil withRect: NSMakeRect(100,100, 50,50) andAngle: 0] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
dealloc
{
	[ image dealloc ];
	[ super dealloc ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rappresentazione in XML
- (NSMutableDictionary *)
getXMLDescription
{
	// variabili comuni
    NSMutableDictionary *dict = [super getXMLDescription];
    [dict setObject:[NSString stringWithFormat:@"%f", compFraction] 
				forKey: kStrElem_compFraction ];
    [dict setObject:[NSArchiver archivedDataWithRootObject: image ] 
				forKey: kStrElem_image];
	return ( dict );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// finisco di inizializzare un elemento con le proprieta' XML 
- (void)
loadElemXMLDesc:(NSDictionary *)dict 
{
	// variabili comuni
    [super loadElemXMLDesc:dict];
	compFraction = [ [dict objectForKey: kStrElem_compFraction ]
						floatValue ] ;
	[self setImage: [ NSUnarchiver unarchiveObjectWithData:
						[ dict objectForKey:kStrElem_image ] ] ] ;
	imgRepDim = [ image size ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	[ super encodeWithCoder: encoder ] ;
	if ( [encoder allowsKeyedCoding] ) 
	{		
		[ encoder encodeObject: image forKey: keyStr(objID, kStrElem_image) ] ;
		[ encoder encodeFloat: compFraction forKey: keyStr(objID, kStrElem_compFraction) ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
initWithCoder:(NSCoder *)decoder 
{
	[ super initWithCoder: decoder ] ;
    if ( [decoder allowsKeyedCoding] ) 
	{
		[ self setImage: [ decoder decodeObjectForKey: keyStr( objID, kStrElem_image) ] ];
		imgRepDim = [ image size ];
        compFraction = [decoder decodeFloatForKey: keyStr( objID, kStrElem_compFraction) ];	
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// editing dell'immagine: seleziono un nuovo file sorgente
- (void)
startEditing
{
	// tutti i tipi di immagine leggibili
	NSArray		* fileTypes = [NSImage imageFileTypes ];
	// il pannello di selezione file
	NSOpenPanel * oPanel = [NSOpenPanel openPanel];
	NSImage		* curImg  ;	// l'immagine prescelta
	NSString	* aFile ;	// il path dell'immagine
	// dico che si puo' selezionare un solo file alla volta
	[oPanel setAllowsMultipleSelection:NO];
	// apro il pannello e consento la selezione
	if ( [oPanel runModalForTypes:fileTypes] != NSOKButton) 
		return ;
	// se arrivo qui, e' stato selezionato un file
	// recupero il path del file
	aFile = [ [oPanel filenames] objectAtIndex:0];
	// costruisco l'immagine a partire dal file
	curImg = [[ NSImage alloc] initWithContentsOfFile: aFile ];
	// rovescio le coordinate, visto che anche la CoverView lo fa
	[ curImg setFlipped: YES ];
	// imposto l'elemento immagine con le nuova immagine
	[ self setImage: curImg ];
	// le nuove dimensioni
	[ self setImgRepDim: [ curImg size]];
	// forzo un ridisegno della vista
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// impostazione della dimensione originale dell'immagine
- (void)
setOriginalSize
{
	NSRect		aRect ;
	// reimposto le dimensioni dell'immagine a quelle originali
	[ self setLocalSize: imgRepDim ];
	// ricostruisco il percorso
	aRect = NSMakeRect( 0, 0, localSize.width, localSize.height );
	[ self setTheDrawPath: [NSBezierPath bezierPathWithRect: aRect ] ];
	// e ricalcolo le trasformazioni e gli handle, che son cambiati
	[ self calcLocalTranform ];
	[ self buildHdlList ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// menu contestuale, che ne ha uno tutto suo
- (NSMenu*)
contextMenu 
{
	return ( [ ownerView imgElemMenu ] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// stringa che descrive l'elemento
- (NSString*)
describeMe
{
	return ( [ NSString stringWithFormat:@"Image Elem ID %d", [ self objID] ] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// disegno dell'immagine
- (void)
specificDrawing: (NSRect) inRect
{
	// aggiusto le dimensioni per non ingenerare confusione
	NSRect  locrect = arrangeRect( localSize );
	// disegno il rettangolo e lo sfondo
	[ super specificDrawing: inRect ] ;
	// disegno l'immagine scalandola opportunamente
	[ image drawInRect: locrect
			fromRect: NSMakeRect(0,0, imgRepDim.width, imgRepDim.height) 
			operation: NSCompositeSourceOver fraction: compFraction ];
}

// ---------------------------------------------------------------------
// accessor
- (NSImage *)image {
    return image ;
}

- (void)setImage:(NSImage *)newImage {
    if (image != newImage) {
        [image release];
        image = [newImage copy];
    }
}

- (NSSize)imgRepDim {
    return imgRepDim;
}

- (void)setImgRepDim:(NSSize)newImgRepDim {
        imgRepDim = newImgRepDim;
}

- (float)compFraction {
    return compFraction;
}

- (void)setCompFraction:(float)newCompFraction {
        compFraction = newCompFraction;
}

@end
