// -----------------------------------------------------------------------------
//  CCE_Line.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe per disegnare una linea

#import		"CCE_Line.h"

#include	<math.h>
#import		"CoverView.h"

@implementation CCE_Line

// ---------------------------------------------------------------------
// inizializzione: ho bisogno di due punti
- (id)
initWithId: (int) ident inView: (CoverView*) cv fromPt: (NSPoint) stPt toPt: (NSPoint) endPt
{
	self = [ super initWithId: ident inView: cv ] ;
	if ( self )
	{
		numOfHdl = 2 ;
		[ self buildMeFromPt: stPt withSize: NSMakeSize(endPt.x-stPt.x, endPt.y-stPt.y) ];
		[ self setCceIsStroked: TRUE ];
	}
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
buildHdlList
{
	// due soli handle, che sono i punti estremi
	hdlList[0] = NSMakePoint( 0, 0 ) ;
	hdlList[1] = NSMakePoint( localSize.width, 0) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// costruzione di una linea tramite punto base ed offset del secondo
- (void)
buildMeFromPt: (NSPoint) stPt withSize: (NSSize)newSize
{
	NSBezierPath *path = [NSBezierPath bezierPath];
	// calcola la lunghezza del segmento
	float   c3 = sqrt( newSize.width*newSize.width + newSize.height*newSize.height) ;
	// calcolo l'angolo formato dal segmento col sistema di riferimento della
	// finestra, e lo trasformo in numero tra 0 e 360
	[ self setRotAngle: (180 * atan2( newSize.height, newSize.width) / M_PI) ] ;
	// il punto base e' il primo punto
	drawPoint = stPt ;
	[ self calcLocalTranform ];
	// la dimensioen della linea e' la sua lunghezza e zero.
	localSize = NSMakeSize( c3, 0) ;
	// costruisco il percorsoin coordinate locali
	[path moveToPoint: NSMakePoint(0,0) ];
	[path lineToPoint: NSMakePoint(c3,0) ];
	// lo assegno
	[ self setTheDrawPath: path ];
	// costruisco gli handle
	[ self buildHdlList ] ;
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// le linee si ridimensionano con un metodo tutto loro
- (void)
resizeMeWith: (NSSize) newSize
{
	NSSize		realSize ;
	float		aaa = rotAngle/180*M_PI ;
	// gestione dello undo del movimento
	[[self undoManager] setActionName: @"BasicForm -> resizeMe"];
	[[[self undoManager] prepareWithInvocationTarget: [ownerView theElements]]
		resizeElem: self withSize: localSize ];
	// ricalcolo la posizione del secondo punto
	realSize.width = newSize.width * cos ( aaa ) + newSize.height * sin ( aaa ) ;
	realSize.height = newSize.width * sin ( aaa ) + newSize.height * cos ( aaa ) ;
	[self buildMeFromPt: drawPoint withSize: realSize ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// disegno la linea
- (void)
specificDrawing: (NSRect) inRect
{
	// imposto il colore
	[ cceLineColor set ];
	// imposto lo spessore della linea
	[ theDrawPath setLineWidth: [ self cceLineWidth ]];
	// disengo la linea
	[ theDrawPath stroke ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// gestione della costruzione della linea
- (BOOL)
handleCreation: (NSEvent *)theEvent
{
	CoverView 	* view  = [ self ownerView];
	NSPoint 	point1 = [view convertPoint:
		[theEvent locationInWindow] fromView:nil];
	NSPoint 	point2 ;
	BOOL		shiftKeyPress ;
	NSSize		tmpSize ;
	// inizializzo l'aspetto dell'oggetto (praticamente vuoto)
	point1 = getNearestSnapPoint( point1, view);
	// inizializzo l'aspetto dell'oggetto (praticamente vuoto)
	[ self buildMeFromPt: point1 withSize: NSMakeSize(0,0) ] ;
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse
		// oppure un mouseup (che finisce il tutto)
		theEvent = [[view window] nextEventMatchingMask:
			(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
		shiftKeyPress = (([theEvent modifierFlags] & NSShiftKeyMask) ?
						YES : NO);
		// recupero la coordinata locale
		point2 = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		point2 = getNearestSnapPoint( point2, view);
		tmpSize = NSMakeSize(point2.x-point1.x, point2.y-point1.y) ;
		if ( shiftKeyPress )
		{
			// costringo il rettangolo ad essere quadrato
			float   delta1 = tmpSize.width ;
			float   delta2 = tmpSize.height ;
			float	dirx = 1, diry = 1 ;
			if ( delta1 < 0 )
			{
				delta1 = - delta1 ; dirx = -1 ;
			}
			if ( delta2 < 0 )
			{
				delta2 = - delta2 ; diry = -1 ;
			}
			// piglio la dimensione piu' corta
			if ( delta1 < delta2*0.2 )
				tmpSize.width = 0 ;
			else if ( delta2 < delta1*0.2 )
				tmpSize.height = 0 ;
			else if ( delta1 < delta2 )
				tmpSize.height = diry * delta1 ;
			else
				tmpSize.width = dirx * delta2 ;
		}
		// aggiusto l'aspetto dell'oggetto
		[ self buildMeFromPt: point1 withSize: tmpSize ] ;
		// forzo un ridisegno della finestra
		[ view setNeedsDisplay: YES ];
	}
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// gestione della modifica di una linea
- (BOOL)
handleResize: (NSEvent *)theEvent
	movingHandle: (short) hdlIdx
{
	CoverView * view  = [ self ownerView];
	NSPoint fixedPoint = ( hdlIdx == 0) ? hdlList[ 1 ] : hdlList[ 0 ];
	NSPoint movingPoint = ( hdlIdx == 0) ? hdlList[ 0 ] : hdlList[ 1 ];
	NSPoint point ;
	NSSize  newSize = NSMakeSize(movingPoint.x-fixedPoint.x, movingPoint.y-fixedPoint.y) ;
	// se e' locked, non se ne fa nulla
	if ( cceIsLocked ) return YES ;
	fixedPoint = [ localTF transformPoint: fixedPoint ];
	newSize = [ localTF transformSize: newSize ];
	// ridisegno l'oggetto con un nuovo sistema di riferimento
	[ self buildMeFromPt: fixedPoint withSize: newSize ] ;
	// mi preparo lo undo
	[[self undoManager] setActionName: @"Line -> handleResize"];
	[[[self undoManager] prepareWithInvocationTarget:
		[ownerView theElements]] resizeElem: self withSize: newSize ];
	// continuo a ciclare in attesa di un mouseup
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse
		// oppure un mouseup (che finisce il tutto)
		theEvent = [[view window] nextEventMatchingMask:
			(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
		// recupero la coordinata locale
		point = [view convertPoint:[theEvent locationInWindow] fromView:nil];
		point = getNearestSnapPoint( point, view);
		newSize = NSMakeSize(point.x-fixedPoint.x,point.y-fixedPoint.y) ;
		// aggiusto l'aspetto dell'oggetto
		[ self buildMeFromPt: fixedPoint withSize: newSize] ;
	}
	while ( [theEvent type] != NSLeftMouseUp) ;
	// arrivato qui, ho finito il tutto
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// stringa che descrive l'elemento
- (NSString*)
describeMe
{
	return ( [ NSString stringWithFormat: @"Line Elem ID %d", [ self objID] ] );
}

@end
