// -----------------------------------------------------------------------------
//  CCE_Text.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe per il disegno di testo

#import		"CCE_Text.h"

#import		"CoverView.h"
#import		"CoverWinCtl.h"
#import		"djZeroUtils.h"

@implementation CCE_Text

// ---------------------------------------------------------------------
// per inizializzare devo aver gia' pronto (e completo!) un NSTextStorage,
// e poi specificare dove disegnare e con che angolo
- (id)
initWithId: (int) ident inView: (CoverView*) cv
	withAttributedString: (NSMutableAttributedString *) theText 
	inRect: (NSRect) aRect  withAngle: (float) angle
{
	self = [ super initWithId: ident inView: cv 
					withRect: aRect  andAngle: angle] ;
	if ( self )
	{
		NSTextStorage   * loctextStorage ;
		[ theText setAttributes: defaultTextAttr( ) range: NSMakeRange(0, [theText length])];
		loctextStorage = [[NSTextStorage alloc] initWithAttributedString: theText] ;
		[ self setTextStorage: loctextStorage ];
		numOfColumns = 1 ;
		editingText = NO ;
		automaticText = CATAUTOTEXT_NULL ;
		[ self prepareTextStorage: loctextStorage ];
		[ loctextStorage release ] ;
	}
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo che occorre comunque nei casi di emergenza, per inizializzare
// le variabili d'istanza
- (id )
init
{
	self = [ self initWithId: 0 inView: nil withAttributedString: 
			[[ NSMutableAttributedString alloc] initWithString: @"##text##" ]
		inRect: NSMakeRect(0,0,1,1)  withAngle: 0 ] ;
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// distruggo tutto
- (void)
dealloc
{
	[ textStorage release ] ;
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rappresentazione in XML
- (NSMutableDictionary *)
getXMLDescription
{
	// elementi comuni
    NSMutableDictionary *dict = [super getXMLDescription];
	[dict setObject:[NSArchiver archivedDataWithRootObject: textStorage ] 
				forKey: kStrElem_textStorage ];
    [dict setObject:[NSString stringWithFormat:@"%d", automaticText] 
				forKey: kStrElem_automaticText ];
    [dict setObject:[NSString stringWithFormat:@"%d", numOfColumns] 
				forKey: kStrElem_numOfColumns ];
	return ( dict );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// finisco di inizializzare un elemento con le proprieta' XML 
- (void)
loadElemXMLDesc:(NSDictionary *)dict 
{
	// variabili comuni
    [super loadElemXMLDesc:dict];
	[self setTextStorage:[NSUnarchiver unarchiveObjectWithData:
				[dict objectForKey: kStrElem_textStorage ]]];
	[ self prepareTextStorage: textStorage ];
	automaticText = [ [dict objectForKey: kStrElem_automaticText ] 
						intValue ] ;
	numOfColumns = [ [dict objectForKey: kStrElem_numOfColumns] 
						intValue ] ;
	editingText = NO ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// salvataggio con encoder
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	[ super encodeWithCoder: encoder ] ;
	if ( [encoder allowsKeyedCoding] ) 
	{		
		// devo salvare le due variabili d'istanza fondamentali
		[ encoder encodeObject: textStorage 
					forKey: keyStr(objID, kStrElem_textStorage) ] ;
		[ encoder encodeInt: automaticText 
					forKey: keyStr(objID, kStrElem_automaticText) ] ;
		[ encoder encodeInt: numOfColumns 
					forKey: keyStr(objID, kStrElem_numOfColumns) ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
initWithCoder:(NSCoder *)decoder 
{
	[ super initWithCoder: decoder ] ;
    if ( [decoder allowsKeyedCoding] ) 
	{
		// mi ripiglio le variabili d'istanza e costruisco cio' che manca
		[ self setTextStorage: 
			[ decoder decodeObjectForKey: keyStr(objID, kStrElem_textStorage) ] ];
        automaticText = [decoder decodeIntForKey: keyStr( objID, kStrElem_automaticText) ];	
        numOfColumns = [decoder decodeIntForKey: keyStr( objID, kStrElem_numOfColumns) ];	
		[ self prepareTextStorage: textStorage ];
		editingText = NO ;
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// predispongo il textStorage per il suo corretto funzionamento
- (void)
prepareTextStorage: (NSTextStorage *) ts
{
	NSLayoutManager *layoutManager;
	// layout gestisce come il testo e' rappresentato
	layoutManager = [[NSLayoutManager alloc] init] ;
	[ ts addLayoutManager:layoutManager];
	[ self adjustTextColumn ] ;
	[ layoutManager release ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiusto la visualizzazione con le colonne
- (void)
adjustTextColumn
{
	int				i, notcAr ;
	NSLayoutManager	* lm ;
	// recupero il layout manager del textStorage
	lm = [ [ textStorage layoutManagers ] objectAtIndex: 0] ;
	// guardo quanti textContainer sono presenti
	notcAr = [[ lm textContainers ] count ];
	// li tolgo tutti, che non mi servono piu'
	for ( i = (notcAr-1); i >= 0 ; i -- )
		[ lm removeTextContainerAtIndex: i ];
	// ci aggiungo quelli nuovi in base al numero di colonne
	for ( i = 0; i < numOfColumns ; i ++ )
	{
		NSTextContainer * tc;
		// ricalcolo le dimensioni per non averne negative
		NSRect  locrect = arrangeRect( localSize );
		// costruisco un nuovo textContainer
		tc = [[NSTextContainer alloc] init ];
		// dovrei specificare un NSSize per il textcontainer
		[ tc setContainerSize: 
				NSMakeSize( locrect.size.width / numOfColumns - 2, 
							locrect.size.height) ];
		// lo aggiungo al layout manager
		[ lm addTextContainer: tc];
		[ tc release ] ;
	}
}

// ---------------------------------------------------------------------
// per disegnare il testo, un sacco di lavoro
- (void)
specificDrawing: (NSRect) inRect
{
	// determino punto e dimensione di dove disegnare
	// per tenere conto di eventuali dimensioni negative
	// disegno del testo
	NSRect  locrect = arrangeRect( localSize );
	[ super specificDrawing: inRect ] ;
	// se sto modificando il testo...
	if (  ! editingText || [NSGraphicsContext currentContextDrawingToScreen] )
	{
		int			i ;
		NSLayoutManager *layoutManager ;
		NSArray	* tcAr ;
		// recupero il layout manager
		layoutManager = [ [ textStorage layoutManagers ] objectAtIndex: 0] ;
		// recupero tutti i textContainer (sono numOfColumns)
		tcAr = [ layoutManager textContainers ] ;
		// li spazzolo tutti
		for ( i = 0 ; i < numOfColumns ; i ++ )
		{
			NSRange glyphRange  ;
			NSPoint	pt ;
			NSTextContainer * tc1 ;
			// recupero i textContainer in ordine
			tc1 = [ tcAr objectAtIndex: i ] ;
			// predispongo la loro dimensione
			[ tc1 setContainerSize: 
				NSMakeSize( locrect.size.width / numOfColumns - 2, 
							locrect.size.height) ];
			// recupero il range del testo
			glyphRange = [layoutManager glyphRangeForTextContainer: tc1 ];
			// calcolo il punto di origine del singolo textContainer
			pt = NSMakePoint(	locrect.origin.x + 
									i * locrect.size.width / numOfColumns + 2 , 
								locrect.origin.y );
			// disegno i caratteri nel singolo textContainer
			[ layoutManager drawGlyphsForGlyphRange:glyphRange 
					atPoint: pt  ];
		}
	}
	else
	{	// sto editando il testo e non sto stampando
		// e' tutto fatto dalla NSTextView
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// predisposizione allo editing del testo
- (void)
startEditing
{
	float			tmpFlt ;
	// recupero puntatori alla scrollView ed alla textView nascoste
	NSScrollView	* theScroll = [ ownerView shScroll ];
	NSTextView		* theEditText = [ ownerView shText ];
	// calcolo il rettangolo circoscritto; usero' l'origne come punto
	// base per il disegno della scrollview
	NSRect  locrect = [ [ localTF transformBezierPath: theDrawPath ] bounds ] ;
	// le dimensioni della scrollview sono quelle del testo, corrette
	// perche' siano positive per non ingenerre confusione
	locrect.size.width = ( localSize.width < 0 ) ?
					- localSize.width : localSize.width ;
	locrect.size.width /= numOfColumns ;
	locrect.size.height = ( localSize.height < 0 ) ?
					- localSize.height : localSize.height ;
	// meglio che la scrollView non esca dai bordi
	if ( locrect.origin.x < 0 ) locrect.origin.x = 5 ;
	if ( locrect.origin.y < 0 ) locrect.origin.y = 5 ;
	tmpFlt = locrect.origin.x + locrect.size.width 
			- convertMeasure(VIEW_DIM_HOR, UNITS_CM, UNITS_PIXELS) ;
	if ( tmpFlt > 0 )
		locrect.origin.x -= ( tmpFlt + 15 ); // c'e' anche la barra di scorrimento
	tmpFlt = locrect.origin.y + locrect.size.height 
			- convertMeasure(VIEW_DIM_VEL, UNITS_CM, UNITS_PIXELS) ;
	if ( tmpFlt > 0 )
		locrect.origin.y -= ( tmpFlt + 5 ); 
	// la visualizzazione funziona meglio se arrotondo a pixel interi
	locrect.origin.x = round( locrect.origin.x );
	locrect.origin.y = round( locrect.origin.y );
	// dico che questo elemento e' in fase di editing
	editingText = YES ;
	// ed aggiusto la variabile della view per la sua gestione
	[ownerView setEditingInProgress:self ];
	// dimensioni della scrollview, faccio spazio per lo scroller
	[ theScroll setFrame:  NSMakeRect(locrect.origin.x , locrect.origin.y,
				locrect.size.width + 11 , locrect.size.height ) ];
	// questa scrollview/textview e' gestita come unlayout manager aggiuntivo
	[ textStorage addLayoutManager: [ theEditText layoutManager ] ] ;
	// mi riporto in testa e seleziono tutto il testo
	[ theEditText scrollPoint: NSMakePoint( 0, 0 ) ];
	[ theEditText setSelectedRange: NSMakeRange(0, [ textStorage length ] ) ];
	// dico che la text e' il primo risponditore degli eventi
	// cosi' che tastiera e menu relativi al testo cadano qui
	[[ownerView window] makeFirstResponder:theEditText];
	// visualizzo la scrollviea
	[ theScroll setHidden: NO ] ;
	// forzo un ridisegno
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// fine delle operazioni di editing
- (void)
endEditing
{
	// recupero puntatori alla scrollView ed alla textView
	NSScrollView   * theScroll = [ ownerView shScroll ];
	NSTextView   * theEditText = [ ownerView shText ];
	// se non e' per me, lascio perdere
	if ([ownerView editingInProgress] != self)
		return ;
	// se arrivo qui, fine editing per me
	editingText = NO ;
	[ ownerView setEditingInProgress:nil ];
	// tolgo il layout manager, che non serve piu'
	[ textStorage removeLayoutManager: [ theEditText layoutManager ] ] ;
	// nascondo la scrollview
	[ theScroll setHidden: YES ] ;
	// lascio che adesso tastiera e menu insistano sulla coverview
	[ [ownerView window] makeFirstResponder:ownerView ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cambiamento del font; rimane necessario quando si intende cambiare il
// font di tutto il blocco di testo
- (void)
changeFont:(id)sender
{
	// recupero il font corrente
	NSFont  * curFont = [ textStorage font ];
	// lo converto nel nuovo font come stabilito da NSFontManager
	NSFont  * newFont = [sender convertFont:curFont];
	// imposto il nuovo font
	[textStorage setFont:newFont];
	// ridisegno tutto
	[ ownerView setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiorno il contenuto con un testo automatico
- (void)
updAutoText: (CatDataSrc *) catalog 
{
	if ( automaticText == CATAUTOTEXT_NULL )
		return ;
	switch ( automaticText ) {
	case CATAUTOTEXT_FILELIST :
		[ catalog setCACFileListFor: textStorage ] ;  
		break ;
	case CATAUTOTEXT_VOLNAME :
		[ catalog setCACAttribFor: textStorage ofName: COLID_FILENAME ] ;  
		break ;
	case CATAUTOTEXT_MODDATE :
		[ catalog setCACAttribFor: textStorage ofName: COLID_MODDATE ] ;  
		break ;
	case CATAUTOTEXT_CREATDATE :
		[ catalog setCACAttribFor: textStorage ofName: COLID_CREATDATE ] ;  
		break ;
	case CATAUTOTEXT_VOLSIZE :
		[ catalog setCACAttribFor: textStorage ofName: @"volSize" ] ;  
		break ;
	case CATAUTOTEXT_VOLFREESIZE :
		[ catalog setCACAttribFor: textStorage ofName: @"volFreeSize" ] ;  
		break ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// menu contestuale proprio dell'elemento
- (NSMenu*)
contextMenu 
{
	return ( [ ownerView txtElemMenu ] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// stringa che descrive l'elemento
- (NSString*)
describeMe
{
	return ( [ NSString stringWithFormat:@"Text Elem ID %d", [ self objID] ] );
}

// ---------------------------------------------------------------------
// accessor
- (NSTextStorage *)textStorage {
	return [[textStorage retain] autorelease];
}

- (void)setTextStorage:(NSTextStorage *)newTextStorage {
	if (textStorage != newTextStorage) {
		[textStorage release];
		textStorage = [newTextStorage retain];
	}
}

- (BOOL)editingText {
	return editingText ;
}

- (void)setEditingText:(BOOL)newEditingText {
	editingText = newEditingText ;
}
- (int)automaticText {
    return automaticText;
}

- (void)setAutomaticText:(int)newAutomaticText {
	automaticText = newAutomaticText;
}

- (int)numOfColumns {
    return numOfColumns;
}

- (void)setNumOfColumns:(int)newNumOfColumns {
        numOfColumns = newNumOfColumns;
}

@end

