// -----------------------------------------------------------------------------
//  CatDataSrc.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// Realizzazione della classe sorgente dati

#import		"CatDataSrc.h"

#import		"CdCatDoc.h"
#import		"FileStruct.h"
#import		"VolInfo.h"
#import		"ImageAndTextCell.h"
#import		"UserPrefs.h"
#import		"djZeroUtils.h"
#import		"LocalFormatters.h"

// alcune funzioni ausiliarie
// funzione che discrimina quale file saltare
BOOL		skipDotFile ( NSString * fileName ) ;
// funzione che calcola il numero di file normali
short		countNormalFiles( FileStruct * directory );
// funzione che restituisce l'i-esimo file normale di una directory
FileStruct*	getNormalFile ( FileStruct * directory, short index ) ;
// long long sembra avere die problemi
NSString *	fileSizeFromLongLong( unsigned long long fSize) ;
// cancellazione di un elemento dalla gerarchia
BOOL		deleteItemFromHierarchy ( NSMutableArray * hier , FileStruct * fItem ) ;
// funzioni per produrre una versione testo della lista file
void		addStringExpandItem ( NSTextStorage* str, NSMutableDictionary * d1, 
					CatDataSrc * myDataSrc, FileStruct * elem, int nn, int lev );
NSString *	prefixLevString( int lev );
void		terminalWithLevel ( NSTextStorage* str, NSMutableDictionary	* d1, 
					NSString *fname, int lev, BOOL boldface ) ;

@implementation CatDataSrc
// ---------------------------------------------------------------------
// inizializzazione e deallocazione: devo solo trattare il vettore
- (id )
init
{
	self = [ super init ] ;
	[ self setStartPoint: [ NSMutableArray array ]];
	[ self setOrderColumn: [ NSString stringWithString: COLID_FILENAME]];
	[ self setOrderDirection: TRUE ];
	[ self setRefDoc: nil ];
	currentVol = -1 ;
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
dealloc
{
	[ startPoint release ] ;
	[ orderColumn release ] ;
	[ refDoc release ] ;
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// devo aggiungere un oggetto al vettore
- (void)
addFileEntry: (id) newEntry
{
	[ startPoint addObject: newEntry ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// devo eliminare un elemento dal catalogo
- (void )
removeEntry: (FileStruct *) fInfo
{
	//considero ricorsivamente gli start points e vado
	deleteItemFromHierarchy( startPoint, fInfo );
	// ovviamente, il documento e' stato sporcato
	[ refDoc updateChangeCount: NSChangeDone ] ;
	// rinfresco il contenuto delle finestre
	[ refDoc refreshWinsWithListSts: nil coverSts: nil ];
}

// ---------------------------------------------------------------------
// per la visualizzazione della lista dei volumi
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodi che devono essere realizzati da dataSource
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (int)
numberOfRowsInTableView:(NSTableView *)tv
{
	// numero di volumi
	return ( [ startPoint count]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
tableView:(NSTableView *)tv objectValueForTableColumn:(NSTableColumn *)tc row:(int)row
{
	// c'e' sempre e solo una sola colonna
	// predispongo l'immagine da mostrare
	[ [tc dataCell] setImage: [[startPoint objectAtIndex: row] fileIcon ] ];
	// restituisco il volume i-esimo
	return ( [ [startPoint objectAtIndex: row ] fileName ]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (BOOL)
tableView:(NSTableView *)tv shouldEditTableColumn:(NSTableColumn *)tc row:(int)row
{
	return ( NO );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodi che devono essere realizzati dalla classe delegata
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// e' cambiata la selzione corrente della tabella
- (void)
tableViewSelectionDidChange:(NSNotification *)aNotification
{
	NSTableView		* ov = (NSTableView*)[ aNotification object ] ;
	CoverView		* cv = [[refDoc coverWin] cvrView] ;
	// predispongo il volume in selezione corrente
	currentVol = [ ov selectedRow ] ;
	[ [[cv theElements] elemArray] makeObjectsPerformSelector: 
			@selector(updAutoText:) withObject: self ] ;
	// forzo un rinfresco della vocerView
	[ cv setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCACFileListFor: (NSTextStorage *) txtSto  
{
	NSTextStorage   * newFileList ;
	int				elNum, jj, nn ;
	NSMutableDictionary	* curStyle ;
	if ( currentVol == -1 ) 
	{
		newFileList = [[NSTextStorage alloc] initWithString:@"##File List##"];
	}
	else
	{
		// costruisco un nuovo contenitore per il testo
		newFileList = [[NSTextStorage alloc] initWithString:@""];
	}
	// estraggo dal testo corrente font e paragrafo
	curStyle = [ NSMutableDictionary dictionaryWithCapacity: 2 ] ;
	if ( [txtSto length] > 0 )
	{
		NSDictionary	* dd ;
		dd = [ txtSto fontAttributesInRange: NSMakeRange(0, [txtSto length]) ] ;
		[ curStyle addEntriesFromDictionary: dd ] ;
		dd = [ txtSto rulerAttributesInRange: NSMakeRange(0, [txtSto length]) ] ;
		[ curStyle addEntriesFromDictionary: dd ];	
	}
	else
	{
		[ curStyle addEntriesFromDictionary: defaultTextAttr( ) ];
	}
	// recupero il primo livello del volume
	if ( currentVol == -1 )
	{
		// aggiorno la stringa
		[ txtSto setAttributedString:  newFileList ] ;
		return ;
	}
	// se arrivo qui, ho un vero e proprio volume
	elNum = [ self outlineView: nil numberOfChildrenOfItem: nil ];
	// esploro il primo livello
	for ( jj = 0 ; jj < elNum; jj ++ )
	{
		FileStruct  * elem ;
		// esamino tutti gli elementi uno ad uno
		elem = [ self outlineView:	 nil child: jj ofItem: nil ];
		// controllo se ha figli
		nn =  [ self outlineView: nil numberOfChildrenOfItem: elem ];
		// se non e' da stampare, salto
		if ( ! [ elem is2Print ] ) continue ;
		// se e' una directory, espando di un livello
		if (nn > 0 )
			addStringExpandItem (newFileList, curStyle, self, elem, nn, 0 );
		else	// lo aggiungo al testo gia' predisposto
			terminalWithLevel( newFileList, curStyle, [ elem fileName], 0, NO );
	}
	// aggiorno la stringa
	[ txtSto setAttributedString:  newFileList ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setCACAttribFor: (NSTextStorage *) txtSto ofName: (NSString*) colId
{
	VolInfo			* localItem ;
	NSTextStorage   * newFileList ;
	NSString		* localString ;
	id				valloasape ;
	NSMutableDictionary	* curStyle ;
	// costruisco un nuovo contenitore per il testo, che calcolo subito
	if ( currentVol == -1 ) 
	{
		localString = [	NSString stringWithFormat: @"##%@##", colId ] ;
	}
	else  
	{
		localItem = [ startPoint objectAtIndex: currentVol ] ;
		valloasape = [ localItem valueForKey: colId  ] ;
		if ( [ colId isEqual: @"volSize" ] || [ colId isEqual: @"volFreeSize" ] )
		{
			FileSizeForm	* numFmt ;
			long long	tmp ;
			numFmt = [ [ FileSizeForm alloc ] init ] ;
			tmp = [ valloasape longValue ] ;
			localString = [ numFmt stringForObjectValue: [ NSNumber numberWithLongLong: tmp ] ] ;
		}
		else localString = [ NSString stringWithString: [valloasape description ] ] ;
	}
	newFileList = [[NSTextStorage alloc] initWithString: localString ];
	// adotto il font corrente
	curStyle = [ NSMutableDictionary dictionaryWithCapacity: 2 ] ;
	if ( [txtSto length] > 0 )
	{
		NSDictionary	* dd ;
		dd = [ txtSto fontAttributesInRange: NSMakeRange(0, [txtSto length]) ] ;
		[ curStyle addEntriesFromDictionary: dd ] ;
		dd = [ txtSto rulerAttributesInRange: NSMakeRange(0, [txtSto length]) ] ;
		[ curStyle addEntriesFromDictionary: dd ];	
	}
	else
	{
		[ curStyle addEntriesFromDictionary: defaultTextAttr( ) ];	
	}
	[ newFileList setAttributes: curStyle range: NSMakeRange(0, [newFileList length])];
	// aggiorno la stringa
	[ txtSto setAttributedString:  newFileList ] ;
}

// ---------------------------------------------------------------------
// per la visualizzazione della lista dei file
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodi che devono essere realizzati da dataSource
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// numero di figli di un dato elemento
- (int)
outlineView:			(NSOutlineView *)outlineView
	numberOfChildrenOfItem:	(id)item
{
	VolInfo * localItem = item ;
	// se sto recupernado dati per la CoverCtlWin
	if ( (outlineView  == nil) || ([ outlineView tag] == TAG_CDCATDOC_FILELIST) )
	{
		// se non c'e' alcuna selezione, devo mostrare nulla
		if ( currentVol == -1 )
			return ( 0 ) ;
		// se arrivo qui, c'e' un elemento selezionato
		// se l'item e' nil, stiamo parlando della radice
		if (item == nil)
			// salto di un livello proponendo il volume come radice
			localItem = [ startPoint objectAtIndex: currentVol ];
		// altrimenti e' un elemento standard
		else localItem = item ;
		// da qui si prosegue normalmente
	}
	else if ( [ outlineView tag] == TAG_CDCATDOC_FULLLIST )
	{
		// se l'item e' nil, stiamo parlando della radice
		if (item == nil)
			// dico quindi che ci sono tanti elementi quanti presenti nel vettore
			return( [ [self startPoint] count ]);
		// imposto l'elemento radice come quello corrente
		localItem = item ;
	}
	// da qui in poi, la trattazione e' la stessa
	// se arrivo qui, mi si chiede quali figli ha un file
	// tratto subito i file normali, che non hanno figli
	if ( [localItem numOfFiles] == 0 )
		return ( 0 ) ;
	// se arrivo qui, ho una directory
	// se devo mostrare anche i dotfiles, conto tutto
	if ( [[ UserPrefs getPrefValue: keyShowDotFiles] boolValue] )
		return ( [localItem numOfFiles] ) ;
	// se arrivo qui, devo eliminare dal computo i dotFiles
	// ed alora, mi tocca esaminare tutti i file e contare quelli
	// che mi interessano
	return ( countNormalFiles( localItem ) ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// con questo metodo dico se item e' espandibile
- (BOOL)
outlineView:			(NSOutlineView *)outlineView
	isItemExpandable:	(id)item
{
	// se sto recupernado dati per la CoverCtlWin
	if ( (outlineView  == nil) || ([ outlineView tag] == TAG_CDCATDOC_FILELIST) )
	{
		// se non c'e' alcuna selezione, devo mostrare nulla
		if ( currentVol == -1 )
			return ( NO ) ;
		// se arrivo qui, c'e' un elemento selezionato
		if (item == nil)
		{
			VolInfo * curVol ;
			// mi si chiede se si puo' espandere.. dipende da
			// quanti elementi ci sono all'interno del volume
			// (che succede se non ci sono elementi?)
			curVol = [ startPoint objectAtIndex: currentVol ];
			return ( ([curVol numOfFiles] == 0) ? NO : YES );
		}
		// altrimenti, e' un elemento standard
	}
	else if ( [ outlineView tag] == TAG_CDCATDOC_FULLLIST )
	{
		if (item == nil)
			return ( YES ) ;
	}
	// se non ha figli, e' un file, non si espande
	if ( [item numOfFiles] == 0 )
		return ( NO ) ;
	// se arrivo qui, l'item ha figli
	// se devo espandere anche i bundle, espando tutto
	if (  [[ UserPrefs getPrefValue:keyExpandBundle] boolValue] )
		return ( YES ) ;
	// se arrivo qui, non devo espandere i bundle
	// espando allora i non bundle
	return (! checkIfBundleDirectory([ item fileName ]) ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// con questo metodo dico qual e' il figlio "index-simo" di item
- (id)
outlineView:		(NSOutlineView *) outlineView
	child:		(int) index
	ofItem:		(id) item
{
	VolInfo * localItem = item ;
	// se sto recupernado dati per la CoverCtlWin
	if ( (outlineView  == nil) || ([ outlineView tag] == TAG_CDCATDOC_FILELIST) )
	{
		// se non c'e' alcuna selezione, devo dire nulla
		if ( currentVol == -1 )
			return ( nil ) ;
		// c'e' un elemento selezionato
		if (item == nil)
				localItem = [ startPoint objectAtIndex: currentVol ];
		else	localItem = item ;
	}
	else if ( [ outlineView tag] == TAG_CDCATDOC_FULLLIST )
	{
		if (item == nil)
		{
			NSArray			* newArray ;
			NSMutableArray  * tmpArray ;
			// ordino gli elementi secondo criterio
			if ( [ self orderDirection ] )
					newArray = [ [self startPoint] sortedArrayUsingFunction: ascSortFunc
						context: [ self orderColumn]] ;
			else	newArray = [ [self startPoint] sortedArrayUsingFunction: desSortFunc
						context: [ self orderColumn]] ;
			// trasformo il risultato in un NSMutableArray
			tmpArray = [ NSMutableArray arrayWithCapacity: [ newArray count]];
			[ tmpArray addObjectsFromArray: newArray ];
			return( [ tmpArray objectAtIndex: index ]);
		}
		else localItem = item ;
	}
	// negli altri casi, recupero la lista
	[ localItem sortFileList: [ self orderColumn] direction: [ self orderDirection ] ];
	// se visualizzo anche i dotFiles, non c'e' problema
	if ( [[ UserPrefs getPrefValue:keyShowDotFiles] boolValue] )
		return( [localItem getFileAtIndex:index]);
	// altrimenti, e' un bel pasticcio, devo saltare i dotfiles
	return ( getNormalFile ( localItem, index ) );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// con questo metodo dico cosa deve mostrare item nella colonna indicata
- (id)
outlineView:				(NSOutlineView *) outlineView
	objectValueForTableColumn:	(NSTableColumn *) tableColumn
	byItem:				(id)item
{
	NSString * colId  ;
	// se l'oggetto e' vuoto, c'e' qualche problema...
	if (item == nil)
	{
		NSLog( @"Error getting data for the catalog");
		return ( @"????" ) ;
	}
	// recupero l'identificatore della colonna
	colId = [ tableColumn identifier] ;
	// se devo mostrare il nome del file, assegno l'icona
	if ( [colId isEqualToString: COLID_FILENAME] )
	{
		// ricavo la cella ed imposto l'immagine
		[((ImageAndTextCell*) [tableColumn dataCell]) setImage: [item fileIcon] ];
	}
	// se devo mostrare la dimensione del file, tratto il long long a parte
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		long long	tmp ;
		tmp = [ item fileSize ] ;
		return ( [ NSNumber numberWithLongLong: [ item fileSize ]] );
	}
	// in tutti gli altri casi, uso il metodo valueForKey
	return ( [ item valueForKey: colId ] ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
outlineView:(NSOutlineView *) oView
	setObjectValue:(id)object
	forTableColumn:(NSTableColumn *)tableColumn
	byItem:(id)item
{
	// la colonna is2Print e' manipolabile in entrambe le outlineView
	if ( [[ tableColumn identifier] isEqualToString: COLID_ADD2PRINT ] )
	{
		BOOL    state = [ item is2Print];
		[ item setIs2Print: ! state ] ;
		// qui dovrei fare in modo di dire che il documento e' cambiato
		[ [ self refDoc ] updateChangeCount: NSChangeDone ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodi che devono essere realizzati dalla classe delegata
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// nessuna colonna e' potenzialmente modificabile
- (BOOL)
outlineView:			(NSOutlineView *) outlineView
	shouldEditTableColumn:	(NSTableColumn *) tableColumn
	item:			(id) item
{
	return NO;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo invocato da un clic sulla colonna
- (BOOL)
outlineView:(NSOutlineView *) oView
	shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// imposto il contenuto della cella all'ultimo momento
- (void)
outlineView: (NSOutlineView *) oView
	willDisplayCell:(id)aCell
	forTableColumn:(NSTableColumn *)aTableColumn
	item:(id)item
{
	// solo per la visualizzazione del pulsante di spunta
	if ( [[ aTableColumn identifier] isEqualToString: COLID_ADD2PRINT ] )
	{
		BOOL    state = [ item is2Print];
		[aCell setState: ( state ? NSOnState: NSOffState) ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo invocato quando cambia l'elemento selezionato nella outlineView
- (void)
outlineViewSelectionDidChange:(NSNotification *)notification
{
	FileStruct		* currsel ;
	NSString		* statusStr ;
	NSOutlineView   * ov ;
	// se sto visualizzando la outlineView della ListWinCtl
	ov = (NSOutlineView*)[ notification object ] ;
	if ( [ ov tag] == TAG_CDCATDOC_FULLLIST )
	{
		// e' cambiata la selezione
		int cs = [ ov selectedRow ] ;
		if ( cs == -1 )
		{
			// imposto il testo della barra di stato
			statusStr = [ NSString stringWithString: @"No Volume selected"];
			[ refDoc refreshWinsWithListSts: statusStr coverSts: nil ];
			return ;
		}
		// se arrivo qui, c'e' qualcosa di selezionato
		currsel = [ ov itemAtRow: cs ];
		// se l'elemento selezionato e' uno dei volumi
		if ([ startPoint indexOfObject: currsel] != NSNotFound)
		{
			// imposto la stringa con info utili
			unsigned long long  d1, d2, d3 ;
			VolInfo   * curVol ;
			curVol = (VolInfo *) currsel ;
			d1 = [ curVol volSize];
			d2 = [ curVol volFreeSize] ;
			d3 = [ curVol fileSize] ;
			statusStr = [ NSString stringWithFormat: @"Volume size: %@ Used: %@ Free %@",
				fileSizeFromLongLong( d1), fileSizeFromLongLong( d3), fileSizeFromLongLong(d2) ];
		}
		else	statusStr = [ NSString stringWithFormat: @"Full path: %@", [ currsel fileFullPath]];
		// rinfresco le finestre associate alla dataSource
		[ refDoc refreshWinsWithListSts: statusStr coverSts: nil ];
	}
}

// ---------------------------------------------------------------------
// partenza della ricerca/filtraggio degli elementi del catalogo
- (void)
critSelect: (FltElemPtr) fltCrit withLogic: (int) logic 
		oper: (int) operaz placeIn: (NSMutableArray *) where
{
	// piglio un elenco di tutti i volumi presenti
	NSEnumerator 	* enumerator = [[self startPoint] objectEnumerator];
	FileStruct	* item ;
	// li esamino uno ad uno
	while (item = [enumerator nextObject])
	{
		// e faccio una ricerca in profondita' sugli elementi contenuti
		[ item critSelect: fltCrit withLogic: logic oper: operaz placeIn:  where ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo che espande gli elementi di un outlineView fino all'elemento
// indicato
- (void)
expandDStoItem: (FileStruct*) expItem inDocument: (CdCatDoc *) currDoc
{
	// recupero il path completo dell'elemento obiettivo
	NSString	* fPath = [expItem fileFullPath] ;
	// lo spezzo ordinamente nei vari elementi
	NSEnumerator 	* pathComps = [[fPath pathComponents] objectEnumerator];
	// piglio anche la lista dei volumi
	NSEnumerator 	* listavolumi = [[self startPoint] objectEnumerator];
	FileStruct	* item ;
	// pulisco i primi elementi del percorso completo dell'obiettivo
	NSString	* name = [pathComps nextObject] ;	// "/"
	name = [pathComps nextObject] ;			// "Volume"
	name = [pathComps nextObject] ;			// nome volume

	// cerco nella lista dei volumi il volume che mi serve
	while (item = [listavolumi nextObject])
	{
		// quando ho trovato il volume...
		if ( [[item fileName] isEqual:  name] )
		{
			NSEnumerator 	* enumerator = [[ currDoc windowControllers ] objectEnumerator];
			NSWindowController  * winCtl ;
			// finche' ci sono controllori
			while ( winCtl = [enumerator nextObject] )
			{
				if ( [winCtl isMemberOfClass:[ListWinCtl class]] )
				{
					NSOutlineView   * locOV ;
					int				thisRow ;
					// recupero la vista principale
					locOV = [((ListWinCtl *)winCtl) fullList ] ;
					thisRow = [locOV rowForItem: item];
					// dico che questo item va espanso perche' fa parte della
					// catena che porta all'obiettivo
					[ locOV expandItem: item ];
					// lo seleziono
					[ locOV selectRow: thisRow  byExtendingSelection: FALSE ];
					// e lo porto in bella vista
					[ locOV scrollRowToVisible: thisRow ];
					// poi proseguo nell'espansione
					[ item expandFStoItem: pathComps inOutlineView: locOV] ;
					[ locOV collapseItem: expItem ];
				}
			}
			return ;
		}
	}
}

// ---------------------------------------------------------------------
// accessor
- (NSMutableArray *)startPoint {
	return [[startPoint retain] autorelease];
}

- (void)setStartPoint:(NSMutableArray *)newStartPoint {
	if (startPoint != newStartPoint) {
		[startPoint release];
		startPoint = [newStartPoint retain];
	}
}

- (NSString *)orderColumn {
	return [[orderColumn retain] autorelease];
}

- (void)setOrderColumn:(NSString *)newOrderColumn {
	if (orderColumn != newOrderColumn) {
		[orderColumn release];
		orderColumn = [newOrderColumn copy];
	}
}

- (BOOL)orderDirection {
	return orderDirection;
}

- (void)setOrderDirection:(BOOL)newOrderDirection {
	orderDirection = newOrderDirection ;
}

- (CdCatDoc *)refDoc {
	return [[refDoc retain] autorelease];
}

- (void)setRefDoc:(CdCatDoc *)newRefDoc {
	if (refDoc != newRefDoc) {
		[refDoc release];
		refDoc = [newRefDoc retain];
	}
}

@end

// ---------------------------------------------------------------------
// funzioni ausiliari alla gestione della sorgente dati
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// funzione che dice se il file va bene o meno
BOOL
skipDotFile ( NSString * fileName )
{
	const char	* fn ;		// nome del file in C
	// trasformo il nome del file in una stringa C
	fn = [ fileName cString] ;
	// perche' poi cosi' escludo quelli che cominciano con '.'
	if (*fn == '.' )
		return ( TRUE ) ;
	// escludo anche i file che si chiamano "Icon\r"
	// non chiedete il perche' delle '\r'
	if ( [ fileName isEqual: @"Icon\r"] )
		return ( TRUE ) ;
	// ... altre condizioni di esclusione
	return ( FALSE );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// conto quanti file "normali" sono presenti in una directory
// per file normali si intendono file il cui nome non comincia con '.'
// file che non si chiamano "Icon" o in qualche altro modo noto
short
countNormalFiles( FileStruct * directory )
{
	short	i, count ;

	count = 0 ;	// all'inizio, nessuno
	for ( i = 0 ; i < [directory numOfFiles] ; i++ )
	{
		// se devo saltarlo, vado avanti senza incrementare il contatore
		if ( skipDotFile( [[directory getFileAtIndex:i] fileName ] ) )
			continue ;
		// se arrivo qui, e' un file normale, incremento il contatore
		count += 1 ;
	}
	return ( count );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per riconoscere se una directory e' un bundle, non ho altre possibilita'
// che confrontare l'estensione del file...
BOOL
checkIfBundleDirectory( NSString * fileName )
{
	// per ricavare l'estensione, c'e' un metodo apposito
	if ( [ fileName hasSuffix: @".app"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".pkg"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".mpkg"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".bundle"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".framework"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".nib"] )
		return ( YES );
	return ( NO );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// restituisce l'i-esimo file normale all'interno di una directory
// saltando appunto i dotFiles
FileStruct *
getNormalFile ( FileStruct * directory, short index )
{
	short	i, count ;

	count = 0 ;	// contatore corrente
	for ( i = 0 ; i < [directory numOfFiles] ; i++ )
	{
		FileStruct * currFile ;
		// recupero il file
		currFile = [directory getFileAtIndex:i] ;
		// se devo saltarlo, passo avanti
		if ( skipDotFile( [ currFile fileName ] ) )
			continue ;
		// sono arrivato all'indice richiesto ?
		if ( count == index )
			// si, restituisco l'elemento corrente
			return ( currFile );
		// se arrivo qui, non ho ancora raggiunto l'elemento
		count += 1 ;
	}
	// ovviamente, qui non dovrei mai arrivare...
	return ( nil) ;
}

// ---------------------------------------------------------------------
// costruisce una stringa di dimensione file a partire dal numero di byte
NSString *
fileSizeFromLongLong(unsigned long long fSize )
{
	unsigned long long	fSizeK ;
	float 			fsizeM, fsizeG ;

	// se il file e' piccolo, mostro byte
	if ( fSize < 1024 )
	{
		long	tmp = fSize ;
		return ( [ NSString stringWithFormat: @" %4d b", tmp] );
	}
	// la dimensione e' in byte, divido per 1024 ed arrotondo, ottengo K
	fSizeK = (long) (( fSize / 1024.0 ) + 0.5 );
	// se il file e' medio, mostro K
	if ( fSizeK < 1024 )
	{
		long	tmp = fSizeK ;
		return ( [ NSString stringWithFormat: @" %4d K", tmp] );
	}
	// se inferiore al giga, restituisco mega con tre decimali
	fsizeM = ( fSizeK / 1024.0 ) ;
	if ( fsizeM < 1024 )
		return ( [ NSString stringWithFormat: @" %6.3f M", fsizeM] );
	// negli altri casi, ritorno Giga, con tre cifre decimali
	fsizeG = ( fsizeM / 1024.0 ) ;
	return ( [ NSString stringWithFormat: @" %6.3f G", fsizeG] );
}

// ----------------------------------------------------------------------------------------
// cancella un elemento dalla lista file esplorando ricorsivamente l'albero
BOOL
deleteItemFromHierarchy ( NSMutableArray * hier , FileStruct * fItem )
{
	int 		numOfItem, i;

	// se lo item e' nell'array, bene
	if ([  hier containsObject: fItem ])
	{
		// lo rimuovo direttamente e dico che ho finito
		[ hier removeObject: fItem ] ;
		return ( TRUE );
	}
	// non c'e', lo cerco in tutti i suoi figli
	numOfItem = [ hier count ] ;	// conto i figli e ciclo
	for ( i = 0 ; i < numOfItem ; i ++ )
	{
		// questo e' l'elemento corrente
		FileStruct * locRoot = [ hier objectAtIndex: i ];
		// cerco ricorsivamente l'elemento da cancellare
		if ( deleteItemFromHierarchy ( [locRoot fileList] , fItem ) )
			return ( TRUE );
	}
	// se arrivo qui (ma non dovrebbe succedere), non ho ancora trovato il file
	return ( FALSE );
}

// ----------------------------------------------------------------------------------------
// esamina un livello di gerarchia aggiungendo la stringa col nome dei file
void
addStringExpandItem (  NSTextStorage* str, NSMutableDictionary * curStyle, CatDataSrc * myDataSrc, FileStruct * elem, int nn, int lev )
{
	int			jj, nnx ;
	// intanto aggiungo il nome della directory alla stringa
	terminalWithLevel( str, curStyle, [ elem fileName], lev, YES );
	// poi esamino il contenuto della directory
	for ( jj = 0 ; jj < nn; jj ++ )
	{
		FileStruct  * elemx ;
		// ricavo l'elemento corrente
		elemx = [ myDataSrc outlineView: nil child: jj ofItem: elem ];
		// guardo se e' una directory
		nnx =  [ myDataSrc outlineView: nil numberOfChildrenOfItem: elemx ];
		// se non partecipa alla giostra, si salta tutto
		if ( ! [ elemx is2Print ] ) continue ;
		// se e' una directory, si scende di un livello
		if ( nnx > 0 )
		{
			addStringExpandItem ( str, curStyle, myDataSrc, elemx, nnx, lev+1 );
		}
		else
		{
			// si aggiunge il nome del file alla stringa
			terminalWithLevel( str, curStyle, [ elemx fileName], lev+1, NO );
		}
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiunge una stringa all'elenco dei file
void
terminalWithLevel ( NSTextStorage* str, NSMutableDictionary * curStyle, NSString *fname, int lev, BOOL boldface )
{
	NSString		* xxx ;
	NSMutableAttributedString  * temp ;
	NSFont			* locFont ;
	NSMutableDictionary	* tmpD = [ NSMutableDictionary dictionaryWithDictionary: curStyle ] ;

	// costruisco la stringa col nome del file
	xxx = [ NSString stringWithFormat: @"%@%@\n", prefixLevString( lev ), fname ] ;
	// la trasformo in una AttributedString
	temp = [ [ NSMutableAttributedString alloc] initWithString: xxx ];
	// recupero il font attualmente in uso
	locFont =  (NSFont*) [ tmpD objectForKey: NSFontAttributeName ] ;
	// se il caso, aggiungo il tratto Grassetto alla stringa
	if ( boldface )
		locFont = [ [NSFontManager sharedFontManager] convertFont: locFont toHaveTrait:  NSBoldFontMask];
	// assegno gli attributi alla stringa
	[ tmpD setObject: locFont forKey:NSFontAttributeName];
	[ temp setAttributes: tmpD range: NSMakeRange(0, [temp length])];
	// e poi l'aggiungo all'ambaradan
	[ str appendAttributedString: temp ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// produce una stringa per stampe con livello di profondita'
NSString *
prefixLevString( int lev )
{
	int			 i;
	char		blankChar[40];
	// se ci sono troppi livelli, tengo buono il massimo
	if ( lev > 10 ) lev = 10 ;
	// tanto per aggiungere spazi
	for ( i = 0; i < lev ; i ++ )
		blankChar[i]= '\t';
	blankChar[lev] = 0 ;
	// ne faccio una stringa
	return ( [ NSString stringWithCString: blankChar] );
}
