// -----------------------------------------------------------------------------
//  CatFileInfo.h
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// Interfaccia della classe che modella le informazioni relative ad un file

#import 	<Cocoa/Cocoa.h>

// nomi delle colonne dei dati
#define		COLID_FILENAME		@"fileName"
#define		COLID_MODDATE		@"modDate"
#define		COLID_CREATDATE		@"creatDate"
#define		COLID_FILESIZE		@"fileSize"
#define		COLID_GROUPNAME		@"ownGroupName"
#define		COLID_OWNERNAME		@"ownerName"
#define		COLID_POSIXPERM		@"filePosixPerm"
#define		COLID_OSCREATOR		@"creatorCode"
#define		COLID_OSTYPE		@"typeCode"
#define		COLID_ADD2PRINT		@"is2Print"

@interface 	CatFileInfo : NSObject {
	// il solo nome del file, pronto per tutti gli usi
	NSString  		*fileName ;
	// data di ultima modifica del file
	NSDate 			*modDate ;		// NSFileModificationDate
	NSDate 			*creatDate ;	// NSFileCreationDate
	// dimensione in byte del file
	unsigned long long	fileSize;	// NSFileSize
	// nome del gruppo di utenti proprietari del file
	NSString 		*ownGroupName ;	// NSFileGroupOwnerAccountName
	// utente proprietario del file
	NSString 		*ownerName ;	// NSFileOwnerAccountName
	// permessi di accesso secondo lo standard Posix
	unsigned long	filePosixPerm ;	// NSFilePosixPermissions
	// codice CREATOR come in MacOS9
	OSType			creatorCode ;	// NSFileHFSCreatorCode
	// codice TYPE come in MacOS9
	OSType			typeCode ;		// NSFileHFSTypeCode
	// per stamparla o meno
	BOOL			is2Print ;		// definito dall'utente
	// icona del file
	NSImage			* fileIcon ;
	// caratteristiche che non saranno visualizzate
	// nome completo di path del file
	NSString  		*fileFullPath ;
	// tipo di file (normale, directory, eccetera)
	NSString 		*fileType ;		// NSFileType
}

// inizializzazione dell'istanza in base al nome completo
- (id)		initWithPath:		(NSString*) fullPath ;
- (void)	dealloc ;

// metodi per il salvataggio/recupero su file
- (void)	encodeWithCoder:	(NSCoder *)encoder ;
- (id)		initWithCoder:		(NSCoder *)decoder ;

// accessor
- (NSString *)  fileName;
- (void)		setFileName:(NSString *)newFileName;
- (NSDate *)	modDate;
- (void)		setModDate:(NSDate *)newModDate;
- (NSDate *)	creatDate;
- (void)		setCreatDate:(NSDate *)newCreatDate;
- (NSString *)  ownGroupName;
- (void)		setOwnGroupName:(NSString *)newOwnGroupName;
- (NSString *)  ownerName;
- (void)		setOwnerName:(NSString *)newOwnerName;
- (OSType)		creatorCode;
- (void)		setCreatorCode:(OSType)newCreatorCode;
- (OSType)		typeCode;
- (void)		setTypeCode:(OSType)newTypeCode;
- (BOOL)		is2Print;
- (void)		setIs2Print:(BOOL)newIs2Print;
- (NSImage *)   fileIcon;
- (void)		setFileIcon:(NSImage *)newFileIcon;
- (NSString *)  fileFullPath;
- (void)		setFileFullPath:(NSString *)newFileFullPath;
- (NSString *)  fileType;
- (void)		setFileType:(NSString *)newFileType;
- (unsigned long long)fileSize;
- (void)		setFileSize:(unsigned long long)newFileSize;
- (unsigned long)filePosixPerm;
- (void)		setFilePosixPerm:(unsigned long)newFilePosixPerm;

@end
