// -----------------------------------------------------------------------------
//  ChooseVolCtrl.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della Finestra di scelta di volume

#import 	"ChooseVolCtrl.h"

#import		"ImageAndTextCell.h"

@implementation ChooseVolCtrl

// ---------------------------------------------------------------------
- (id )
init
{	// carico la finestra dal nib
	self = [ self initWithWindowNibName: @"chooseVolume" ];
	// inizializzo i vettori con i volumi
	[ self setVolumeNames: [NSMutableArray array] ];
	[ self setVolumePaths: [NSMutableArray array] ];
	[ self setVolumeIcons: [NSMutableArray array] ];
	return ( self );
}
 
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
dealloc
{
	// devo buttar via i vettori
	[volumeNames release];
	[volumePaths release];
	[volumeIcons release];
	[super dealloc];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
windowDidLoad
{
	NSTableColumn *tableColumn = nil;
	ImageAndTextCell *imageAndTextCell = nil;

	// metto a posto l'elenco dei volumi presenti
	[self checkLocalVolumes ];
	// attenzione a quale notification center attaccarsi!!
	// dopo di che, dico che mi segnalino operazioni coi volumi
	[[ [NSWorkspace sharedWorkspace] notificationCenter] addObserver: self
		selector: @selector( situationChanged: )
		name: NSWorkspaceDidMountNotification object: nil ] ;
	[[ [NSWorkspace sharedWorkspace] notificationCenter] addObserver: self
		selector: @selector( situationChanged: )
		name: NSWorkspaceDidUnmountNotification object: nil ] ;
	// l'unica colonna ha come cella una ImageAndTextCell
	tableColumn = [listaVolumi tableColumnWithIdentifier: @"listaVol"];
	// creo l'oggetto autorelease perche' passa in carico alla tableColumn
	imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
	[imageAndTextCell setEditable: NO];
	[tableColumn setDataCell:imageAndTextCell];
	[ listaVolumi reloadData ];
}

// ---------------------------------------------------------------------
// quando l'utente fa clic sopra il pulsante Cancel
- (IBAction)
cancelButton:	(id)sender
{
	// annullo l'abbonamento
	[[ [NSWorkspace sharedWorkspace] notificationCenter] removeObserver: self ];
	// butto via la finestra
	[[self window] setIsVisible: FALSE ];
	// ho finito di lavorare modale con la sheet
	// il codice di ritorno e' -1, ovvero, nessuna selezione
	[NSApp endSheet: [self window] returnCode: -1];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
selectButton:	(id)sender
{
	// annullo l'abbonamento
	[[ [NSWorkspace sharedWorkspace] notificationCenter] removeObserver: self ];
	// butto via la finestra
	[[self window] setIsVisible: FALSE ];
	// ho finito di lavorare modale con la sheet
	// il codice di ritorno e' la riga selezionata
	[NSApp endSheet: [self window] returnCode: [ listaVolumi selectedRow ]  ];
}

// ---------------------------------------------------------------------
// metodo che aggiorna la lista dei volumi, con nome e icona
- (void)
checkLocalVolumes
{
	NSString	* volPath ;
	// recupero l'elenco dei volumi correntemente montati
	NSArray		* tmp = [ [NSWorkspace sharedWorkspace] mountedLocalVolumePaths ] ;
	// ne faccio un enumerator
	NSEnumerator 	* enumerator = [tmp objectEnumerator];
	// costruisco tre vettori per tenere nomi ed icone
	NSMutableArray	* tmpVolNames = [NSMutableArray array] ;
	NSMutableArray	* tmpVolPaths = [NSMutableArray array] ;
	NSMutableArray	* tmpVolIcons = [NSMutableArray array] ;

	// ciclo sui volumi correntemente montati
	while ((volPath = [enumerator nextObject]))
	{
		// recupero l'icona del volume
		NSImage		* tmpImg ;
		tmpImg = reduceImageToIcon( [ [ NSWorkspace sharedWorkspace] iconForFile: volPath ] ) ;		// la riduco ai minimi termini
		[ tmpVolIcons addObject: tmpImg ];	// aggiungo l'icona al vettore
		[ tmpVolPaths addObject: volPath ];	// aggiungo il path al vettore
		// aggiungo il nome del volume al vettore
		[ tmpVolNames addObject: [[ NSFileManager defaultManager] displayNameAtPath: volPath] ];
	}
	// aggiorno i vettori d'istanza coi nomi ed icone
	[ self setVolumeNames: tmpVolNames ];
	[ self setVolumePaths: tmpVolPaths ];
	[ self setVolumeIcons: tmpVolIcons ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo invocato quando e' cambiata la situazione dei volumi montati
- (void )
situationChanged: ( NSNotification *) notification
{
	// in realta', nella notifica, c'e' scritto cosa e' successo...
	// ma e' per me piu' comodo aggiornare da zero la lista dei volumi
	[ self checkLocalVolumes ];
	// dico che la lista deve essere rinfrescata
	[ listaVolumi reloadData ];
}

// ---------------------------------------------------------------------
// metodi per fornire i dati alal tabella
// il numero di elementi e' pari al numero dei volumi
- (int)
numberOfRowsInTableView:	(NSTableView *)tableView
{
	return [volumeNames count];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// l'elemento da visualizzare
- (id)
tableView:			(NSTableView *)tableView
	objectValueForTableColumn:	(NSTableColumn *)tableColumn
	row: 			(int)row
{
	// assegno alla cella l'immagine
	[((ImageAndTextCell*) [tableColumn dataCell]) setImage: [volumeIcons objectAtIndex: row] ];
	// e poi restituisco il nome del volume
	return ( [volumeNames objectAtIndex: row] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo per indicare se la cella e' editabile o meno
- (BOOL)
tableView:			(NSTableView *)aTableView
	shouldEditTableColumn:	(NSTableColumn *)aTableColumn
	row:			(int)rowIndex
{
	return NO ;
}

// ---------------------------------------------------------------------
// metodi accessor standard
- (NSMutableArray *)volumeNames {
	return [[volumeNames retain] autorelease];
}

- (void)setVolumeNames:(NSMutableArray *)newVolumeNames {
	if (volumeNames != newVolumeNames) {
		[volumeNames release];
		volumeNames = [newVolumeNames retain];
	}
}

- (NSMutableArray *)volumePaths {
	return [[volumePaths retain] autorelease];
}

- (void)setVolumePaths:(NSMutableArray *)newVolumePaths {
	if (volumePaths != newVolumePaths) {
		[volumePaths release];
		volumePaths = [newVolumePaths retain];
	}
}

- (NSMutableArray *)volumeIcons {
	return [[volumeIcons retain] autorelease];
}

- (void)setVolumeIcons:(NSMutableArray *)newVolumeIcons {
	if (volumeIcons != newVolumeIcons) {
		[volumeIcons release];
		volumeIcons = [newVolumeIcons retain];
	}
}


@end
