// -----------------------------------------------------------------------------
//  CoverView.h
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// interfaccia della classe della view che contiene la copertina

#import		<Cocoa/Cocoa.h>

#import		"CCE_ElemGroup.h"

@class		CoverWinCtl ;

// dimensioni di un retro copertina in centimentri
#define		BACK_OUTER_HOR_DIM		15.1F
#define		BACK_OUTER_VER_DIM		11.8F

#define		BACK_LABEL_HOR_DIM		0.65F
#define		BACK_LABEL_VER_DIM		11.8F

#define		CUTSIGN_DIM				1.0F
#define		CUTSIGN_DIST			0.5F

#define		STARTPOINT_X			1.0F
#define		STARTPOINT_Y			1.0F

#define		VIEW_DIM_HOR	(2*(STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST) + BACK_OUTER_HOR_DIM)
#define		VIEW_DIM_VEL	(2*(STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST) + BACK_OUTER_VER_DIM)

@interface	CoverView : NSView
{
	IBOutlet CoverWinCtl	* winCtl ;
	IBOutlet NSScrollView	* shScroll ;
	IBOutlet NSTextView		* shText ;
	// vettore con gli elementi da disegnare
	CCE_ElemGroup		* theElements ;
	// se c'e' un elemento in corso di disegno
	CCE_BasicForm		* curCreatingElem ;
	// rettangolo di selezione
	CCE_Rect			* curSelectRect ;
	// parametri per la visualizzazione della griglia
	float				gridSpacing ;
	BOOL				showGrid ;
	BOOL				snapToGrid ;
	NSColor				* gridLineColor ;
	// contatore per identificatori
	long				nextObjId ;
	// flag di comodo sparsi
	// indica se e' in corso l'editing di un elemento
	CCE_BasicForm		* editingInProgress ;
	// menu contestuali
	IBOutlet NSMenu		* stdElemMenu ;
	IBOutlet NSMenu		* txtElemMenu ;
	IBOutlet NSMenu		* grpElemMenu ;
	IBOutlet NSMenu		* mltElemMenu ;
	IBOutlet NSMenu		* imgElemMenu ;
	// gestione dei template
	NSString			* curTemplateName ;
	// dimensioni correnti della vista
	int					curUnits ;
	NSSize				curViewSize ;
	float				curZoomFactor ;
	// informazioni per la stampa della copertina
	NSPrintInfo			* pInfo ;
}
// inizializzazioni
- (id)		initWithFrame:(NSRect)frameRect	;
- (void)	loadFromTemplate: (NSString *) templatePath ;
- (void)	dealloc ;
// salvataggio di template
- (void)	encodeWithCoder:(NSCoder *)encoder ;
- (id)		initWithCoder:(NSCoder *)decoder ;
// altri metodi di comodo
- (void)	insertBasicDraw	;
- (BOOL)	isFlipped ;
- (BOOL)	acceptsFirstMouse:(NSEvent *)theEvent ;
// disegno
- (void)	drawRect:(NSRect)rect ;
- (void)	drawGrid:(NSRect)rect ;
- (void)	setZoom:(float)scaleFactor  ;
// gestione mouse
- (void)	mouseDown:(NSEvent *)theEvent ;
- (void)	handleMouseClick: (NSEvent *)theEvent ;
- (void)	handleMoveAll: (NSEvent *)theEvent ;
- (CCE_BasicForm *) getClickedElem:(NSPoint)point ;
- (NSRect)	handleRectSelect: (NSEvent *)theEvent ;
// gestione tastiera
- (void)	keyDown:(NSEvent *)event ;
- (void)	moveLeft:(id)sender ;
- (void)	moveRight:(id)sender ;
- (void)	moveUp:(id)sender ;
- (void)	moveDown:(id)sender ;
- (void)	moveLeftAndModifySelection:(id)sender ;
- (void)	moveRightAndModifySelection:(id)sender ;
- (void)	moveUpAndModifySelection:(id)sender ;
- (void)	moveDownAndModifySelection:(id)sender ;
// allineamento, centraggio, bloccaggio, spostamento
- (void)	executeAlignment: (int) menuTag ;
- (void)	executeCentering: (int) menuTag ;
- (void)	elemAlign: (id)sender ;
- (void)	groupElem: (id)sender ;
- (void)	lockElems: (id)sender ;
- (void)	sendElemFB: (id)sender ;
- (void)	rotateElem: (id) sender ;
// finestre accessorie
- (void)	showElemInfo:(id)sender;		// finestra informazioni elemento
- (void)	showAlignDiag: (id)sender ;	// palette di allineamento
- (void)	showRotateWin: (id)sender ;	// palette per la rotazione

// cancellazione elementi
- (void)	clear: (id)sender ;
- (void)	deleteElem:(id)sender ;
- (void)	deleteForward:(id)sender ;
- (void)	deleteBackward:(id)sender ;
- (void)	deleteSelectedElements ;
- (void)	alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo ;
- (IBAction) showElemInfo:(id)sender ;		// info dell'elemento
- (IBAction) showAlignDiag: (id)sender ;	// palette di allineamento
- (IBAction) showRotateWin: (id)sender ;	// palette per la rotazione
- (IBAction) setOriginalSize: (id)sender ;	// dimensione originale figura
- (IBAction) setTransparency: (id)sender ;	// trasparenza di una figura
- (IBAction) setTextParams: (id)sender ;	// impostazioni parametri testo
// override per la gestione dei menu contestuali
- (NSMenu *) menuForEvent: (NSEvent *)theEvent ;

// soliti accessor
- (CoverWinCtl *)winCtl;
- (void)setWinCtl:(CoverWinCtl *)newWinCtl;
- (CCE_ElemGroup *)theElements;
- (void)setTheElements:(CCE_ElemGroup *)newTheElements;
- (CCE_BasicForm *)curCreatingElem;
- (void)setCurCreatingElem:(CCE_BasicForm *)newCurCreatingElem;
- (CCE_Rect *)curSelectRect;
- (void)setCurSelectRect:(CCE_Rect *)newCurSelectRect;
- (float)gridSpacing;
- (void)setGridSpacing:(float)newGridSpacing;
- (BOOL)showGrid;
- (void)setShowGrid:(BOOL)newShowGrid;
- (BOOL)snapToGrid;
- (void)setSnapToGrid:(BOOL)newSnapToGrid;
- (NSColor *)gridLineColor;
- (void)setGridLineColor:(NSColor *)newGridLineColor;
- (long)nextObjId;
- (void)setNextObjId:(long)newNextObjId;
- (long) getNextObjId ;
- (CCE_BasicForm *)editingInProgress;
- (void)setEditingInProgress:(CCE_BasicForm *)newEditingInProgress;
- (NSString *)curTemplateName;
- (void)setCurTemplateName:(NSString *)newCurTemplateName;
- (int)curUnits;
- (void)setCurUnits:(int)newCurUnits;
- (NSSize)curViewSize;
- (void)setCurViewSize:(NSSize)newCurViewSize;
- (float)curZoomFactor;
- (void)setCurZoomFactor:(float)newCurZoomFactor;
- (NSPrintInfo *)pInfo;
- (void)setPInfo:(NSPrintInfo *)newPInfo;

- (NSScrollView *)shScroll;
- (NSTextView *)shText;
- (NSMenu *)stdElemMenu;
- (NSMenu *)txtElemMenu;
- (NSMenu *)grpElemMenu;
- (NSMenu *)mltElemMenu;
- (NSMenu *)imgElemMenu;

- (void)	managePrintWithPanels:( BOOL ) showPanels ;
- (void)	managePageLayout ;
- (void)	checkPageDims ;

@end

NSPoint getNearestGridPoint( NSPoint point, CoverView * view) ;
NSPoint getNearestSnapPoint( NSPoint point, CoverView * view) ;
