// -----------------------------------------------------------------------------
//  CoverView.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe della view che contiene la copertina

#import		"CoverView.h"

#import		"ImageAndTextCell.h"
#import		"CatFileInfo.h"
#import		"CatDataSrc.h"
#import		"CoverWinCtl.h"
#import		"CoverPalWinCtl.h"
#import		"AppDelegate.h"
#import		"ElemParamWinCtl.h"
#import		"djZeroUtils.h"
#import		"UnitsSizeGridWinCtl.h"

#import		"PrtMrgWinCtl.h"

// per copia/incolla
#define			CCE_ELEMTYPE		@"LS CdCatalog Element"

@implementation CoverView

// ---------------------------------------------------------------------
- (id)
initWithFrame:(NSRect)frameRect
{

	if ((self = [super initWithFrame:frameRect]) != nil)
	{
		float	val = convertMeasure(14, UNITS_CM, UNITS_PIXELS) ;
		// costruisco il vettore degli elementi
		[ self setTheElements: [[CCE_ElemGroup alloc] initWithId: 1 inView: self ] ];
		nextObjId = 2 ;
		[ self setCurCreatingElem: nil ] ;
		[ self setCurSelectRect: nil ] ;
		gridSpacing = 20 ;
		showGrid = NO ;
		snapToGrid = NO ;
		gridLineColor = [ NSColor lightGrayColor] ;
		editingInProgress = nil ;
		curUnits  = UNITS_PIXELS ;
		curViewSize = NSMakeSize( val, val ) ;
		curZoomFactor = 1.0 ;
		[ self insertBasicDraw ];
		[ self setPInfo: [[NSPrintInfo sharedPrintInfo] copy] ];
	}
	return self;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiusto la CoverView in base ai dati contenuti nel template
- (void)
loadFromTemplate: (NSString *) templatePath
{
	CoverView	* tmpCover ;
	[ winCtl arrangeZoom: 0 ];
	// se il template e' effettivo
	if ( templatePath )
	{
		// estraggo i dati dal template
		tmpCover = [NSKeyedUnarchiver unarchiveObjectWithFile: templatePath ];		
		[ theElements setElemArray: [ [tmpCover theElements ] elemArray ] ] ;
		[ [ self theElements] setOwnerView: self ] ;
		// aggiusto tutti gli altri valori della coverView
		[self setCurTemplateName: [tmpCover curTemplateName ] ];
		[self setGridSpacing: [tmpCover gridSpacing ] ];
		[self setShowGrid: [tmpCover showGrid ] ];
		[self setSnapToGrid: [tmpCover snapToGrid ] ];
		[self setGridLineColor: [tmpCover gridLineColor ] ];
		[self setNextObjId: [tmpCover nextObjId ] ];
		[self setCurUnits: [tmpCover curUnits ] ];
		[self setCurViewSize: [tmpCover curViewSize ] ];
		[self setFrameSize: [tmpCover curViewSize] ];		
		[self setPInfo: [tmpCover pInfo] ];		
	}
	else
	{
		// non ho un template, riempio con valori di default
		[ theElements setElemArray: [ NSMutableArray array ] ] ;
		[ [ self theElements] setOwnerView: self ] ;
		// aggiusto tutti gli altri valori della coverView
		[self setCurTemplateName: @"---" ];
		[self setGridSpacing: 20 ];
		[self setShowGrid: NO ];
		[self setSnapToGrid: NO ];
		[self setGridLineColor: [ NSColor lightGrayColor] ];
		[self setNextObjId: 2 ];
		[self setCurUnits: UNITS_PIXELS ];
		[self setCurViewSize: NSMakeSize(
			convertMeasure(VIEW_DIM_HOR, UNITS_CM, UNITS_PIXELS), 
			convertMeasure(VIEW_DIM_VEL, UNITS_CM, UNITS_PIXELS) )];
		[self setFrameSize: NSMakeSize(
			convertMeasure(VIEW_DIM_HOR, UNITS_CM, UNITS_PIXELS), 
			convertMeasure(VIEW_DIM_VEL, UNITS_CM, UNITS_PIXELS) )];		
	}
	// aggiorno il testo automatico
	[ [ theElements elemArray] makeObjectsPerformSelector: 
			@selector(updAutoText:) withObject: [[ winCtl document] dataSource] ] ;
	// aggiorno l'intera finestra
	[self setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void) dealloc
{
	[ theElements dealloc ];
	[ super dealloc ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// codifica dei dati di un template, parte generale
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	if ( [encoder allowsKeyedCoding] ) 
	{
		[ encoder encodeObject: curTemplateName forKey: @"tmpl_Name" ] ;
		[ encoder encodeFloat: gridSpacing forKey:  @"tmpl_gridSpacing"] ;
		[ encoder encodeBool: showGrid forKey:  @"tmpl_showGrid"] ;
		[ encoder encodeBool: snapToGrid forKey:  @"tmpl_snapToGrid"] ;
		[ encoder encodeObject: gridLineColor forKey: @"tmpl_gridLineColor"  ] ;
		[ encoder encodeInt32: nextObjId forKey:  @"tmpl_nextObjId"] ;
		[ encoder encodeInt: curUnits forKey:  @"tmpl_curUnits"] ;
		[ encoder encodeSize: curViewSize forKey:  @"tmpl_curViewSize"] ;
		[ encoder encodeObject: theElements forKey: @"tmpl_Elements"] ;
		[ encoder encodeObject: pInfo forKey: @"tmpl_printinfo" ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// estrazione dei dati di un template, parte generico
- (id)
initWithCoder:(NSCoder *)decoder 
{
    if ( [decoder allowsKeyedCoding] ) 
	{
		// in realta' l'ordine non ha importanza
 		[ self setCurTemplateName: [ decoder decodeObjectForKey: @"tmpl_Name" ] ];
		gridSpacing = [decoder decodeFloatForKey: @"tmpl_gridSpacing" ];	
		showGrid = [decoder decodeBoolForKey: @"tmpl_showGrid" ];	
		snapToGrid = [decoder decodeBoolForKey: @"tmpl_snapToGrid" ];	
		[ self setGridLineColor: [ decoder decodeObjectForKey: @"tmpl_gridLineColor" ] ];
		nextObjId = [decoder decodeInt32ForKey: @"tmpl_nextObjId" ];	
		curUnits = [decoder decodeIntForKey: @"tmpl_curUnits" ];	
		curViewSize = [decoder decodeSizeForKey: @"tmpl_curViewSize" ];	
		[ self setCurCreatingElem: nil ] ;
		[ self setCurSelectRect: nil ] ;
		editingInProgress = nil ;
		[ self setTheElements: [ decoder decodeObjectForKey: @"tmpl_Elements" ] ];
		[ self setPInfo: [ decoder decodeObjectForKey: @"tmpl_printinfo" ] ];		
	}
	return ( self );
}

// ---------------------------------------------------------------------
// gestione della stampa della view
// impostazione della pagina, con vista accessoria
- (void)
managePageLayout 
{
	NSPageLayout	* pageLayout = [NSPageLayout pageLayout];
	PrtMrgWinCtl	* prtWinCtl ;
	// carico la vista accessoria con l'impostazione dei margini
	prtWinCtl = [ [ PrtMrgWinCtl alloc] initWithWindowNibName: @"printMargin" ];
	[ prtWinCtl setRefCvrView: self ] ;
	[ pageLayout setAccessoryView: [ [ prtWinCtl window ] contentView ] ];
	// recupero i margini dall'istanza corrente e li assegno alla vista
	[ prtWinCtl setUpMargins: NSMakeRect( 
		[ pInfo topMargin ], [ pInfo leftMargin ], 
		[ pInfo rightMargin ], [ pInfo bottomMargin ]) ];
	// faccio girare l'ambaradan
	[ pageLayout beginSheetWithPrintInfo: pInfo
		modalForWindow: [ winCtl window ] 
		delegate: self 
		didEndSelector:@selector(pageLayoutDidEnd:returnCode:contextInfo:) 
		contextInfo: nil ] ;
	// eventuali modifiche ai margini sono gestiti direttamente da PrtMrgWinCtl
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
checkPageDims
{

	float		printableWidth, printableHeight, scaleFactor, tmp ;
	// tengo conto del fattore di scala
    scaleFactor = [[[pInfo dictionary] objectForKey:NSPrintScalingFactor] floatValue];
	tmp = curViewSize.width * scaleFactor ;
	printableWidth = [pInfo paperSize].width - tmp
					- ([pInfo leftMargin] + [pInfo rightMargin]);  
	tmp = curViewSize.height * scaleFactor ;
	printableHeight = [pInfo paperSize].height - tmp
					- ([pInfo topMargin]+[pInfo bottomMargin]);  
	// potremmo avere delle dimensioni troppo piccole
	if ( printableWidth < 0 || printableHeight < 0 )
	{
		// costruisco un alert
		NSAlert *alert = [[NSAlert alloc] init];
		NSString	* messaggio = [ NSString string ];
		// e' un warning, che tanto si puo' andare avanti
		[alert setAlertStyle:NSWarningAlertStyle];	
		// c'e' un solo pulsante sdi ok
		[alert addButtonWithTitle:@"OK"];
		// dico che la copertina non ci sta nella pagina
		[alert setMessageText:@"The cover will not fit into the page"];
		// e dico anche per quanto
		if ( printableWidth < 0 )
		{
			float	tmpVal1, tmpVal2 ;
			tmpVal1 = convertMeasure( -printableWidth, UNITS_PIXELS, curUnits );
			tmpVal2 = convertMeasure( curViewSize.width, UNITS_PIXELS, curUnits );
			messaggio = [ messaggio stringByAppendingFormat: 
				@"Required width: %7.3f (%@), missing %7.3f\n",
				tmpVal2, stringForUnits(curUnits), tmpVal1 ] ;
		}
		if ( printableHeight < 0 )
		{
			float	tmpVal1, tmpVal2 ;
			tmpVal1 = convertMeasure( -printableHeight, UNITS_PIXELS, curUnits );
			tmpVal2 = convertMeasure( curViewSize.height, UNITS_PIXELS, curUnits );
			messaggio = [ messaggio stringByAppendingFormat: 
				@"Required height: %7.3f (%@), missing %7.3f",
				tmpVal2, stringForUnits(curUnits), tmpVal1 ] ;
		}
		[ alert setInformativeText: messaggio ];
		[ alert runModal ];
	}
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
pageLayoutDidEnd: (NSPageLayout *)pageLayout 
	returnCode:(int)returnCode 
	contextInfo:(void *)contextInfo
{
	[ self checkPageDims ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// processo di stampa vero e proprio
- (void)
managePrintWithPanels:( BOOL ) showPanels 
{
	NSPrintOperation	* po ;
	// verifico se ci sta nella pagina
	[ self checkPageDims ];
	// comincio le operazioni di stampa
    po = [ NSPrintOperation printOperationWithView: self printInfo: pInfo ];
	[ po setShowPanels: showPanels ];
	[ po runOperationModalForWindow: [ winCtl window]
		delegate: nil didRunSelector:nil contextInfo: nil ];
}

// ---------------------------------------------------------------------
// per rovesciare l'asse delle ordinate
- (BOOL)
isFlipped
{
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (BOOL)
acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
setZoom:(float)scaleFactor
{
	NSRect frame = [self frame];
	NSRect bounds = [self bounds];
	frame.size.width = bounds.size.width * scaleFactor;
	frame.size.height = bounds.size.height * scaleFactor;
	[self setFrameSize: frame.size];
	[self setBoundsSize: bounds.size];
	curZoomFactor = scaleFactor ;
}

// ---------------------------------------------------------------------
// il disegno della vista
- (void)
drawRect:(NSRect)rect
{
	BOOL	scrOrPrt = [NSGraphicsContext currentContextDrawingToScreen] ;
	// sfondo bianco
	if ( scrOrPrt )
	{
		[[ NSColor whiteColor] set ];
		NSRectFill( rect );
	}
	// se c'e' la griglia, la disegno
	if ( scrOrPrt && showGrid) 
		[ self drawGrid: rect ];
	// poi dico di disegnarsi a tutti gli elementi
	[ theElements drawElement: rect ];
	// e poi all'elemento in corso
	if ( scrOrPrt && curCreatingElem) 
	{
		[ curCreatingElem drawElement: rect ] ;
	}
	if ( scrOrPrt && curSelectRect) 
	{
		NSBezierPath	* curPath = [ curSelectRect theDrawPath ];
		float			strokePattern[2] ={ 5, 5 } ;
		[ curPath setLineDash: strokePattern count: 2 phase: 0.0];
		[ curSelectRect setTheDrawPath: curPath ];
		[ curSelectRect drawElement: rect ] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// disegno della griglia
- (void)
drawGrid:(NSRect)rect
{
	NSPoint			topLeft, bottomRight ;
	float			kk, strokePattern[2] ={0.25*gridSpacing, 0.25*gridSpacing } ;
	NSBezierPath	* curPath1 = [[[ NSBezierPath init ] alloc ] autorelease ]  ;

	topLeft = getNearestGridPoint( rect.origin, self );
	bottomRight = NSMakePoint( rect.origin.x + rect.size.width, rect.origin.y + rect.size.height);
	bottomRight = getNearestGridPoint( bottomRight, self );
	[ gridLineColor set ];
	for ( kk = topLeft.x ; kk <= bottomRight.x; kk += gridSpacing )
	{
		[curPath1 moveToPoint: NSMakePoint( kk, topLeft.y-gridSpacing) ];
		[curPath1 lineToPoint: NSMakePoint( kk, bottomRight.y+gridSpacing) ];
	}
	for ( kk = topLeft.y ; kk <= bottomRight.y ; kk += gridSpacing )
	{
		[curPath1 moveToPoint: NSMakePoint( topLeft.x-gridSpacing, kk) ];
		[curPath1 lineToPoint: NSMakePoint( bottomRight.x+gridSpacing, kk) ];
	}
	[ curPath1 setLineWidth: 0];
	[ curPath1 setLineDash: strokePattern count: 2 phase: gridSpacing*0.125];
	[ curPath1 stroke ];
}

// ---------------------------------------------------------------------
// gestione della tastiera
- (void)
keyDown:(NSEvent *)event
{
	[self interpretKeyEvents:[NSArray arrayWithObject:event]];
	// anche qui, verosimilmente, cambia qualcosa
	[ [NSNotificationCenter defaultCenter]
		postNotificationName:@"CoverViewChangedSelection"
		object:self];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// binding per la quattro frecce
- (void)
moveLeft:(id)sender
{
	float   delta = snapToGrid ? -gridSpacing : -1.0 ;
	[ theElements moveSelectedElements: NSMakePoint(delta, 0.0)];
}

- (void)
moveRight:(id)sender
{
	float   delta = snapToGrid ? gridSpacing : 1.0 ;
	[ theElements moveSelectedElements: NSMakePoint(delta, 0.0)];
}

- (void)
moveUp:(id)sender
{
	float   delta = snapToGrid ? -gridSpacing : -1.0 ;
	[ theElements moveSelectedElements: NSMakePoint(0.0, delta)];
}

- (void)
moveDown:(id)sender
{
	float   delta = snapToGrid ? gridSpacing : 1.0 ;
	[ theElements moveSelectedElements: NSMakePoint(0.0, delta)];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// binding per la quattro frecce con tasto maiuscole
- (void)
moveLeftAndModifySelection:(id)sender
{
	float   delta = snapToGrid ? -(gridSpacing*5) : -10.0 ;
	[ theElements moveSelectedElements: NSMakePoint(delta, 0.0)];
}

- (void)
moveRightAndModifySelection:(id)sender
{
	float   delta = snapToGrid ? (gridSpacing*5) : 10.0 ;
	[ theElements moveSelectedElements: NSMakePoint(delta, 0.0)];
}

- (void)
moveUpAndModifySelection:(id)sender
{
	float   delta = snapToGrid ? -(gridSpacing*5) : -10.0 ;
	[ theElements moveSelectedElements: NSMakePoint(0.0, delta)];
}

- (void)
moveDownAndModifySelection:(id)sender
{
	float   delta = snapToGrid ? (gridSpacing*5) : 10.0 ;
	[ theElements moveSelectedElements: NSMakePoint(0.0, delta)];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cancellazione di un elemento
- (void)
deleteForward:(id)sender
{
	[self clear:sender];
}

- (void)
deleteBackward:(id)sender
{
	[self clear:sender];
}

// ---------------------------------------------------------------------
// cancellazione di un elemento, passo 1
- (void)
clear: (id)sender
{
	NSMutableArray  * selArr = [ theElements getSelectedGraphics ];
	int				  numElem = [ selArr count ];
	// se non ci sono elementi selezionai, faccio nulla
	if ( numElem == 0 )
		return ;
	[self deleteElem:sender];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cancellazione di un elemento, passo 2
- (void)
deleteElem:(id)sender
{
	// costruisco un bell'alert
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert addButtonWithTitle:@"OK"];
	[alert addButtonWithTitle:@"Cancel"];
	[alert setMessageText:@"Delete the selected elements?"];
	[alert setInformativeText:@"Deleted elements cannot be restored."];
	[alert setAlertStyle:NSWarningAlertStyle];
	// lo mostro modale
	[alert beginSheetModalForWindow:[winCtl window]
			modalDelegate:self
			didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:)
			contextInfo:nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cancellazione di un elemento, passo 3
- (void)
alertDidEnd:(NSAlert *)alert
	returnCode:(int)returnCode
	contextInfo:(void *)contextInfo
{
	// se c'e' stata conferma...
	if (returnCode == NSAlertFirstButtonReturn)
		// cancello gli elementi selezionati
		[ self deleteSelectedElements ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// cancellazione di un elemento, passo 4
- (void)
deleteSelectedElements
{
	int		i , numElem ;
	CCE_BasicForm   * elem ;
	// ciclo su tutti gli elementi del gruppo
	numElem = [ [ theElements elemArray] count ];
	// dalal fine all'inizio, eliminando dei pezzi
	for ( i = (numElem-1) ; i >= 0 ; i -- )
	{
		elem = [ [ theElements elemArray] objectAtIndex: i ];
		if ( [ elem cceIsSelected ] )
		{
			[ theElements remElemAtIndex: i ];
		}
	}
	[ self setNeedsDisplay: YES ];
}

// ---------------------------------------------------------------------
// gestione di un evento di click del mouse
- (void)
mouseDown:(NSEvent *)theEvent
{
	// vado a pigliare qual e' il tool correntemente selezionato nella finestra
	Class selCls = [[CoverPalWinCtl sharedCoverPalWinCtrl] getCurrClassTool];

	if ( editingInProgress )
	{
		[ editingInProgress endEditing ];
	}
	// se e' selezionato un elemento da disegnare
	if (selCls)
	{
		// costruisco l'elemento grafico
		CCE_BasicForm   * curElem = [[ [selCls alloc ] init] autorelease ] ;
		CoverPalWinCtl  * pal = [CoverPalWinCtl sharedCoverPalWinCtrl] ;

		// metto a posto un di attributi base dell'elemento
		[ curElem  setCceLineWidth: [pal getLineWidth] ];
		[ curElem  setCceFillColor: [pal getCurrFillColor] ];
		[ curElem  setCceLineColor: [pal getCurrLineColor] ];
		[ curElem  setCceIsFilled:  [pal getFillSts] ];
		[ curElem  setCceIsStroked: [pal getStrokeSts] ];
		[ curElem  setOwnerView: self ];
		// deseleziono tutto
		[ theElements cascadeSelection: NO ] ;
		// dico che l'elemento che sto disegnando e' selezionato
		[ curElem  setCceIsSelected: YES ];
		// e che si tratta dell'elemento in lavorazione
		[ self setCurCreatingElem: curElem ] ;
		// gestisco la costruzione col mouse
		if ([curElem handleCreation: theEvent ])
		{
			// se la creazione e' andata a buon fine,lo aggiungo alla lista
			[ curElem setObjID: [self getNextObjId]];
			[ theElements addElem:curElem atIndex: 0 ];
			[ curElem startEditing ] ;
		}
		// non ci sono piu' elementi in lavorazione
		[ self setCurCreatingElem: nil ] ;
		// ripristino il tool normale
		[ pal resetTool ] ;
	}
	else
	{
		// non si deve disegnare un nuovo elemento
		// tratto un generico click all'interno della finestra
		[self handleMouseClick: theEvent];
	}
	// ci sono ottime possibilita' che sia cambiata la selezione corrente
	[ [NSNotificationCenter defaultCenter]
		postNotificationName:@"CoverViewChangedSelection"
		object:self];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// tratto un generico click all'interno della finestra
- (void)
handleMouseClick: (NSEvent *)theEvent
{
	NSPoint			whereClick;
	CCE_BasicForm   * curElem = nil;
	BOOL			shiftKeyPress = (([theEvent modifierFlags] & NSShiftKeyMask) ? YES : NO);
	short			clicInHandle, numofclic ;
	// vado a vedere dove e' avvenuto il click
	whereClick = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	numofclic = [ theEvent clickCount ] ;
	// a prescidenre dal clic, riporto il focus sulla view
	[[winCtl window] makeFirstResponder: self];
	// niente niente l'utente vuole selezionare un elemento
	curElem = [self getClickedElem:whereClick];
	// se c'e' un elemento sotto il mouse
	if ( curElem )
	{
		// abbiamo qualcosa sotto il mouse
		// se e' un doppio clic, editing se il caso
		if ( numofclic >= 2 )
			[ curElem startEditing ] ;
		// vedo se e' gia' selezionato
		else if ( [ curElem cceIsSelected ] )
		{
				if ( shiftKeyPress )
				{
					// lo tolgo dagli elementi selezionati
					[ curElem setCceIsSelected: NO ];
				}
				else
				{
					// potrebbe aver fatto clic su uno degli hangle
					clicInHandle = [ curElem clicIntoHandle: whereClick ] ;
					// provo a vedere se e' il caso di muoverlo
					if ( clicInHandle == -1 )
						[ self handleMoveAll: theEvent ];
					else [ curElem handleResize: theEvent movingHandle: clicInHandle ];
				}
		}
		else
		{
			if ( ! shiftKeyPress )
			{
				// deseleziono tutto
				[ theElements cascadeSelection: NO ] ;
			}
			// potrebbe aver fatto clic su uno degli hangle
			clicInHandle = [ curElem clicIntoHandle: whereClick ] ;
			// seleziono questo elemento
			[ curElem  setCceIsSelected: YES ];
			// provo a vedere se e' il caso di muoverlo
			if ( clicInHandle == -1 )
				[ self handleMoveAll: theEvent ];
			else [ curElem handleResize: theEvent movingHandle: clicInHandle ];
		}
	}
	else
	{
		int		i , numElem ;
		CCE_BasicForm   * elem ;
		NSRect selRect, locrect ;
		// comincio la selezione multipla
		selRect = [ self handleRectSelect: theEvent ];
		locrect = arrangeRect( selRect.size );
		selRect.origin.x += locrect.origin.x ;
		selRect.origin.y += locrect.origin.y ;
		selRect.size = locrect.size ;
		// ciclo su tutti gli elementi del gruppo
		numElem = [ [ theElements elemArray] count ];
		for ( i = 0 ; i < numElem ; i ++ )
		{
			elem = [ [ theElements elemArray] objectAtIndex: i ];
			[ elem selectMeIfInRect: selRect addingToSelez: shiftKeyPress] ;
		}
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (NSMenu *)
menuForEvent: (NSEvent *)theEvent
{
	NSPoint			whereClick;
	CCE_BasicForm   * curElem  ;
	whereClick = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	curElem = [self getClickedElem:whereClick] ;
	if ( curElem == nil || ( ! [ curElem cceIsSelected]) )
		return ( nil ) ;
	curElem = [ AppDelegate getTheSelectedElem];
	if ( curElem == nil )
		return ( nil ) ;
	else if ( curElem == (CCE_BasicForm*)(-1) )
		return ( mltElemMenu );
	return ( [ curElem contextMenu ]);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// qui l'utente vuole muovere qualche elemento
- (void)
handleMoveAll: (NSEvent *)theEvent
{
	// recupero la coordinata locale
	NSPoint point1 = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSPoint point2, delta ;

	point1 = getNearestSnapPoint( point1, self);
	// ciclo piu' o meno lungo di attesa eventi
	do  {
		// recupero il prossimo evento, che sia un movimento del mouse
		// oppure un mouseup (che finisce il tutto)
		theEvent = [[self window] nextEventMatchingMask:(NSLeftMouseDraggedMask | NSLeftMouseUpMask)];
		// recupero la coordinata locale
		point2 = [self convertPoint:[theEvent locationInWindow] fromView:nil];
		point2 = getNearestSnapPoint( point2, self);
		// aggiusto l'aspetto dell'oggetto
		delta = NSMakePoint(point2.x-point1.x,point2.y-point1.y) ;
		if ( fabs( delta.x) > 1e-8 || fabs( delta.y) > 1e-8 )
		{
			[ theElements moveSelectedElements: delta];
			// il nuovo punto di partenza
			point1 = point2 ;
		}
	}
	while ( [theEvent type] != NSLeftMouseUp) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// qui l'utente vuole selezionare gli elementi dentro un rettangolo
- (NSRect)
handleRectSelect: (NSEvent *)theEvent
{
	NSRect		selRect = NSMakeRect(1, 1, 0, 0);
	// costruisco l'elemento grafico
	CCE_Rect   * curElem = [[ [CCE_Rect alloc ] init] autorelease ] ;
	BOOL shiftKeyPress = (([theEvent modifierFlags] & NSShiftKeyMask) ? YES : NO);
	// metto a posto un di attributi base dell'elemento
	[ curElem  setCceLineWidth: 0.0F ];
	[ curElem  setCceLineColor: [NSColor blackColor]  ];
	[ curElem  setCceIsFilled:  NO ];
	[ curElem  setCceIsStroked: YES ];
	[ curElem  setOwnerView: self ];
	if ( ! shiftKeyPress )
	{
		// deseleziono tutto
		[ theElements cascadeSelection: NO ] ;
	}
	// dico che l'elemento che sto disegnando non e' selezionato
	[ curElem  setCceIsSelected: NO ];
	[ self setCurSelectRect: curElem ] ;
	// gestisco la costruzione col mouse
	[curElem handleCreation: theEvent] ;
	selRect = NSMakeRect(   [curElem drawPoint].x, [curElem drawPoint].y,
							[curElem localSize].width, [curElem localSize].height );
	[ self setCurSelectRect: nil ] ;
	return ( selRect );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// vado a vedere se il click e' avvenuto su di un elemento
- (CCE_BasicForm *)
getClickedElem:(NSPoint)point
{
	int				i , numElem ;
	CCE_BasicForm   * elem = nil ;
	NSMutableArray		* listaelem ;
	// esamino tutti gli elemento
	listaelem = [ theElements elemArray ];
	numElem = [ listaelem count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ listaelem objectAtIndex: i ];
		// e controllo se il click gli appartiene
		if ( [ elem checkClickOnMe: point ])
		{
			// ho finito, esco dal ciclo
			break;
		}
		// controllo un eventuale click su handle
		if ( [ elem clicIntoHandle: point] != -1 )
			break ;
	}
	// potrei non avere trovato nulla
	if ( i == numElem )
		return ( nil );
	// se arrivo qui, ho trovato un elemento
	return ( elem );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// menu di select all
- (IBAction)
selectAll:(id)sender 
{
	[ theElements cascadeSelection: YES ] ;
}

// ---------------------------------------------------------------------
// operazione di copia degli elementi selezionati
- (IBAction)
copy:(id)sender 
{
    NSMutableDictionary * copyPasteDict ;
    NSMutableArray	* elementsDict, * selArr ;
	int				i, numElem ;
    NSString		* dictRepAsStr ;
	// accedo alla clipboard
	NSPasteboard	* pb = [NSPasteboard generalPasteboard];
	// recupero gli elementi selezionati
	selArr = [ theElements getSelectedGraphics ];
	numElem = [ selArr count ];
	// se non ce ne sono, ho gia' finito
	if ( numElem == 0 ) return ;
	// dico che nella clipboard ci sara' un mio tipo proprietario
	[ pb declareTypes: [NSArray arrayWithObjects:CCE_ELEMTYPE, nil] 
				owner:nil];
	// mi serve un NSData da mettere nella clipboard
	// costruisco un dizionario con gli elementi selezionati
    copyPasteDict = [ NSMutableDictionary dictionary ];
	// il dizionario e' composto da un array
    elementsDict = [NSMutableArray arrayWithCapacity: numElem ];
    for (i = 0; i < numElem; i++) 
	{
		// ogni elemento dello array e' un dictionary che raccoglie 
		// la descrizione dell'elemento in XML
		NSMutableDictionary * elemDict ;
		// descrizione dell'elemento in XML
		elemDict = [[selArr objectAtIndex:i] getXMLDescription] ;
		// aggiungo il dizionario allo array
        [elementsDict addObject: elemDict ];
    }
	// il dizionario principale e' composto dal solo array
    [ copyPasteDict setObject:elementsDict forKey: @"Lista Elementi"];
	// trasformo il dizionario in una stringa
    dictRepAsStr = [copyPasteDict description];
	// e quindi in un NSData che inserisco nella clipboard
	[ pb setData:[dictRepAsStr dataUsingEncoding: NSASCIIStringEncoding] 
				forType: CCE_ELEMTYPE ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// operazione di taglia = copia + cancella
- (IBAction)
cut:(id)sender 
{
    [self copy:sender];
    [ self deleteSelectedElements ];
	[[theElements undoManager] setActionName: @"CoverView -> cut"];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// incolla elementi estratti dalla clipboard nella Coverview
- (IBAction)
paste:(id)sender 
{
    NSArray		* elementsDict ;
    int			i, numElem ;
	NSDictionary * copyPasteDict ;
	NSString	* dictRepAsStr ;
	// accedo alla clipboard
    NSPasteboard * pb = [NSPasteboard generalPasteboard];
	// guardo che tipo di dati e' presente
    NSString	* type = [pb availableTypeFromArray:
						[NSArray arrayWithObjects:CCE_ELEMTYPE, nil]];
	// se ci sono tipi di dati che posso trattare
	if (! [type isEqualToString:CCE_ELEMTYPE]) 
		return ;
	// se arrivo qui, ho dati per me nella clipboard
	// estraggo i dati sotto forma di stringa
   dictRepAsStr = [[NSString allocWithZone:[self zone]] 
				initWithData:[pb dataForType:type] 
				encoding:NSASCIIStringEncoding];
	// che trasformo in un dizionario
    copyPasteDict = [dictRepAsStr propertyList];
    [dictRepAsStr release];
	// dal quale ricavo poi lo array con la descrizione degli elementi
    elementsDict = [copyPasteDict objectForKey:@"Lista Elementi"];
    numElem = [elementsDict count];
	// non dovrebbe accadere...
	if ( numElem <= 0 )	return ;
	// deseleziono tutto
	[ theElements cascadeSelection: NO ] ;
	// per ogni elemento dello array costruisco un l'elemento descritto
    for (i = 0; i < numElem ; i++) 
	{
		id elem ;
		elem = [ CCE_BasicForm buildElemFromXMLDesc: [elementsDict objectAtIndex:i]] ;
		[ elem  setOwnerView: self ];
		[ elem  setCceIsSelected: YES ];
		[ theElements addElem:elem atIndex: 0 ];
    }
	[[theElements undoManager] setActionName: @"CoverView -> paste"];
}

// ---------------------------------------------------------------------
// esecuzione di un allineamento su un insieme di elementi
// comando da una voce di menu
- (void)
elemAlign: (id)sender
{
	int		menuTag = [ sender tag ];
	[ self executeAlignment: menuTag ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
executeAlignment: (int) menuTag
{
	NSMutableArray  * selArr ;
	int				numElem, i ;
	float   		curTop, newTop, delta ;
	CCE_BasicForm   * elem ;
	BOOL			chooseDir  ;
	SEL				getDim  ;
	// vedo che tipo di allineamento
	switch ( menuTag ) {
	case 1 : chooseDir = TRUE ; getDim = @selector(getElemLeft) ; break ;
	case 2 : chooseDir = FALSE ; getDim = @selector(getElemRight) ; break ;
	case 3 : chooseDir = TRUE ; getDim = @selector(getElemTop) ; break ;
	case 4 : chooseDir = FALSE ; getDim = @selector(getElemBottom) ; break ;
	default : return ;
	}
	// recupero tutti gli elementi selezionati
	selArr = [ theElements getSelectedGraphics ];
	numElem = [ selArr count ];
	if ( numElem <= 1 ) return ;
	elem = [ selArr objectAtIndex: 0 ];
	curTop =  [[ elem performSelector: getDim ] floatValue ];
	// vado a cercare la coordinata su cui effettuare l'allineamento
	for ( i = 1 ; i < numElem ; i ++ )
	{
		elem = [ selArr objectAtIndex: i ];
		newTop = [[ elem performSelector: getDim ] floatValue ];
		if ( chooseDir && newTop < curTop )
			curTop = newTop ;
		else if ( ! chooseDir && newTop > curTop )
			curTop = newTop ;
	}
	// adesso che ho la coordinata, sposto tutto
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ selArr objectAtIndex: i ];
		delta = curTop - [[ elem performSelector: getDim ] floatValue ];
		if ( menuTag == 1 || menuTag == 2 )
		{
			[ elem moveMeBy: NSMakePoint( delta, 0 ) onlyIfSelected: YES];
		}
		else
		{
			[ elem moveMeBy: NSMakePoint( 0, delta ) onlyIfSelected: YES ];
		}
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// esecuzione di un centraggio su un insieme di elementi
- (void)
executeCentering: (int) menuTag
{
	NSMutableArray  * selArr ;
	int				numElem, i ;
	float   		curMax, curMin, curMid, newTop, delta ;
	CCE_BasicForm   * elem ;
	SEL				getDim  ;
	// middle: centro le ordinate, center: centro le ascisse
	switch ( menuTag ) {
	case 5 : getDim = @selector(getElemMiddle) ; break ;
	case 6 : getDim = @selector(getElemCenter) ; break ;
	default : return ;
	}
	// recupero tutti gli elementi selezionati
	selArr = [ theElements getSelectedGraphics ];
	numElem = [ selArr count ];
	if ( numElem <= 1 ) return ;
	elem = [ selArr objectAtIndex: 0 ];
	curMax = curMin =  [[ elem performSelector: getDim ] floatValue ];
	// vado a cercare la coordinata su cui effettuare il centraggio
	for ( i = 1 ; i < numElem ; i ++ )
	{
		elem = [ selArr objectAtIndex: i ];
		newTop = [[ elem performSelector: getDim ] floatValue ];
		if ( newTop < curMin )
			curMin = newTop ;
		else if ( newTop > curMax )
			curMax = newTop ;
	}
	// adesso che ho il max ed il min, sposto tutto
	curMid = 0.5 * (curMax + curMin );
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ selArr objectAtIndex: i ];
		delta = curMid - [[ elem performSelector: getDim ] floatValue ];
		if ( menuTag == 6 )
		{
			[ elem moveMeBy: NSMakePoint( delta, 0 ) onlyIfSelected: YES ];
		}
		else
		{
			[ elem moveMeBy: NSMakePoint( 0, delta ) onlyIfSelected: YES ];
		}
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// rotazione di un elemento da menu
- (void)
rotateElem: (id) sender
{
	float		angle = ([ sender tag] == 90) ? +90.0 : -90.0 ;
	[ theElements rotateElemAngle: angle absolute: NO  ] ;
	[ [NSNotificationCenter defaultCenter]
		postNotificationName:@"CoverViewChangedSelection"
		object: self ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// attiviazione/disattivazione di un gruppo di elementi
- (void)
groupElem: (id)sender
{
	int		menuTag = [ sender tag ];

	// controllo il tag della voce di menu
	if ( menuTag == 200 )
	{
		// devo fare un gruppo, passo la palla ad altri
		[ theElements makeGroupFromSelected ];
	}
	else
	{
		// devo disfare un gruppo
		NSMutableArray  * selArr ;
		int				numElem, i ;
		CCE_BasicForm   * elem ;
		// esamino gli elementi cercando quelli selezionati
		selArr = [ theElements elemArray ];
		numElem = [ selArr count ];
		for ( i = numElem-1 ; i >= 0; i -- )
		{
			elem = [ selArr objectAtIndex: i ];
			if ( ! [ elem cceIsSelected ])
				continue ;
			// se arrivo qui, l'elemento e' selezionato
			// ma potrebbe non essere un gruppo
			if ( ! [elem isKindOfClass:[CCE_ElemGroup class]] )
				continue ;
			// se arrivo qui, l'elemento e' selezionato ed e' un gruppo
			// %%%% manca gestione undo %%%%
			[ elem retain ];
			// lo elimino dall'insieme degli oggetti
			[ [theElements elemArray] removeObjectAtIndex: i ];
			// e poi appiccico gli elementi
			[ ((CCE_ElemGroup*) elem) ungroupAndInsertIn: theElements startingFrom: i ];
			[ elem release ];
		}
	}
	// forzo un ridisegno
	[ self setNeedsDisplay: YES ];
	[ [NSNotificationCenter defaultCenter]
		postNotificationName:@"CoverViewChangedSelection"
		object:self];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// dichiarazioen di elementi selezionati come locked/normali
- (void)
lockElems: (id)sender
{
	NSMutableArray  * selArr ;
	int				numElem, i ;
	CCE_BasicForm   * elem ;
	int				menuTag = [ sender tag ];
	// spazzoli tutti gli elementi selezionati
	selArr = [ theElements getSelectedGraphics ];
	numElem = [ selArr count ];
	for ( i = 0 ; i < numElem ; i ++ )
	{
		elem = [ selArr objectAtIndex: i ];
		// e blocco/sblococ a seconda del tag del menu
		if ( menuTag == 100 )
				[ elem setCceIsLocked: YES ];
		else	[ elem setCceIsLocked: NO ];
	}
	// forzo un ridisegno
	[ self setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// spostamento nell'ordine degli elementi selezionati
- (void)
sendElemFB: (id)sender
{
	int		menuTag = [ sender tag ];
	switch ( menuTag ) {
	case 1 : [ theElements sendElemFrontAndBack: +2 ]; break ;
	case 2 : [ theElements sendElemFrontAndBack: +1 ]; break ;
	case 3 : [ theElements sendElemFrontAndBack: -1 ]; break ;
	case 4 : [ theElements sendElemFrontAndBack: -2 ]; break ;
	default :
		return ;
	}
	// forzo un ridisegno
	[ self setNeedsDisplay: YES ];
}

// ---------------------------------------------------------------------
- (IBAction)
showElemInfo:(id)sender
{
	// mostro la finestra invocando l'istanza condivisa
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] showWindow: sender ] ;
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] selectPanel: SECTSEL_INFO ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
showAlignDiag: (id)sender
{
	// mostro la finestra invocando l'istanza condivisa
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] showWindow: sender ] ;
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] selectPanel: SECTSEL_ALIGNROTATE ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
showRotateWin: (id)sender
{
	// mostro la finestra invocando l'istanza condivisa
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] showWindow: sender ] ;
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] selectPanel: SECTSEL_ALIGNROTATE ] ;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
setTextParams: (id)sender 
{
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] showWindow: sender ] ;
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] selectPanel: SECTSEL_SPECIAL ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// dimensione originale figura
- (IBAction) 
setOriginalSize: (id)sender 
{
	CCE_Image	* myImg = (CCE_Image*) [ AppDelegate getTheSelectedElem] ;
	[ myImg setOriginalSize ] ;	
	[ self setNeedsDisplay: YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// trasparenza di una figura
- (IBAction) 
setTransparency: (id)sender 
{
	// mostro la finestra invocando l'istanza condivisa
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] showWindow: sender ] ;
	[ [ ElemParamWinCtl sharedElemParamWinCtl ] selectPanel: SECTSEL_SPECIAL ] ;
}	

// ---------------------------------------------------------------------
// accessor

- (CoverWinCtl *)winCtl {
	return  winCtl ;
}

- (void)setWinCtl:(CoverWinCtl *)newWinCtl {
	if (winCtl != newWinCtl) {
		[winCtl release];
		winCtl = [newWinCtl retain];
	}
}

- (CCE_ElemGroup *)theElements {
	return theElements ;
}

- (void)setTheElements:(CCE_ElemGroup *)newTheElements {
	if (theElements != newTheElements) {
		[theElements release];
		theElements = [newTheElements retain];
//		theElements = [newTheElements copy];
	}
}

- (CCE_BasicForm *)curCreatingElem {
	return curCreatingElem ;
}

- (void)setCurCreatingElem:(CCE_BasicForm *)newCurCreatingElem {
	if (curCreatingElem != newCurCreatingElem) {
		[curCreatingElem release];
		curCreatingElem = [newCurCreatingElem retain];
	}
}

- (CCE_Rect *)curSelectRect {
	return curSelectRect ;
}

- (void)setCurSelectRect:(CCE_Rect *)newCurSelectRect {
	if (curSelectRect != newCurSelectRect) {
		[curSelectRect release];
		curSelectRect = [newCurSelectRect retain];
	}
}

- (float)gridSpacing {
	return gridSpacing;
}

- (void)setGridSpacing:(float)newGridSpacing {
		gridSpacing = newGridSpacing;
}

- (BOOL)showGrid {
	return showGrid;
}

- (void)setShowGrid:(BOOL)newShowGrid {
		showGrid = newShowGrid;
}

- (BOOL)snapToGrid {
	return snapToGrid;
}

- (void)setSnapToGrid:(BOOL)newSnapToGrid {
		snapToGrid = newSnapToGrid;
}

- (NSColor *)gridLineColor {
	return gridLineColor ;
}

- (void)setGridLineColor:(NSColor *)newGridLineColor {
	if (gridLineColor != newGridLineColor) {
		[gridLineColor release];
		gridLineColor = [newGridLineColor copy];
	}
}
- (long)nextObjId {
	return nextObjId;
}

- (void)setNextObjId:(long)newNextObjId {
		nextObjId = newNextObjId;
}

- (long) getNextObjId {
	long	tmp = nextObjId ;
	nextObjId += 1 ;
	return ( tmp );
}

- (CCE_BasicForm *)editingInProgress {
	return editingInProgress ;
}

- (void)setEditingInProgress:(CCE_BasicForm *)newEditingInProgress {
		editingInProgress = newEditingInProgress ;
}

- (NSScrollView *)shScroll {
	return shScroll ;
}

- (NSTextView *)shText{
	return shText ;
}

- (NSMenu *)stdElemMenu {
    return stdElemMenu ;
}

- (NSMenu *)txtElemMenu {
    return txtElemMenu ;
}

- (NSMenu *)grpElemMenu {
    return grpElemMenu ;
}

- (NSMenu *)mltElemMenu {
    return mltElemMenu ;
}

- (NSMenu *)imgElemMenu {
    return imgElemMenu ;
}

- (NSString *)curTemplateName {
    return curTemplateName ;
}

- (void)setCurTemplateName:(NSString *)newCurTemplateName {
    if (curTemplateName != newCurTemplateName) {
        [curTemplateName release];
        curTemplateName = [newCurTemplateName copy];
    }
}

- (int)curUnits {
    return curUnits;
}

- (void)setCurUnits:(int)newCurUnits {
        curUnits = newCurUnits;
}

- (NSSize)curViewSize {
    return curViewSize;
}

- (void)setCurViewSize:(NSSize)newCurViewSize {
        curViewSize = newCurViewSize;
}

- (float)curZoomFactor {
    return curZoomFactor;
}

- (void)setCurZoomFactor:(float)newCurZoomFactor {
        curZoomFactor = newCurZoomFactor;
}

- (NSPrintInfo *)pInfo {
    return pInfo ;
}

- (void)setPInfo:(NSPrintInfo *)newPInfo {
    if (pInfo != newPInfo) {
        [pInfo release];
        pInfo = [newPInfo copy];
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
insertBasicDraw
{
	// aggiungo un po' di roba
	float	val1 = convertMeasure(20, UNITS_CM, UNITS_PIXELS) ;
	float	val2 = convertMeasure(17, UNITS_CM, UNITS_PIXELS) ;
	curViewSize = NSMakeSize( val1, val2 ) ;
	if ( 1 )
	{   // qui disegno tutto lo sfondo della copertina
		CCE_Line		* tmpLine ;
		float   x1, x2, y1, y2 ;
		CCE_Rect		* tmpRect ;

		// due linee di taglio inferiori verticali
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST ;
		y1 = STARTPOINT_Y ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM ;
		y1 = STARTPOINT_Y ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		// due linee di taglio inferiori orizzontali
		x1 = STARTPOINT_X ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM + CUTSIGN_DIST ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		// due linee di taglio superiori veritcali
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		// due linee di taglio superiori orizzontali
		x1 = STARTPOINT_X ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		x2 = STARTPOINT_X + CUTSIGN_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM + CUTSIGN_DIST ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor redColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		// rettangolo completo in nero
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		x2 = BACK_OUTER_HOR_DIM ;
		y2 = BACK_OUTER_VER_DIM ;
		tmpRect = [ [ CCE_Rect alloc] initWithId: [self getNextObjId] inView: self
			withRect:NSMakeRect(
						convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
						convertMeasure(y1, UNITS_CM, UNITS_PIXELS), 
						convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
						convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) 
			andAngle: 0 ];
		[ tmpRect setCceLineWidth: 1 ];
		[ tmpRect setCceLineColor: [ NSColor blackColor] ];
		[ tmpRect setCceIsStroked: TRUE ];
		[ tmpRect setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpRect atIndex: 0 ];
		// le due linee per le barre laterali del titolo
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_LABEL_HOR_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_LABEL_HOR_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceLineColor: [ NSColor blackColor] ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM - BACK_LABEL_HOR_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		x2 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM - BACK_LABEL_HOR_DIM ;
		y2 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y1, UNITS_CM, UNITS_PIXELS) )
			toPt: NSMakePoint(	 convertMeasure(x2, UNITS_CM, UNITS_PIXELS), 
								 convertMeasure(y2, UNITS_CM, UNITS_PIXELS) ) ];
		[ tmpLine setCceLineColor: [ NSColor blackColor] ];
		[ tmpLine setCceLineWidth: 1 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
	}
	if ( 0 )
	{
		CCE_Line		* tmpLine ;
		CCE_Rect		* tmpRect ;
		tmpRect = [ [ CCE_Rect alloc] initWithId: [self getNextObjId] inView: self
			withRect:NSMakeRect(100, 200, 100, 50) andAngle: 0 ];
		[ tmpRect setCceLineWidth: 1 ];
		[ tmpRect setCceLineColor: [ NSColor blackColor] ];
		[ tmpRect setCceIsStroked: TRUE ];
		[ tmpRect setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpRect atIndex: 0 ];

		tmpRect = [ [ CCE_Rect alloc] initWithId: [self getNextObjId] inView: self
			withRect:NSMakeRect(50, 50, 70, 20) andAngle: 10 ];
		[ tmpRect setCceLineWidth: 5 ];
		[ tmpRect setCceLineColor: [ NSColor blueColor] ];
		[ tmpRect setCceIsStroked: TRUE ];
		[ tmpRect setCceFillColor: [ NSColor redColor] ];
		[ tmpRect setCceIsFilled: TRUE ];
		[ tmpRect setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpRect atIndex: 0 ];

		tmpLine = [ [ CCE_Line alloc] initWithId: [self getNextObjId] inView: self
			fromPt: NSMakePoint( 70, 70 ) toPt: NSMakePoint( 170, 170 ) ];
		[ tmpLine setCceLineColor: [ NSColor blueColor] ];
		[ tmpLine setCceLineWidth: 2 ];
		[ tmpLine setCceIsLocked: NO ];
		[ [self theElements] addElem: tmpLine atIndex: 0 ];
	}
	if ( 1 )
	{   // aggiungo il testo
		CCE_Text		* tmpText ;
		NSFont			* locFont ;
		NSMutableAttributedString  * temp ;
		float			x1, y1 ;

		temp = [ [ NSMutableAttributedString alloc] initWithString: @"dummy" ];
		locFont =  [ NSFont fontWithName: @"Arial" size: 12 ] ;
		[ temp addAttribute: NSFontAttributeName value: locFont range: NSMakeRange(0, [ temp length] ) ];
		// calcolo il punto
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + CUTSIGN_DIM ;
		tmpText = [[ CCE_Text alloc] initWithId: [self getNextObjId] inView: self
			withAttributedString: temp 
			inRect: NSMakeRect(convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
					convertMeasure(y1, UNITS_CM, UNITS_PIXELS), 100, 100)
			withAngle: 0.0 ];
		[ tmpText setAutomaticText: CATAUTOTEXT_FILELIST ] ;
		// aggiungo l'elemento al vettore
		[ [self theElements] addElem: tmpText atIndex: 0 ];
	
		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_LABEL_HOR_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST ;
		temp = [ [ NSMutableAttributedString alloc] initWithString: @"Volume Name" ];
		[ temp setAttributes: defaultTextAttr( ) range: NSMakeRange(0, [temp length])];
		// calcolo il punto
		tmpText = [[ CCE_Text alloc] initWithId: [self getNextObjId] inView: self
			withAttributedString: temp 
			inRect: NSMakeRect(
					convertMeasure(x1, UNITS_CM, UNITS_PIXELS), 
					convertMeasure(y1, UNITS_CM, UNITS_PIXELS), 
					convertMeasure(BACK_LABEL_VER_DIM, UNITS_CM, UNITS_PIXELS),
					convertMeasure(BACK_LABEL_HOR_DIM, UNITS_CM, UNITS_PIXELS)  ) 
			withAngle: 90 ];
		[ tmpText setAutomaticText: CATAUTOTEXT_VOLNAME ] ;
		[ tmpText setCceFillColor: [ NSColor lightGrayColor] ];
		[ tmpText setCceIsFilled: TRUE ];
		// aggiungo l'elemento al vettore
		[ [self theElements] addElem: tmpText atIndex: 0 ];

		x1 = STARTPOINT_X + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_HOR_DIM - BACK_LABEL_HOR_DIM ;
		y1 = STARTPOINT_Y + CUTSIGN_DIM + CUTSIGN_DIST + BACK_OUTER_VER_DIM ;
		temp = [ [ NSMutableAttributedString alloc] initWithString: @"mod Name" ];
		[ temp setAttributes: defaultTextAttr( ) range: NSMakeRange(0, [temp length])];
		// calcolo il punto
		tmpText = [[ CCE_Text alloc] initWithId: [self getNextObjId] inView: self
			withAttributedString: temp 
			inRect: NSMakeRect(
					convertMeasure(x1, UNITS_CM, UNITS_PIXELS),
					convertMeasure(y1, UNITS_CM, UNITS_PIXELS),
					convertMeasure(BACK_LABEL_VER_DIM, UNITS_CM, UNITS_PIXELS),
					convertMeasure(BACK_LABEL_HOR_DIM, UNITS_CM, UNITS_PIXELS)  ) 
			withAngle: 270 ];
		[ tmpText setAutomaticText: CATAUTOTEXT_MODDATE ] ;
		[ tmpText setCceFillColor: [ NSColor lightGrayColor] ];
		[ tmpText setCceIsFilled: TRUE ];
		// aggiungo l'elemento al vettore
		[ [self theElements] addElem: tmpText atIndex: 0 ];
	}
	if ( 0 )
	{
		CCE_Image   * curElem = [[ [CCE_Image alloc ] init] autorelease ] ;
		[ curElem setObjID: [self getNextObjId]];
		[ curElem setOwnerView: self ];
		// deseleziono tutto
		[ theElements addElem:curElem atIndex: 0 ];
	}
}

@end

// ---------------------------------------------------------------------
// funzione per determinare il punto di aggancio; il punto dipende dalla
// attivazione o meno della griglia
NSPoint
getNearestSnapPoint( NSPoint point, CoverView * view)
{
	BOOL	gridOn = [ view snapToGrid ];

	// se la griglia non e' attiva, nessuna modifica al punto
	if ( gridOn == NO )
		return ( point );
	// con griglia attiva, cerco il punto piu' vicino
	return ( getNearestGridPoint(point, view) );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// funzione per trovare il piu' vicino punto della griglia
NSPoint
getNearestGridPoint( NSPoint point, CoverView * view)
{
	float		gridSpacing = [ view gridSpacing ];
	NSPoint retPt ;
	// arrotondo al multiplo intero piu' vicino
	retPt.x = round(point.x / gridSpacing) * gridSpacing ;
	retPt.y = round(point.y / gridSpacing) * gridSpacing ;
	return ( retPt );
}

