// -----------------------------------------------------------------------------
//  ElemParamWinCtl.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// finestra dei parametri di un elemento grafico

#import		"ElemParamWinCtl.h"

#import 	"CoverWinCtl.h"
#import 	"CoverView.h"
#import 	"FileStruct.h"
#import 	"AppDelegate.h"
#import 	"djZeroUtils.h"

// maschere per l'impostazione stato dei controlli
#define		ENABCTL_NULLENAB		0x0000
#define		ENABCTL_ELEMINFO		0x0001
#define		ENABCTL_ALIGN			0x0002
#define		ENABCTL_ROTATE			0x0004
#define		ENABCTL_TXTPARAM		0x0008
#define		ENABCTL_TRANSPIMG		0x0010

@implementation ElemParamWinCtl

// ---------------------------------------------------------------------
// istanza condivisa
+ (id)
sharedElemParamWinCtl
{
	static	ElemParamWinCtl	* locWinCtrl  = nil ;

	if ( ! locWinCtrl )
	{
		locWinCtrl = [[ ElemParamWinCtl allocWithZone: [self zone]] init ] ;
	}
	return ( locWinCtrl );
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
selectPanel: (NSString *) uot 
{
	[ thetab selectTabViewItemWithIdentifier: uot ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (id)
init
{
	self = [ self initWithWindowNibName: @"ElemParam" ];
	theSelectedElem = nil;
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
windowDidLoad
{
	[super windowDidLoad];
	[ self setSelectedElem: [ AppDelegate getTheSelectedElem ] ];
	// predisposizione dei vari elementi della finestra
	// informazioni elemento
	[fillCheckbox setState:NSOffState];
	[fillColorWell setColor:[NSColor whiteColor]];
	[lineCheckbox setState:NSOnState];
	[lineColorWell setColor:[NSColor blackColor]];
	[lineWidthSlider setFloatValue:0.0];
	[lineWidthTextField setFloatValue:0.0];
	[rotationSlider setFloatValue: 0.0 ];
	[rotationText setFloatValue: 0.0 ];
	[xTextField setStringValue:@""];
	[yTextField setStringValue:@""];
	[widthTextField setStringValue:@""];
	[heightTextField setStringValue:@""];
	// allineamento
	// nulla da predisporre
	// rotazione
	[ crcSlider setFloatValue: 0.0F ] ;
	[ linSlider setFloatValue: 0.0F ] ;
	[ valField setFloatValue: 0.0F ] ;
	[ self  setSelectedElem: [AppDelegate getTheSelectedElem ] ];
	// parametri testo
	[ autoTxtMenu selectItemAtIndex: 0 ] ;
	[ columnNum setIntValue: 1 ] ;
	// trasparenza immagine
	[ transpSlider setFloatValue: 1.0F ] ;
	[ transpText setFloatValue: 1.0F ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiusto lo stato dei controlli
- (void)
setControlsEnabled: (unsigned long) statusMask
{
	BOOL	status ;
	// informazioni elemento
	status = ( statusMask & ENABCTL_ELEMINFO ) ? YES : NO ;
	[fillCheckbox setEnabled: status ];
	[fillColorWell setEnabled: status ];
	[lineCheckbox setEnabled: status ];
	[lineColorWell setEnabled: status ];
	[lineWidthSlider setEnabled: status ];
	[lineWidthTextField setEnabled: status ];
	[rotationSlider setEnabled: status ];
	[rotationText setEnabled: status ];
	[xTextField setEnabled: status ];
	[yTextField setEnabled: status ];
	[widthTextField setEnabled: status ];
	[heightTextField setEnabled: status ];
	// allineamento
	status = ( statusMask & ENABCTL_ALIGN ) ? YES : NO ;
	[ selectedAlign setEnabled: status ];
	[ execBtn setEnabled: status ];
	// rotazione
	status = ( statusMask & ENABCTL_ROTATE ) ? YES : NO ;
	[ crcSlider setEnabled: status ];
	[ linSlider setEnabled: status ];
	[ linSlider setEnabled: status ];
	[ dirRadioBtn setEnabled: status ];
	// parametri testo
	status = ( statusMask & ENABCTL_TXTPARAM ) ? YES : NO ;
	[ autoTxtMenu	setEnabled: status ];
	[ columnNum		setEnabled: status ];
	// trasparenza immagine
	status = ( statusMask & ENABCTL_TRANSPIMG ) ? YES : NO ;
	[ transpSlider setEnabled: status ];
	[ transpText setEnabled: status ];
	[ makeImgOriginalsize setEnabled: status ] ;
}

// ---------------------------------------------------------------------
// aggiusto la finestra a seconda della selezione corrente
- (void)
setSelectedElem: (CCE_BasicForm *) selElem
{
	unsigned long maskSts ;
	
	theSelectedElem = selElem;
	// se non ci sono eselementi selezionati
	if ( theSelectedElem == nil )
	{
		// dico che se ne fa nulla
		[ elemId setStringValue: @"No graphic element selected"];
		[ self setControlsEnabled: ENABCTL_NULLENAB ] ;
		return ;
	}
	// ... o ce ne sono troppi
	else if ( theSelectedElem == (CCE_BasicForm*)(-1) )
	{
		[ elemId setStringValue: @"Too many graphic elements selected"];
		// attivo solo rotazione ed allineamento
		[ self setControlsEnabled: (ENABCTL_ALIGN | ENABCTL_ROTATE) ] ;
		return ;
	}
	// se arrivo qui, c'e' un solo elemento selezionato
	// attivo tutti gli elementi della finestra
	maskSts = (ENABCTL_ELEMINFO | ENABCTL_ROTATE) ;
	// i parametri testo se il caso
	if ( [theSelectedElem isKindOfClass:[CCE_Text class]] )
		maskSts += ENABCTL_TXTPARAM ;
	// i parametri immagine se il caso
	if ( [theSelectedElem isKindOfClass:[CCE_Image class]] )
		maskSts += ENABCTL_TRANSPIMG ;	
	// attivo o disattivo i controlli
	[ self setControlsEnabled: maskSts ] ;
	// aggiorno il contenuto dei vari campi
	[ self updateWindowInfo ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiornamento dei dati della finestra a seconda della selezione
- (void)
updateWindowInfo
{
	float	tmpVal, angle ;
	int		curUnit ;
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;

	curUnit = [cvrView curUnits ] ;
	[ curUnits setStringValue: stringForUnits(curUnit) ];
	// predispongo gli elementi che corrispondono agli attributi comuni
	[fillCheckbox setState: [ theSelectedElem cceIsFilled] ];
	[fillColorWell setColor: [ theSelectedElem cceFillColor] ];
	[lineCheckbox setState: [ theSelectedElem cceIsStroked] ];
	[lineColorWell setColor:[ theSelectedElem cceLineColor] ];
	// limite massimo e minimo nelel unita' correnti
	[lineWidthSlider setMinValue: 0 ];
	tmpVal = convertMeasure( 10.0, UNITS_PIXELS, curUnit );
	[lineWidthSlider setMaxValue: tmpVal ];
	// valore corrente dello slider
	tmpVal = convertMeasure( [ theSelectedElem cceLineWidth], UNITS_PIXELS, curUnit );
	[lineWidthSlider setFloatValue: tmpVal ];
	[lineWidthTextField setFloatValue: tmpVal ];
	[rotationSlider setFloatValue: [ theSelectedElem rotAngle]];
	[rotationText setFloatValue: [ theSelectedElem rotAngle]];
	// prima calcolo il valore secondo le unita', poi lo assegno
	tmpVal = convertMeasure( [theSelectedElem drawPoint].x, UNITS_PIXELS, curUnit );
	[xTextField setStringValue: [ NSString stringWithFormat: @"%f", tmpVal ] ];
	tmpVal = convertMeasure( [theSelectedElem drawPoint].y, UNITS_PIXELS, curUnit );
	[yTextField setStringValue: [ NSString stringWithFormat: @"%f",tmpVal ] ];
	tmpVal = convertMeasure( [theSelectedElem localSize].width, UNITS_PIXELS, curUnit );
	[widthTextField setStringValue: [ NSString stringWithFormat: @"%f", tmpVal ] ];
	tmpVal = convertMeasure( [theSelectedElem localSize].height , UNITS_PIXELS, curUnit );
	[heightTextField setStringValue: [ NSString stringWithFormat: @"%f", tmpVal ] ];
	// preparo una scritta a seconda dell'elemento
	[ elemId setStringValue:  [ theSelectedElem describeMe ] ];
	// rotazione
	angle = [ theSelectedElem rotAngle ] ;
	if ([ [ dirRadioBtn selectedCell] tag ] == 96)
		angle = 360 - angle ;
	// con l'angolo di quell'elemento, imposto gli slider
	[ crcSlider setFloatValue: angle ] ;
	[ linSlider setFloatValue: angle ] ;
	[ valField  setFloatValue: angle ] ;
	if ( [theSelectedElem isKindOfClass:[CCE_Text class]] )
	{
		CCE_Text	* selez = (CCE_Text*)theSelectedElem ;
		// se arrivo qui, un solo elemento selezionato testo
		switch ( [ selez automaticText] ) {
		case CATAUTOTEXT_FILELIST :
			[ autoTxtMenu selectItemAtIndex: 2 ] ;
			break ;
		case CATAUTOTEXT_VOLNAME :
		case CATAUTOTEXT_MODDATE :
		case CATAUTOTEXT_CREATDATE :
		case CATAUTOTEXT_VOLSIZE :
		case CATAUTOTEXT_VOLFREESIZE :
			[ autoTxtMenu selectItemAtIndex: ([ selez automaticText]-CATAUTOTEXT_VOLNAME+4) ] ;
			break ;
		default :
			[ autoTxtMenu selectItemAtIndex: 0 ] ;
		}
		[ columnNum setIntValue: [ selez numOfColumns] ] ;
	
	}
	if ( [theSelectedElem isKindOfClass:[CCE_Image class]] )
	{
		CCE_Image	* selez = (CCE_Image*)theSelectedElem ;
		// attivo i controlli
		[ transpSlider setFloatValue: [ selez compFraction ] ] ;
		[ transpText setFloatValue: [ selez compFraction ] ] ;
	}
}

// ---------------------------------------------------------------------
// pannello info
// ---------------------------------------------------------------------
// metodo attivato dopo una modifica di un attributo
- (IBAction)
changedElem:(id)sender
{
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;
	float		p1, p2 ;
	// a seconda dell'attributo modificato
	switch ( [ sender tag ] ) {
	case 10 : 	// stato di riempimento
		[ theSelectedElem setCceIsFilled: [ fillCheckbox state ]];
		break ;
	case 11 : 	// colore di riempimento
		[ theSelectedElem setCceFillColor: [ fillColorWell color ]];
		break ;
	case 12 : 	// stato di disegno linea/contorno
		[ theSelectedElem setCceIsStroked: [ lineCheckbox state ]];
		break ;
	case 13 : 	// colore della linea
		[ theSelectedElem setCceLineColor: [ lineColorWell color ]];
		break ;
	case 14 : 	// dimensione della linea
	case 15 :
		// riconverto la dimensione in pixel
		p1 = convertMeasure( [ lineWidthSlider floatValue ] , [cvrView curUnits ], UNITS_PIXELS ) ;
		[ theSelectedElem setCceLineWidth: p1 ];
		break ;
	case 16 : 	// origine dell'elemento
	case 17 :
		if ( [ theSelectedElem cceIsLocked] ) break ;
		// riconverto la dimensione in pixel
		p1 = convertMeasure( [xTextField floatValue ] , [cvrView curUnits ], UNITS_PIXELS ) ;
		p2 = convertMeasure( [yTextField floatValue ] , [cvrView curUnits ], UNITS_PIXELS ) ;
		[ theSelectedElem setDrawPoint: NSMakePoint( p1, p2) ];
		[ theSelectedElem calcLocalTranform ];
		break ;
	case 18 :	// dimensioni dell'elemento
	case 19 :
		if ( [ theSelectedElem cceIsLocked] ) break ;
		// riconverto la dimensione in pixel
		p1 = convertMeasure( [widthTextField floatValue ] , [cvrView curUnits ], UNITS_PIXELS ) ;
		p2 = convertMeasure( [heightTextField floatValue ] , [cvrView curUnits ], UNITS_PIXELS ) ;
		[theSelectedElem resizeMeWith: NSMakeSize( p1, p2 ) ];
		break ;
	case 20 : 	// angolo di rotazione
	case 21 :
		if ( [ theSelectedElem cceIsLocked] ) break ;
		[ theSelectedElem rotateMeAtAngle: [ rotationSlider floatValue ] ];
		break ;
	default:
		NSLog( @"changed %@", sender );
	}
	// forzo un ridisegno
	[ [theSelectedElem ownerView] setNeedsDisplay: YES ];
	// ed anche un aggiornamento delle informazioni
	[ self updateWindowInfo ];
}

// ---------------------------------------------------------------------
// lo slider dello spessore linea va gestito a mano assieme al testo
- (IBAction)
slideChange:(id)sender
{
	// recupero il valore corrente
	float   newVal = [ sender floatValue ];
	// limite massimo nell'unita' corrente
	float	tmpVal = convertMeasure( 10.0, UNITS_PIXELS, [[AppDelegate getTheDrawView ] curUnits ] );
	if ( newVal > tmpVal ) newVal = tmpVal ;
	// imposto il valore per entrambi
	[ lineWidthTextField setFloatValue: newVal ] ;
	[ lineWidthSlider setFloatValue: newVal ] ;
	// dico che e' cambiato il valore di un attributo
	[ self changedElem: sender ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// lo slider della rotazione va gestito a mano assieme al testo
- (IBAction)
rotationChange:(id)sender
{
	// recupero il valore corrente
	float   newVal = [ sender floatValue ];
	// limite massimo
	if ( newVal > 360 ) newVal = 360 ;
	if ( newVal < 0 ) newVal = 0 ;
	// imposto il valore per entrambi
	[ rotationSlider setFloatValue: newVal ] ;
	[ rotationText setFloatValue: newVal ] ;
	// dico che e' cambiato il valore di un attributo
	[ self changedElem: sender ];
}

// ---------------------------------------------------------------------
// allineamento
// ---------------------------------------------------------------------
// esecuzione di un centraggio o di allineamento
- (IBAction)
executeAlign:(id)sender
{
	CoverView   * cv = [AppDelegate getTheDrawView ] ;
	// se c'e' una potenziale finestra interessata
	if ( cv )
	{
		// recupero il tag della cella selezionata
		int selTag = [[ selectedAlign selectedCell] tag] ;
		// tag 5 e 6 stanno per centraggio, gli altri per l'allineamento
		if ( selTag == 5 || selTag == 6 )
				[ cv executeCentering: selTag ];
		else	[ cv executeAlignment: selTag ];
	}
}

// ---------------------------------------------------------------------
// rotazione elementi
// ---------------------------------------------------------------------
// qui comincia il processo di rotazione degli elementi selezionati
- (IBAction)
execRotation:(id)sender
{
	CoverView   * cv = [AppDelegate getTheDrawView ] ;
	float		angle1, angle2 ;
	// non dovrebbe mai succedere...
	if ( cv == nil ) return ;

	angle1 = [ linSlider floatValue ] ;
	angle2 = ([ [ dirRadioBtn selectedCell] tag ] == 95) ? angle1 : -angle1 ;
	[ [ cv theElements] rotateElemAngle: angle2 absolute: YES ] ;
	// dopo la rotazione, occorre aggiornare la selezione corrente
	// nota che l'oggetto di riferimento e' la coverView
	[ [NSNotificationCenter defaultCenter]
			postNotificationName:@"CoverViewChangedSelection"
			object: cv ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiornamento congiunto degli slider e rotazione dal vivo
- (IBAction)
updateSliders:(id)sender
{
	float   value = [ sender floatValue ];

	[ crcSlider setFloatValue: value ] ;
	[ linSlider setFloatValue: value ] ;
	[ valField setFloatValue: value ] ;
	[ self execRotation: sender ];
}

// ---------------------------------------------------------------------
// parametri testo
// ---------------------------------------------------------------------
// aggiusto i parametri dell'elemento testo
- (IBAction)
setTextParams:(id)sender
{
	CoverView		* cv = [AppDelegate getTheDrawView ] ;
	CCE_BasicForm	* selImg = [AppDelegate getTheSelectedElem ] ;
	// non dovrebbe mai succedere...
	if ( cv == nil ) return ;
	if ( selImg == nil || ( selImg == (CCE_BasicForm*)(-1) ) ) return ;
	if ( [selImg isKindOfClass:[CCE_Text class]] )
	{
		CCE_Text	* selez = (CCE_Text*)selImg ;
		int			noc = [ columnNum intValue ];
		int			at = [ [ autoTxtMenu selectedItem ] tag ] ;
		// il numero di colonne e' sempre tra 1 e 8 (perche' si')
		if ( noc < 1 ) noc = 1 ;
		if ( noc > 8 ) noc = 8 ;		
		[ selez setNumOfColumns: noc ] ;
		[ selez setAutomaticText: at ] ;
		// riaggiusto la visualizzazione del tutto
		[ selez adjustTextColumn ] ;
		[ selez updAutoText: [[ [cv winCtl] document] dataSource ] ] ;
		// forzo un ridisegno
		[ cv setNeedsDisplay: YES ];
	}
}

// ---------------------------------------------------------------------
// trasparenza immagini
// ---------------------------------------------------------------------
- (IBAction)
setTransparency:(id)sender
{
	float		newComp = [ sender floatValue ];
	CoverView   * cv = [AppDelegate getTheDrawView ] ;
	CCE_BasicForm	* selImg = [AppDelegate getTheSelectedElem ] ;

	[ transpSlider setFloatValue: newComp ] ;
	[ transpText setFloatValue: newComp ] ;
	// non dovrebbe mai succedere...
	if ( cv == nil ) return ;
	if ( selImg == nil || ( selImg == (CCE_BasicForm*)(-1) ) ) return ;
	if ( [selImg isKindOfClass:[CCE_Image class]] )
	{
		CCE_Image	* selez = (CCE_Image*)selImg ;
		[ selez setCompFraction: newComp ] ;
		[ [ selez ownerView] setNeedsDisplay:  YES ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
setOriginalSize:(id)sender
{
	CoverView   * cv = [AppDelegate getTheDrawView ] ;
	CCE_BasicForm	* selImg = [AppDelegate getTheSelectedElem ] ;

	// non dovrebbe mai succedere...
	if ( cv == nil ) return ;
	if ( selImg == nil || ( selImg == (CCE_BasicForm*)(-1) ) ) return ;
	if ( [selImg isKindOfClass:[CCE_Image class]] )
	{
		CCE_Image	* selez = (CCE_Image*)selImg ;
		[ selez setOriginalSize ] ;
		[ [ selez ownerView] setNeedsDisplay:  YES ];
	}
}

@end
