// -----------------------------------------------------------------------------
//  FileStruct.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della classe che modella le informazioni gerarchiche

#import		"FileStruct.h"

#import		"djZeroUtils.h"
#import		"WaitPanCtrl.h"
#import		"FilterWinCtl.h"
#import		"CatDataSrc.h"

// funzione per la ricerca con determinate opzioni
BOOL	checkCriterio( FileStruct * item, FltElemPtr fltCrit, int logic ) ;
BOOL	initWithLogic ( int logic ) ;
BOOL	setWithLogic ( BOOL val1, BOOL val2, int logic ) ;
BOOL	stopWithLogic( BOOL val, int logic ) ;

@implementation FileStruct

// ---------------------------------------------------------------------
// inizializzazione dell'istanza in base al nome completo
- (id)
initTreeFromPath: (NSString*) fullPath
{
	// mi metto via un file manager
	NSFileManager 	*fileManager = [NSFileManager defaultManager];
	BOOL			isAdir, fileOK ;
	int				i ;

	// per prima cosa, inizializzo super
	[ super initWithPath: fullPath ] ;
	// e adesso, se il file puntato e' una directory, espando
	fileOK = [ fileManager fileExistsAtPath: fullPath isDirectory: & isAdir ];
	// evito tuttavia di espandere link simbolici (che NON sono alias...)
	if ( [ [self fileType] isEqual: NSFileTypeSymbolicLink ])
		isAdir = FALSE ;
	[ self setOrderColumn: COLID_FILENAME ];
	orderDirection = TRUE ;
	// espando sempre, purche' il file esista e sia una directory effettiva
	if ( fileOK && isAdir)
	{
		// variabile per tenere traccia della dimensione parziale
		long long		dirSize = 0;
		// vettore destinato a contenere i file
		NSMutableArray 	*tmpfileList  ;
		// l'elenco dei file
		NSArray			*dirContent = [ fileManager directoryContentsAtPath: fullPath ];
		// conto quanti file ci sono all'interno
		int				numFile = [ dirContent count ];
		// costruisco un vettore di dimensione adatta
		tmpfileList = [[ NSMutableArray alloc ] initWithCapacity: numFile ] ;
		[ WaitPanCtrl setText: [NSString stringWithFormat: @"Cataloging: %@", [self fileName] ]];
		// adesso, per ogni elemento, costruisco l'albero e poi lo aggiungo
		for ( i = 0; i < numFile; i++)
		{
			FileStruct	* tmpFile ;
			NSString 	* myfile = [ dirContent objectAtIndex: i ] ;
			// costruisco l'albero
			tmpFile = [[FileStruct alloc]  initTreeFromPath:
				[ fullPath stringByAppendingPathComponent: myfile]] ;
			// aggiungo la dimensione (cumulata) alla directory corrente
			dirSize += [ tmpFile fileSize ];
			// aggiungo il file alla lista
			[tmpfileList addObject: tmpFile ];
		}
		// assegna al file la nuova dimensione
		[ self setFileSize: dirSize ];
		// adesso copio il vettore con la giusta dimensione
		fileList = [[ NSMutableArray alloc ] initWithCapacity: [ tmpfileList count] ];
		[ fileList setArray: tmpfileList ];
		[ self sortFileList: COLID_FILENAME direction: TRUE ];
	}
	else
	{
		// va beh, e' un file normale
		fileList = nil ;
	}
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
dealloc
{
	// devo distruggere solo il vettore con i file
	[ fileList release ] ;
	[ orderColumn release ];
	[ super dealloc ];
}

// ---------------------------------------------------------------------
// metodo per il salvataggio/ripristino dell'istanza da file
- (void)
encodeWithCoder:(NSCoder *)encoder
{
	int dim ;
	// salvo le variabili ereditate
	[ super encodeWithCoder: encoder ];
	// vedo quanti file ci sono
	dim = [ self numOfFiles] ;
	// metto da parte questo numero
	[ encoder encodeValueOfObjCType: @encode(int) at: & dim];
	// se ci sono file dipendenti, li salvo tutti
	if ( dim > 0 )
		[ encoder encodeObject: [ self fileList ]];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo per il salvataggio/ripristino dell'istanza da file
- (id)
initWithCoder:(NSCoder *)decoder
{
	int dim ;
	// recupero le variabili ereditate
	[ super initWithCoder: decoder ];
	// vedo se ci sono file dipendenti
	[ decoder decodeValueOfObjCType: @encode(int) at: & dim];
	if ( dim <= 0 )
			[ self setFileList: nil ]; // non ce ne sono
		// recupero le info di tutti i file contenuti
	else 	[ self setFileList: [ decoder decodeObject ]];
	return self ;
}

// ---------------------------------------------------------------------
// metodo di comodo per recuperare un file al posto i-simo
- (FileStruct *)
getFileAtIndex:(int) elem
{
	return [[ self fileList ] objectAtIndex: elem ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metodo di comodo per vedere quanti file sono presenti nella directory
- (int)
numOfFiles
{
	return ( [[self fileList] count] ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (void)
sortFileList: (NSString*) type
	direction: (BOOL) dir
{
	NSArray * newArray ;
	NSMutableArray * tmpArray ;

	// se non ci sono elementi da ordinare, abbiamo finito
	if ( ! [self fileList]  ) return ;
	// il vettore e' gia' ordinato se chiave e direzione coincidono
	if ( [ orderColumn isEqual: type ] && dir == orderDirection )
		return ;
	// altrimenti, bisogna ordinare il vettore; uso una delle due funzioni
	// a seconda della direzione di ordinamento
	if ( dir == TRUE )
			newArray = [ [self fileList] sortedArrayUsingFunction: ascSortFunc context: type] ;
	else	newArray = [ [self fileList] sortedArrayUsingFunction: desSortFunc context: type] ;
	// trasformo il risultato in un NSMutableArray
	tmpArray = [ NSMutableArray arrayWithCapacity: [ newArray count]];
	[ tmpArray addObjectsFromArray: newArray ];
	// assegno le nuove variabili d'istanza
	[ self setFileList: tmpArray];
	[ self setOrderColumn: type ];
	orderDirection = dir ;
	return ;
}

// ---------------------------------------------------------------------
// metodi accessor
- (NSMutableArray *)fileList {
	return [[fileList retain] autorelease];
}

- (void)setFileList:(NSMutableArray *)newFileList {
	if (fileList != newFileList) {
		[fileList release];
		fileList = [newFileList retain];
	}
}

- (NSString *)orderColumn {
	return [[orderColumn retain] autorelease];
}

- (void)setOrderColumn:(NSString *)newOrderColumn {
	orderColumn = newOrderColumn ;
}

- (BOOL)orderDirection {
	return orderDirection;
}

- (void)setOrderDirection:(BOOL)newOrderDirection {
	orderDirection = newOrderDirection ;
}

// ---------------------------------------------------------------------
// gestione dei criteri di selezione su questo elemento
- (void)
critSelect: (FltElemPtr) fltCrit withLogic: (int) logic 
		oper: (int) operaz placeIn: (NSMutableArray *) where 
{
	FileStruct	* item ;
	NSEnumerator 	* enumerator = [[self fileList] objectEnumerator];
	// in ogni caso, vedo se l'elemento corrente va bene
	BOOL	isok = checkCriterio( self, fltCrit, logic );
	// intanto inserisco l'elemento filtrato nel risultato
	// se la modalita' e' ricerca, ho anche finito
	if ( isok )
		[ where addObject: self ] ;
	// se sono su filtro stampa, distinguo i casi
	if ( operaz == FILTER_OPER_SETPRINT )
	{
		// assegno lo stato secondo valutazione
		[ self setIs2Print: isok ] ;
	}
	if ( operaz == FILTER_OPER_ADDPRINT )
	{
		// aggiungo elementi alla stampa
		if ( isok || is2Print )
			[ self setIs2Print: YES ] ;
	}
	if ( operaz == FILTER_OPER_REMPRINT )
	{
		// rimuovi gli elmenti dalal stampa
		if ( isok && is2Print )
			[ self setIs2Print: NO ] ;
	}
	// esploro i figli	
	while (item = [enumerator nextObject])
	{
		[ item critSelect: fltCrit withLogic: logic oper: operaz placeIn: where ];
	}
}

// ---------------------------------------------------------------------
// espande tutti gli elementi fino ad un elemento obiettivo
- (void)
expandFStoItem: (NSEnumerator*) pathComp inOutlineView: (NSOutlineView *) outView
{
	// piglio l'elenco dei file contenuti
	NSEnumerator 	* enumerator = [[self fileList] objectEnumerator];
	// questo e' il (pezzo di) nome dell'elemento obiettivo
	NSString	* name = [ pathComp nextObject ];
	FileStruct	* item ;

	// esamino tutti gli elementi presenti
	while (item = [enumerator nextObject])
	{
		// se il nome coincide con quello cercato
		if ( [[item fileName] isEqual:  name] )
		{
			int	thisRow = [outView rowForItem: item];
			// dico che questo item va espanso perche' fa parte della
			// catena che porta all'obiettivo
			[ outView expandItem: item ];
			// lo seleziono
			[ outView selectRow: thisRow  byExtendingSelection: FALSE ];
			// e lo porto in bella vista
			[ outView scrollRowToVisible: thisRow ];
			// poi proseguo nell'espansione
			[ item expandFStoItem: pathComp inOutlineView: outView] ;
			// non devo esplorare gli altri elementi...
			return ;
		}
	}
}

@end

// ---------------------------------------------------------------------
// qualche funzioni per operazioni logiche
// restituiscono valori booleani per logica in and o in or

// inizializzo una variabile per operazioni in and oppure or
// per and: inizializzo a YES, per or, a NO
BOOL
initWithLogic ( int logic )
{
	if ( logic == FILTER_LOGIC_AND )
		return ( YES ) ;
	return ( NO );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// combinazioen di due variabili, in and o in or
BOOL
setWithLogic ( BOOL val1, BOOL val2, int logic )
{
	if ( logic == FILTER_LOGIC_AND )
		return ( val1 && val2 ) ;
	return ( val1 || val2 );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// restituisce vero se si possono smettere di valutare condizioni in
// and (perche' e' definitivamente NO) o in or (definitamente YES)
BOOL
stopWithLogic( BOOL val, int logic )
{
	if ( logic == FILTER_LOGIC_AND )
		return ( val == NO ) ;
	return ( val == YES );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// determina se un elemento risponde al criterio di ricerca
BOOL
checkCriterio( FileStruct * item, FltElemPtr fltCrit, int logic )
{
	NSComparisonResult	xxx ;
	NSRange				xx ;
	NSString			*s1 ;
	NSNumber			* n1 ;
	long long			tmp1, tmp2 ;
	long				tmp3, tmp4 ;
	int					i ;
	BOOL				tmpVal ;
	NSArray				* id_string = [NSArray arrayWithObjects:
		COLID_FILENAME, COLID_MODDATE, COLID_CREATDATE, COLID_FILESIZE,
		COLID_GROUPNAME, COLID_OWNERNAME, COLID_POSIXPERM, 
		COLID_OSCREATOR, COLID_OSTYPE,nil ];
	// inizializzo il risultato con valore di partenza
	BOOL				select = initWithLogic( logic) ;
	// esamino ordinatamente tutti i criteri
	for ( i = 0 ; i < 10 ; i++ )
	{
		// sentinella
		if ( fltCrit[i].fieldId == -1 )
			break ;
		// le operazioni dipendono dal campo prescelto
		switch ( fltCrit[i].fieldId ) {
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_FILENAME:			// sono tutte stringhe
		case MENUTAG_GROUPNAME :		// cfr: uguale e diverso
		case MENUTAG_OWNERNAME :
			// ricavo la striga
			s1 = [ item valueForKey: [ id_string objectAtIndex: fltCrit[i].fieldId] ];
			// a seconda delle opzioni, faccio il confronto
			switch ( fltCrit[i].operId ) {
			case  MENUTAG_STR_ISEQUAL :
				tmpVal = [ s1 isEqualToString: fltCrit[i].theValue ] ;
				// confronto esatto tra stringhe
				break ;
			case MENUTAG_STR_ISCONTAINED :
				xx = [ s1 rangeOfString: fltCrit[i].theValue ];
				tmpVal = (xx.location != NSNotFound) ;
				break;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break ;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_POSIXPERM:			// sono numeri, ma con strana
		case MENUTAG_OSCREATOR :		// rappresentazione; 
		case MENUTAG_OSTYPE :			// cfr: uguale e diverso
			// ricavo il numero
			n1 = [ item valueForKey: [ id_string objectAtIndex: fltCrit[i].fieldId] ];
			tmp3 = [ n1 longValue ] ;
			tmp4 = [ fltCrit[i].theValue longValue ] ;
			// a seconda delle opzioni, faccio il confronto
			switch ( fltCrit[i].operId ) {
			case MENUTAG_NUM_EQ :
				tmpVal = ( tmp3 == tmp4 ) ;
				break;
			case MENUTAG_NUM_DIFFERENT :
				tmpVal = ( tmp3 != tmp4 ) ;
				break;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break ;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_MODDATE :		// una data, confronto pieno
			xxx = ( [ [ item modDate] compare: fltCrit[i].theValue] ) ;
			switch ( fltCrit[i].operId ) {
			case MENUTAG_NUM_EQ :
				tmpVal = (xxx == NSOrderedSame) ;
				break;
			case MENUTAG_NUM_DIFFERENT :
				tmpVal = (xxx != NSOrderedSame) ;
				break;
			case MENUTAG_NUM_GREAT :
				tmpVal = (xxx == NSOrderedDescending) ;
				break;
			case MENUTAG_NUM_LESS :
				tmpVal = (xxx == NSOrderedAscending) ;
				break;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_CREATDATE :		// una data, confronto pieno
			xxx = ( [ [ item creatDate] compare: fltCrit[i].theValue] ) ;
			switch ( fltCrit[i].operId ) {
			case MENUTAG_NUM_EQ :
				tmpVal = (xxx == NSOrderedSame) ;
				break;
			case MENUTAG_NUM_DIFFERENT :
				tmpVal = (xxx != NSOrderedSame) ;
				break;
			case MENUTAG_NUM_GREAT :
				tmpVal = (xxx == NSOrderedDescending) ;
				break;
			case MENUTAG_NUM_LESS :
				tmpVal = (xxx == NSOrderedAscending) ;
				break;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_FILESIZE :		// un numero, confronto pieno
			// ricavo le due dimensioni
			tmp1 = [ item fileSize ] ;
			tmp2 = (long long) [ fltCrit[i].theValue doubleValue ] ;
			// e le confronto brutalmente secondo l'opzione
			switch ( fltCrit[i].operId ) {
			case MENUTAG_NUM_EQ :
				tmpVal = ( tmp1 == tmp2 ) ;
				break;
			case MENUTAG_NUM_DIFFERENT :
				tmpVal = ( tmp1 != tmp2 ) ;
				break;
			case MENUTAG_NUM_GREAT :
				tmpVal = ( tmp1 > tmp2 ) ;
				break;
			case MENUTAG_NUM_LESS :
				tmpVal = ( tmp1 < tmp2 ) ;
				break;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break ;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		case MENUTAG_SPECIAL :		// confronti sparsi...
			switch ( fltCrit[i].operId ) {
			case MENUTAG_FOLDER :	// figli > 0
				tmpVal = [ item numOfFiles]> 0 ;
				break ;
			case MENUTAG_BUNDLE :	// funzione apposta
				tmpVal = checkIfBundleDirectory( [item fileName]) ;
				break ;
			case MENUTAG_SIMPLEFILE :	// figlio = 0
				tmpVal = [ item numOfFiles] == 0 ;
				break ;
			case MENUTAG_ALLFILES :	// sempre YES
				tmpVal = YES ;
				break ;
			case MENUTAG_NONE :		// sempre NO
				tmpVal = NO ;
				break ;
			default:	// non succede, ma in caso ...
				tmpVal = NO ;
				break ;
			}
			select = setWithLogic( select, tmpVal, logic );
			break;
		// -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
		default:		// non succede, ma in caso ...
			select = NO ;
			break ;
		}
		// verifico se vale la pena continuare
		if ( stopWithLogic( select, logic ) )
			break ;
	}
	// restituisco la valutazione di tutti i criteri
	return ( select );
}
