// -----------------------------------------------------------------------------
//  LocalFormatters.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// Interfaccia delle classi formatter

#import 	"LocalFormatters.h"

// ---------------------------------------------------------------------
@implementation TOS9TCForm

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per convertire un OSType -- creatore e tipo di MacOS9 - in stringa
// (long che contiene in realta' quattro char)

- (NSString *)
stringForObjectValue:	(id)anObject
{

	// controllo che l'oggetto sia un numero...
	if ([anObject isKindOfClass:[NSString class]]) {
		return anObject;
	}
	if (![anObject isKindOfClass:[NSNumber class]]) {
		return nil;
	}
	// la documentazione Apple afferma che e' meglio usare la funzione
	// sotto indicata piuttosto che fare da soli...
	return ( NSFileTypeForHFSTypeCode( [ anObject longValue ]) );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// a partire dalla stringa ricavo il valore
- (BOOL)
getObjectValue:			(id *)anObject
	forString:		(NSString *)string
	errorDescription:	(NSString **)error
{
	// conversione con funzione disponibile
	OSType	xx = NSHFSTypeCodeFromFileType( string ) ;
	// l'oggetto e' un banale numero
	* anObject = [ NSNumber numberWithLong: xx ] ;
	// devo ritornare YES per dire che e' andato bene
	return ( YES );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (NSAttributedString *)
attributedStringForObjectValue:	(id)anObject
	withDefaultAttributes:	(NSDictionary *)attributes
{
	return ( nil);
}
@end

// ---------------------------------------------------------------------
@implementation FileSizeForm

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// funzione per convertire un numero long, che rappresenta una dimensione
// in byte, in qualcosa di piu' gradevole
// se dim < 1K, scrivo byte; se dim < 1Mega, scrivo K; eccetera

- (NSString *)
stringForObjectValue:	(id)anObject
{
	unsigned long long	fSize, fSizeK ;
	float 			fsizeM, fsizeG ;

	// controllo che l'oggetto sia un numero...
	if ([anObject isKindOfClass:[NSString class]]) {
		return anObject;
	}
	if (![anObject isKindOfClass:[NSNumber class]]) {
		return nil;
	}
	// ricavo la dimensione del file
	fSize = [ ((NSNumber*) anObject) longLongValue ];
	// se il file e' piccolo, mostro byte
	if ( fSize < 1024 )
	{
		long	tmp = fSize ;
		return ( [ NSString stringWithFormat: @" %4d b", tmp] );
	}
	// la dimensione e' in byte, divido per 1024 ed arrotondo, ottengo K
	fSizeK = (long) (( fSize / 1024.0 ) + 0.5 );
	// se il file e' medio, mostro K
	if ( fSizeK < 1024 )
	{
		long	tmp = fSizeK ;
		return ( [ NSString stringWithFormat: @" %4d K", tmp] );
	}
	// se inferiore al giga, restituisco mega con tre decimali
	fsizeM = ( fSizeK / 1024.0 ) ;
	if ( fsizeM < 1024 )
		return ( [ NSString stringWithFormat: @" %6.3f M", fsizeM] );
	// negli altri casi, ritorno Giga, con tre cifre decimali
	fsizeG = ( fsizeM / 1024.0 ) ;
	return ( [ NSString stringWithFormat: @" %6.3f G", fsizeG] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// a partire dalla stringa ricavo il valore
- (BOOL)
getObjectValue:			(id *)anObject
	forString:		(NSString *)string
	errorDescription:	(NSString **)error
{
	NSRange	rr ;
	float	xx, scale ; 
	// dalla stringa estraggo l'eventuale numero presente
	xx = [ string floatValue ] ;
	// cerco di ricavare l'unita' di misura guardando se ci sono 
	// sottostringhe di soli caratteri
	rr = [ string rangeOfCharacterFromSet: [ NSCharacterSet letterCharacterSet ]
		options: NSCaseInsensitiveSearch ] ;
	// se non trovo sottostringhe letterali, dco niente unita'
	scale = 1 ;
	// se invece ho trovato qualcosa, calcolo la scala
	if ( rr.location != NSNotFound )
	{
		// vedo che razza di carattere potrei avere
		unichar cc = [ string characterAtIndex: rr.location ] ;
		// tra quelli sensati (byte, Kappa, Mega, Giga)
		if ( cc == 'b' )
			scale = 1 ;
		else if ( cc == 'K' )
			scale = 1024 ;
		else if ( cc == 'M' )
			scale = 1024 * 1024 ;
		else if ( cc == 'G' )
			scale = 1024 *1024 * 1024 ;
	}
	// calcaolo la dimensione in byte e costruisco un NSNumber
	* anObject = [ NSNumber numberWithLong: (long)(xx * scale) ] ;
	return ( YES );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (NSAttributedString *)
attributedStringForObjectValue:	(id)anObject
	withDefaultAttributes:	(NSDictionary *)attributes
{
	return ( nil);
}
@end

// ---------------------------------------------------------------------
@implementation FilePosixPerm

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per rappresentare i permessi posix di lettura/scrittura
// devo convertire un numero long nei permessi posix:

- (NSString *)
stringForObjectValue:	(id)anObject
{
	char		permstring[10] ;
	unsigned long 	perm ;

	// controllo che l'oggetto sia un numero...
	if ([anObject isKindOfClass:[NSString class]]) {
		return anObject;
	}
	if (![anObject isKindOfClass:[NSNumber class]]) {
		return nil;
	}
	// ricavo il long che mi da i permessi
	perm = [ anObject longValue ] ;
	// e poi e' tutto un sottile gioco di bit
	permstring[0] = ( perm & 0x0100 ) ? 'r' : '-' ;
	permstring[1] = ( perm & 0x0080 ) ? 'w' : '-' ;
	permstring[2] = ( perm & 0x0040 ) ? 'x' : '-' ;
	permstring[3] = ( perm & 0x0020 ) ? 'r' : '-' ;
	permstring[4] = ( perm & 0x0010 ) ? 'w' : '-' ;
	permstring[5] = ( perm & 0x0008 ) ? 'x' : '-' ;
	permstring[6] = ( perm & 0x0004 ) ? 'r' : '-' ;
	permstring[7] = ( perm & 0x0002 ) ? 'w' : '-' ;
	permstring[8] = ( perm & 0x0001 ) ? 'x' : '-' ;
	// chiudo la stringa C
	permstring[9] = 0 ;
	// converto la stringa C in NSString e la restituisco
	return ( [ NSString stringWithCString: permstring ] );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// a partire dalla stringa ricavo il valore
- (BOOL)
getObjectValue:			(id *)anObject
	forString:		(NSString *)string
	errorDescription:	(NSString **)error
{
	char		permstring[10] ;
	long		xx = 0 ;
	// dalla stringa estraggo una stringa C
	[ string getCString: permstring ] ;
	// esamino i singoli caratteri e vedo cosa salta fuori
	if ( permstring[0] == 'r' ) xx |= 0x0100 ;
	if ( permstring[1] == 'w' ) xx |= 0x0080 ;
	if ( permstring[2] == 'x' ) xx |= 0x0040 ;
	if ( permstring[3] == 'r' ) xx |= 0x0020 ;
	if ( permstring[4] == 'w' ) xx |= 0x0010 ;
	if ( permstring[5] == 'x' ) xx |= 0x0008 ;
	if ( permstring[6] == 'r' ) xx |= 0x0004 ;
	if ( permstring[7] == 'w' ) xx |= 0x0002 ;
	if ( permstring[8] == 'x' ) xx |= 0x0001 ;
	* anObject = [ NSNumber numberWithLong: xx ] ;
	return ( YES );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (NSAttributedString *)
attributedStringForObjectValue:	(id)anObject
	withDefaultAttributes:	(NSDictionary *)attributes
{
	return ( nil);
}
@end
