// -----------------------------------------------------------------------------
//  UnitsSizeGridWinCtl.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// realizzazione della finestra per unita'/dimensioni/griglia

#import 	"UnitsSizeGridWinCtl.h"

#import 	"CoverWinCtl.h"
#import 	"CoverView.h"
#import		"AppDelegate.h"
#import		"djZeroUtils.h"

@implementation UnitsSizeGridWinCtl
// ---------------------------------------------------------------------
+ ( id )
sharedGridDialogWinCtl
{
	static	UnitsSizeGridWinCtl	* locWinCtrl  = nil ;

	if ( ! locWinCtrl )
	{
		locWinCtrl = [[ UnitsSizeGridWinCtl allocWithZone: [self zone]] init ] ;
	}
	return ( locWinCtrl );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// inizializzo la finestra caricandola dal file nib
- (id )
init
{
	self = [ self initWithWindowNibName: @"UnitsSizeGrid" ];
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// metto a posto la finestra alla prima visualizzazione
- (void)
windowDidLoad
{
	[super windowDidLoad];
	// aggiorno la finestra in accordo alla coverView
	[ self updateWindow: [AppDelegate getTheDrawView ] ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiornamento dello stato dei controlli sulla finestra
- (void)
setControlsEnabled: (BOOL) status
{
	[ gridColor		setEnabled: status ];
	[ gridSnap		setEnabled: status ];
	[ sliderSpacing setEnabled: status ];
	[ gridSpacing   setEnabled: status ];
	[ showGrid		setEnabled: status ];
	[ selUnits		setEnabled: status ];
	[ heightText	setEnabled: status ];
	[ widthText		setEnabled: status ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiorno la finestra rispettando le variabili della converView
- (void)
updateWindow: (CoverView *) cvrView
{
	float	tmpVal ;
	int		curUnit ;
	if ( cvrView == nil)
	{
		[ self setControlsEnabled: NO ];
		return ;
	}
	// se arrivo qui, c'e' una coverView in fronte a tutti
	// predispongo la finestra della griglia in accordo
	// abilito tutti i controlli
	[ self setControlsEnabled: YES ];
	// imposto i vari valori della griglia
	[ gridColor		setColor: [ cvrView gridLineColor]];
	[ gridSnap		setState:
		([ cvrView snapToGrid] == YES ? NSOnState : NSOffState)];
	[ showGrid		setState:
		([ cvrView showGrid] == YES ? NSOnState : NSOffState)];
	[ selUnits		selectItemAtIndex: ([cvrView curUnits] - UNITS_CM) ];
	// valore minimo e massimo della griglia in unita' correnti
	curUnit = [cvrView curUnits ] ;
	tmpVal = convertMeasure( 5, UNITS_PIXELS, curUnit );
	[ sliderSpacing setMinValue: tmpVal ];
	tmpVal = convertMeasure( 100, UNITS_PIXELS, curUnit );
	[ sliderSpacing setMaxValue: tmpVal ];
	// valore corrente della griglia in unita' correnti
	tmpVal = convertMeasure( [cvrView gridSpacing ], UNITS_PIXELS, curUnit );
	[ sliderSpacing setFloatValue: tmpVal ];
	[ gridSpacing   setFloatValue: tmpVal ];
	// aggiusto le dimensioni della copertina
	tmpVal = convertMeasure( [ cvrView curViewSize].width, 
							UNITS_PIXELS, curUnit );
	[ widthText setFloatValue: tmpVal ];
	tmpVal = convertMeasure( [ cvrView curViewSize].height, 
							UNITS_PIXELS, curUnit );
	[ heightText setFloatValue: tmpVal ];
}

// ---------------------------------------------------------------------
// impostazione dei cambiamenti della finestra alla coverview
- (IBAction)
updateGridDisplay:(id)sender
{
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;
	// non dovrebbe succedere mai...
	if ( cvrView == nil ) return ;
	// aggiusto le variabili della griglia
	[ cvrView setGridLineColor: [gridColor color ]];
	[ cvrView setSnapToGrid: ([gridSnap state] == NSOffState ? NO : YES) ] ;	
	[ cvrView setShowGrid: ([showGrid state] == NSOffState ? NO : YES) ] ;
	// ridisegno la griglia
	[ cvrView setNeedsDisplay:  YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
updateGridSpacing:(id)sender
{
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;
	float		sliderValue = [ sender floatValue ] ;
	float		tmpVal ;
	[ gridSpacing setFloatValue: sliderValue ];
	[ sliderSpacing setFloatValue: sliderValue ];
	// non dovrebbe succedere mai...
	if ( cvrView == nil ) return ;
	// converto sempre e comunque in pixel
	tmpVal = convertMeasure( sliderValue, [cvrView curUnits ], UNITS_PIXELS );
	[ cvrView setGridSpacing: tmpVal ];
	// ridisegno la griglia
	[ cvrView setNeedsDisplay:  YES ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- (IBAction)
updateUnits:(id)sender
{
	int		newU = [[ sender selectedItem ] tag ] ;
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;
	// non dovrebbe succedere mai...
	if ( cvrView == nil ) return ;
	[cvrView setCurUnits: newU ] ;
	// faccio finta che sia cambiata la selezione perche' cosi'
	// sia ggiornano tutte le finestre ausiliarie
	[ [NSNotificationCenter defaultCenter]
		postNotificationName:@"CoverViewChangedSelection"
		object: cvrView ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// imposto le nuove dimensioni della vista
- (IBAction)
updateCoverSize:(id)sender
{
	CoverView   * cvrView = [AppDelegate getTheDrawView ] ;
	float	curW = [ widthText floatValue] ;
	float	curH = [ heightText floatValue] ;
	// non dovrebbe succedere mai...
	if ( cvrView == nil ) return ;
	// converto le dimensioni in pixel
	curW = convertMeasure( curW, [cvrView curUnits ], UNITS_PIXELS ) ;
	curH = convertMeasure( curH, [cvrView curUnits ], UNITS_PIXELS ) ;
	// imposto le dimensioni
	[ cvrView setCurViewSize: NSMakeSize( curW, curH) ];
	// bisogna tenere conto del fattore di zoom corrente
	curW *= [ cvrView curZoomFactor ];
	curH *= [ cvrView curZoomFactor ];
	[ cvrView setFrameSize:  NSMakeSize( curW, curH) ];
	[ cvrView setNeedsDisplay: YES ];
}

@end
