// -----------------------------------------------------------------------------
//  djZeroUtils.m
//  CdCat2
//  Livio Sandel, No Copyright 2004.
//
// utilities di vario genere per il mio uso e consumo

#import		"djZeroUtils.h"

#import		"FileStruct.h"
#import		"LocalFormatters.h"
#import		"UserPrefs.h"

int	lsSortFunction( id el1, id el2, void * ctx );

// ---------------------------------------------------------------------
// di una NSImage, sceglie la rappresentazione piu' piccola,
// poi scala tutto a 16x16
NSImage *
reduceImageToIcon( NSImage * img )
{
	NSImage		* reducedImg ;
	NSSize		smallIconSize ;
	int		minSize, minImgIndex;
	unsigned int	i ;
	NSImageRep 	* imageRep ;
	// recupero la lista delle rappresentazioni
	NSArray		* tmp = [ img representations ] ;
	// potrebbe non esserci l'immagine
	if ( [ tmp count ] == 0 )
		return ( nil) ;
	// per ora la rappresentazione minima e' la prima
	minImgIndex = 0 ;
	imageRep = [ tmp objectAtIndex: minImgIndex ];
	minSize = [ imageRep pixelsHigh ];
	// poi guardo tutte le altre rappresentazioni
	for ( i = 1 ; i < [ tmp count ] ; i ++ )
	{
		int		hh ;
		imageRep = [ tmp objectAtIndex: i ];
		// se la dimensione della rappresentazione corrente...
		hh = [ imageRep pixelsHigh ];
		// ...e' maggiore
		if ( hh < minSize )
		{
			// ho trovato una nuova rappresentazione minima
			// me la segno
			minImgIndex = i ;
			minSize = hh ;
		}
	}
	reducedImg = [ [ [NSImage alloc] init ] autorelease ];
	// arrivato qui, ho trovato al rappresentazione piu' piccola
	[ reducedImg addRepresentation: [ tmp objectAtIndex: minImgIndex ]];
	// dico di scalarla quando sara' ridimensionata
	[ reducedImg setScalesWhenResized: TRUE ];
	// perche' adesso la ridimensiono a 16x16
	smallIconSize.width = smallIconSize.height = 16 ;
	// ecco che la ridimensiono
	[ reducedImg setSize: smallIconSize];
	return (reducedImg);
}

// ---------------------------------------------------------------------
// funzioni per la gestioen dell'ordinamento

int
ascSortFunc( id el1, id el2, void * ctx )
{
	return ( lsSortFunction( el1, el2, ctx ));
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
int
desSortFunc( id el1, id el2, void * ctx )
{
	return ( lsSortFunction( el2, el1, ctx ));
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
int
lsSortFunction( id el1, id el2, void * ctx )
{
	id	s1, s2 ;
	// il confronto sulla dimensione del file (sempre per il solito misterioso
	// motivo i long long non funzionano con valueForKey
	if ( [ (NSString*)ctx isEqual: COLID_FILESIZE ] )
	{
		// ricavo le due dimensioni
		long long	tmp1 = [ (CatFileInfo*)el1 fileSize ] ;
		long long	tmp2 = [ (CatFileInfo*)el2 fileSize ] ;
		// e le confronto brutalmente; notare i valori di ritorno
		if (tmp1 < tmp2)
			return NSOrderedAscending;
		else if (tmp1 > tmp2)
			return NSOrderedDescending;
		return NSOrderedSame;
	}
	// per tutte le altre colonne, uso il polimorfismo (late binding)
	// ricavo i due elementi, non meglio specificando la natura
	// possono essere stringhe, numeri o da te
	s1 = [ (CatFileInfo*)el1 valueForKey: ctx ];
	s2 = [ (CatFileInfo*)el2 valueForKey: ctx ];
	// pero' tutti e tre i tipi rispondono al messaggio compare:
	// e quindi, l'istruzione seguente chiama il metodo appropriato
	return ( [ s1 compare: s2]);
	// con questa versione del codice, il compilatore mi da un po'
	// di warning, proprio su questa istruzione...
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// aggiusta una dimensione per tenere conto di valori negativi
NSRect
arrangeRect ( NSSize tmpSize)
{
	NSPoint		wheretodraw = NSMakePoint( 0, 0) ;
	NSSize		newSize = tmpSize ;
	// se ho una dimensione negativa
	if ( tmpSize.width < 0 )
	{
		// sposto il punto di disegno
		wheretodraw.x = tmpSize.width ;
		// in modo da ottnere una dimensione positiva
		newSize.width = -tmpSize.width ;
	}
	if ( tmpSize.height < 0 )
	{
		wheretodraw.y = tmpSize.height ;
		newSize.height = -tmpSize.height ;
	}
	return ( NSMakeRect( wheretodraw.x, wheretodraw.y, newSize.width, newSize.height) ) ;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// conversione tra unita' di misura
#define		CVT_P2I		72.0F
#define		CVT_C2I		2.54F
#define		CVT_P2C		(CVT_P2I / CVT_C2I)

// una matrice (rappresentata come vettore) per contenere 
// i vari fattori di conversione
static	float	cvtFactors[] = { 
	1.0F,			1.0F/CVT_C2I,	CVT_P2C, 
	CVT_C2I,		1.0F,			CVT_P2I, 
	1.0F/CVT_P2C,	1.0F/CVT_P2I,	1.0F		} ;

float
convertMeasure( float meas, int	fromUnit, int toUnit )
{
	int	rr = fromUnit - UNITS_CM ;
	int	cc = toUnit - UNITS_CM ;
	float	cf ;
	if ( rr < 0 || rr > 2 ) return ( 0 );
	if ( cc < 0 || cc > 2 ) return ( 0 );
	cf = cvtFactors[rr *3 + cc];
	return ( cf * meas );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// una stringa per rappresentare unita' di misura
NSString *
stringForUnits ( int unit )
{
	switch ( unit ) {
	case UNITS_CM :		return( [ NSString stringWithString: @"Centimetres" ] ) ; 
	case UNITS_INCHES :	return( [ NSString stringWithString: @"Inches" ] ) ; 
	case UNITS_PIXELS :	return( [ NSString stringWithString: @"Pixels" ] ) ; 
	default:			return( [ NSString stringWithString: @"unknown" ] ) ;
	}
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// per produrre un identificatore di elementi e variabili
NSString *
keyStr( long oid, NSString * name )
{
	NSString	* str ;
	str = [ NSString stringWithFormat: @"E%2d-%@", oid, name ];
	return ( str );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// un dizionario con il default per font e stile
NSDictionary *
defaultTextAttr( ) 
{
	int						i ;
	NSFont					* locFont  ;
	NSMutableDictionary		* txtAtt ;
	NSMutableParagraphStyle	* parStl ;
	// inizializzo un dizionario epr contenere gli attributi
	txtAtt =  [[[NSMutableDictionary alloc] initWithCapacity:2] autorelease];
	// un font di default come primo attributo
	locFont = [ NSFont fontWithName: @"Georgia" size: 12 ];
	// uno stile di paragrafo come secondo attributo
	parStl = [[NSMutableParagraphStyle alloc] init];
	[ parStl setAlignment: NSCenterTextAlignment ] ;
	[ parStl setLineBreakMode: NSLineBreakByTruncatingTail ];
	// aggiungo gli indicatori di tabulazione
	// per prima cosa li tolgo tutti
	[ parStl setTabStops:[NSArray array] ];	
	// poi ne aggiungo dieci
	for (i = 0; i < 10; i++) 
	{
		// uno ogni mezzo centimetro
		float		tabLoc = (36.0 / 2.54F ) * (i + 1);
		NSTextTab	* tabulatore = [ [NSTextTab alloc] 
				initWithType:NSLeftTabStopType location: tabLoc ]  ; 
		[ parStl addTabStop: tabulatore ];
		[ tabulatore release ] ;
	}
	[ parStl setHeadIndent: (36.0 / 2.54F ) ] ;
	// aggiungo il tutto al dizionario 
	[ txtAtt setObject: locFont forKey:NSFontAttributeName];
	[ txtAtt setObject: parStl forKey:NSParagraphStyleAttributeName];
	return ( txtAtt );    
}

// ---------------------------------------------------------------------
// aggiunge o elimina una colonna di una tablecolumn
// addRem = TRUE, aggiunge, FALSE elimina
void
setupColumn( NSOutlineView *outView, BOOL addRem, NSString * colId )
{
	NSTableColumn	*tmpTC ;

	// vedo se esiste una colonna relativa
	tmpTC = [ outView tableColumnWithIdentifier: colId] ;
	// se non c'e', e devo rimuoverla...
	if ( (tmpTC == nil) && (addRem == FALSE) )
		return ; 	// devo fare nulla
	// se c'e', e devo aggiungerla
	if ( tmpTC != nil && addRem )
		return ;	// devo fare nulla
	// se non c'e', e devo aggiungerla
	if ( tmpTC == nil && addRem )
	{
		// creo la NSTableColumn con l'identificatore
		tmpTC = [ [NSTableColumn alloc] initWithIdentifier: colId ];
		// imposto il titolo della colonna
		[[tmpTC headerCell] setStringValue: NSLocalizedString( colId, colId ) ];
		// modifico le caratteristiche visive della colonna
		configureColumn( tmpTC, colId );
		// aggiungo la colonna
		[outView addTableColumn: tmpTC ];
		return ;	// ho finito
	}
	// se arrivo qui, sicuramente c'e' la colonna e devo rimuoverla
	[outView removeTableColumn: tmpTC ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// modifica una tablecolumn aggiungendo formatter
void
configureColumn( NSTableColumn *tc, NSString * colId )
{
	// i formatter sono creati autorelease perche' passano in gestione alla tablecolumn
	if ( [ colId isEqual: COLID_OSTYPE ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_OSType ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_OSCREATOR ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_Creator ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		FileSizeForm 	*myDF2 = [[[ FileSizeForm alloc ] init ] autorelease ];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_FileSize ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF2 ];
		// gia' che ci sono, sbandiero a destra
		[[tc dataCell   ] setAlignment: NSRightTextAlignment];
		[[tc headerCell ] setAlignment: NSRightTextAlignment];
		return ;
	}
	if ( [ colId isEqual: COLID_POSIXPERM ] )
	{
		FilePosixPerm 	*myDF3 = [[[ FilePosixPerm alloc ] init ] autorelease ];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_PosixPerm ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF3 ];
		return ;
	}
	if ( [ colId isEqual: COLID_MODDATE ] )
	{
		NSDateFormatter *myDF4 = [[NSDateFormatter alloc]
			initWithDateFormat:@"%d %b %Y" allowNaturalLanguage:YES];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_ModDate ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF4 ];
		return ;
	}
	if ( [ colId isEqual: COLID_CREATDATE ] )
	{
		NSDateFormatter *myDF4 = [[NSDateFormatter alloc]
			initWithDateFormat:@"%d %b %Y" allowNaturalLanguage:YES];
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_CreatDate ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
		[[tc dataCell ] setFormatter:  myDF4 ];
		return ;
	}
	if ( [ colId isEqual: COLID_ADD2PRINT ] )
	{
		NSButtonCell		* add2print = nil ;
		add2print = [[[ NSButtonCell alloc] init ] autorelease ];
		[ add2print setButtonType: NSSwitchButton ];
		[ add2print setTitle:@""] ;
		[ tc setWidth: 18 ];
		[ tc setResizable: FALSE ];
		[ [ tc headerCell] setStringValue: @"P?" ];
		[ tc setDataCell:add2print];
	}
	if ( [ colId isEqual: COLID_GROUPNAME ] )
	{
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_GroupName ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
	}
	if ( [ colId isEqual: COLID_OWNERNAME ] )
	{
		int	cw = [ [UserPrefs getSinglePrefValue: keyCDLCW_OwnerName ] intValue ];
		if ( cw <= 0 ) cw = 100 ;
		[ tc setWidth:  cw ];
	}
}

// ---------------------------------------------------------------------
void
printCTM(NSString* ctx )
{
	CGAffineTransform	xxx ;
	CGContextRef myContext = [[NSGraphicsContext currentContext] graphicsPort];
	xxx = CGContextGetCTM( myContext );
	NSLog(@"%@ x11: %8.3f, x12: %8.3f x21: %8.3f x22: %8.3f, tx: %8.3f, ty: %8.3f", 
			ctx, xxx.a, xxx.b, xxx.c, xxx.d, xxx.tx, xxx.ty );
}

void
printRectWithMsg( NSString* ctx,NSRect rect)  
{
	NSLog(@"%@ Rx: %8.3f Ry: %8.3f, w: %8.3f h: %8.3f", 
		ctx, rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
}
